import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

import org.hibernate.Session;

import br.pucrio.olis.domain.AcademicEvent;
import br.pucrio.olis.domain.AcademicEventType;
import br.pucrio.olis.domain.AcademicPreferences;
import br.pucrio.olis.domain.AreaOfInterest;
import br.pucrio.olis.domain.CalendarEvent;
import br.pucrio.olis.domain.Deadline;
import br.pucrio.olis.domain.Frequency;
import br.pucrio.olis.domain.User;

/*
 * Created on Jun 22, 2008
 */

/**
 * @author Ingrid Nunes
 * 
 */
public class AcademicSchemaGenerator extends SchemaGenerator {

	private Map<String, AreaOfInterest> areasOfInterest;

	private void createAreasOfInterest(Session session) throws Exception {
		areasOfInterest = new HashMap<String, AreaOfInterest>();

		AreaOfInterest areaOfInterest = new AreaOfInterest();
		areaOfInterest.setLabel("Multi-agent Systems");
		session.save(areaOfInterest);
		areasOfInterest.put("mas", areaOfInterest);
		areaOfInterest = new AreaOfInterest();
		areaOfInterest.setLabel("Software Engineering");
		session.save(areaOfInterest);
		areasOfInterest.put("se", areaOfInterest);
		areaOfInterest = new AreaOfInterest();
		areaOfInterest.setLabel("Artificial Intelligence");
		session.save(areaOfInterest);
		areasOfInterest.put("ai", areaOfInterest);
		areaOfInterest = new AreaOfInterest();
		areaOfInterest.setLabel("Database");
		session.save(areaOfInterest);
		areasOfInterest.put("db", areaOfInterest);
		areaOfInterest = new AreaOfInterest();
		areaOfInterest.setLabel("Reasoning");
		session.save(areaOfInterest);
		areasOfInterest.put("reasoning", areaOfInterest);
		areaOfInterest = new AreaOfInterest();
		areaOfInterest.setLabel("Learning");
		session.save(areaOfInterest);
		areasOfInterest.put("learning", areaOfInterest);
		areaOfInterest.setLabel("Formal Methods");
		session.save(areaOfInterest);
		areasOfInterest.put("formal", areaOfInterest);
		areaOfInterest.setLabel("Algorithms");
		session.save(areaOfInterest);
		areasOfInterest.put("algorithms", areaOfInterest);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see SchemaGenerator#createEvents(org.hibernate.Session)
	 */
	@Override
	public void createEvents(Session session) throws Exception {
		AcademicEvent event = null;
		CalendarEvent calendarEvent = null;

		event = new AcademicEvent();
		event.setAllDay(true);
		event.setCity(cities.get("sanFrancisco"));
		event.setDescription("Software Engineering and Knowledge Engineering");
		event.setLocation("Sofitel");
		event.setStart(getDate(1, 0, 0));
		event.setStartTime(null);
		event.setEnd(getDate(3, 23, 59));
		event.setEndTime(null);
		event.setSubject("SEKE 2008");
		event.setOwner(users.get("ingrid"));
		event.setEventType(AcademicEventType.CONFERENCE);
		event.addAreasOfInterest(areasOfInterest.get("se"));
		event.addAreasOfInterest(areasOfInterest.get("db"));
		event.putDeadline(Deadline.ACCEPT_REVIEW, dateFormat.parse("01032008"));
		event
				.putDeadline(Deadline.REGISTER_PAPER, dateFormat
						.parse("01032008"));
		event.putDeadline(Deadline.REVIEW_PAPER, dateFormat.parse("01032008"));
		event.putDeadline(Deadline.SUBMIT_CAMERA_READY, dateFormat
				.parse("01032008"));
		event.putDeadline(Deadline.SUBMIT_PAPER, dateFormat.parse("01032008"));
		session.save(event);
		eventsBoard.addEvents(event);

		calendarEvent = new CalendarEvent();
		calendarEvent.setAvailable(false);
		calendarEvent.setEvent(event);
		session.save(calendarEvent);
		getCalendarUserService("ingrid").addEvents(calendarEvent);

		event = new AcademicEvent();
		event.setAllDay(false);
		event.setCity(cities.get("estoril"));
		event.setDescription("Autonomous Agents and Multi-agent Systems");
		event.setEventsBoard(null);
		event.setLocation("Hotel Miragem Cascais");
		event.setStart(getDate(12, 8, 0));
		event.setStartTime(null);
		event.setEnd(getDate(16, 16, 30));
		event.setEndTime(null);
		event.setSubject("AAMAS 2008");
		event.setOwner(users.get("ingrid"));
		event.setEventType(AcademicEventType.CONFERENCE);
		event.addAreasOfInterest(areasOfInterest.get("mas"));
		event.addAreasOfInterest(areasOfInterest.get("ai"));
		event.putDeadline(Deadline.ACCEPT_REVIEW, dateFormat.parse("01072008"));
		event
				.putDeadline(Deadline.REGISTER_PAPER, dateFormat
						.parse("01072008"));
		event.putDeadline(Deadline.REVIEW_PAPER, dateFormat.parse("01072008"));
		event.putDeadline(Deadline.SUBMIT_CAMERA_READY, dateFormat
				.parse("01072008"));
		event.putDeadline(Deadline.SUBMIT_PAPER, dateFormat.parse("01072008"));
		session.save(event);
		eventsBoard.addEvents(event);

		calendarEvent = new CalendarEvent();
		calendarEvent.setAvailable(false);
		calendarEvent.setEvent(event);
		session.save(calendarEvent);
		getCalendarUserService("ingrid").addEvents(calendarEvent);
		getCalendarUserService("andrew").addEvents(calendarEvent);

		event = new AcademicEvent();
		event.setAllDay(true);
		event.setCity(cities.get("caparica"));
		event.setDescription("European Agent Systems Summer School");
		event.setEventsBoard(null);
		event
				.setLocation("Faculdade de Ciencia e Tecnologia/Universidade de Nova Lisboa");
		event.setStart(getDate(5, 0, 0));
		event.setStartTime(null);
		event.setEnd(getDate(9, 23, 59));
		event.setEndTime(null);
		event.setSubject("EASSS 2008");
		event.setOwner(users.get("ingrid"));
		event.setEventType(AcademicEventType.CLASS);
		event.addAreasOfInterest(areasOfInterest.get("mas"));
		event.addAreasOfInterest(areasOfInterest.get("ai"));
		session.save(event);
		eventsBoard.addEvents(event);

		calendarEvent = new CalendarEvent();
		calendarEvent.setAvailable(false);
		calendarEvent.setEvent(event);
		session.save(calendarEvent);
		getCalendarUserService("ingrid").addEvents(calendarEvent);
		getCalendarUserService("andrew").addEvents(calendarEvent);

		event = new AcademicEvent();
		event.setAllDay(false);
		event.setCity(cities.get("rio"));
		event.setDescription("Disciplina de Sistemas Multi-agentes");
		event.setEventsBoard(null);
		event.setLocation("13o andar/Predio FLPF");
		event.setStart(dateFormat.parse("03032008"));
		event.setStartTime(timeFormat.parse("16:00"));
		event.setEnd(dateFormat.parse("30062008"));
		event.setEndTime(timeFormat.parse("19:00"));
		event.setSubject("SMA 2008-1");
		event.setFrequency(Frequency.TUESDAY);
		event.setOwner(users.get("andrew"));
		event.setEventType(AcademicEventType.SEMINAR);
		event.addAreasOfInterest(areasOfInterest.get("se"));
		session.save(event);
		eventsBoard.addEvents(event);

		calendarEvent = new CalendarEvent();
		calendarEvent.setAvailable(false);
		calendarEvent.setEvent(event);
		session.save(calendarEvent);
		getCalendarUserService("ingrid").addEvents(calendarEvent);
		getCalendarUserService("andrew").addEvents(calendarEvent);
		getCalendarUserService("camila").addEvents(calendarEvent);
		getCalendarUserService("elder").addEvents(calendarEvent);

		event = new AcademicEvent();
		event.setAllDay(false);
		event.setCity(cities.get("rio"));
		event.setDescription("Disciplina de Projeto de Sistemas de Software");
		event.setEventsBoard(null);
		event.setLocation("13o andar/Predio FLPF");
		event.setStart(dateFormat.parse("03032008"));
		event.setStartTime(timeFormat.parse("13:00"));
		event.setEnd(dateFormat.parse("30062008"));
		event.setEndTime(timeFormat.parse("16:00"));
		event.setSubject("PSS 2008-1");
		event.setFrequency(Frequency.MONDAY);
		event.setOwner(users.get("ingrid"));
		event.setEventType(AcademicEventType.CLASS);
		event.addAreasOfInterest(areasOfInterest.get("mas"));
		session.save(event);
		eventsBoard.addEvents(event);

		calendarEvent = new CalendarEvent();
		calendarEvent.setAvailable(false);
		calendarEvent.setEvent(event);
		session.save(calendarEvent);
		getCalendarUserService("ingrid").addEvents(calendarEvent);
		getCalendarUserService("andrew").addEvents(calendarEvent);
		getCalendarUserService("camila").addEvents(calendarEvent);
		getCalendarUserService("elder").addEvents(calendarEvent);

		event = new AcademicEvent();
		event.setAllDay(false);
		event.setCity(cities.get("rio"));
		event
				.setDescription("Seminario do LES no qual alunos de graduacao e pos-graduacao mostram seus trabalhos correntes.");
		event.setEventsBoard(null);
		event.setLocation("13o andar/Predio FLPF");
		event.setStart(dateFormat.parse("01012008"));
		event.setStartTime(timeFormat.parse("13:00"));
		event.setEnd(dateFormat.parse("31072008"));
		event.setEndTime(timeFormat.parse("15:00"));
		event.setSubject("Seminario do LES");
		event.setFrequency(Frequency.FRIDAY);
		event.setOwner(users.get("elder"));
		session.save(event);

		calendarEvent = new CalendarEvent();
		calendarEvent.setAvailable(true);
		calendarEvent.setEvent(event);
		session.save(calendarEvent);
		getCalendarUserService("ingrid").addEvents(calendarEvent);
		getCalendarUserService("elder").addEvents(calendarEvent);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see SchemaGenerator#createSpecificEntities(org.hibernate.Session)
	 */
	@Override
	public void createSpecificEntities(Session session) throws Exception {
		createAreasOfInterest(session);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see SchemaGenerator#createUserPreferences(org.hibernate.Session,
	 *      br.pucrio.olis.domain.User)
	 */
	@Override
	public void createUserPreferences(Session session, User user)
			throws Exception {
		Random random = new Random(System.currentTimeMillis());
		AcademicPreferences academicPreferences = new AcademicPreferences();
		int number = random.nextInt(2);
		academicPreferences.setForeignEvents(number == 0 ? Boolean.FALSE
				: Boolean.TRUE);

		List<AreaOfInterest> aof = new ArrayList<AreaOfInterest>(
				areasOfInterest.values());
		List<AcademicEventType> et = Arrays.asList(AcademicEventType.values());
		for (int i = 0; i < 3; i++) {
			academicPreferences.addAreasOfInterest(aof.get(random.nextInt(aof
					.size())));
			academicPreferences
					.addEventTypes(et.get(random.nextInt(et.size())));
		}

		session.save(academicPreferences);
		user.setPreferences(academicPreferences);
	}

}
