/*
 * Created on Apr 27, 2008
 */
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.AnnotationConfiguration;
import org.hibernate.cfg.Configuration;
import org.hibernate.tool.hbm2ddl.SchemaExport;

import br.pucrio.olis.domain.CalendarUserService;
import br.pucrio.olis.domain.City;
import br.pucrio.olis.domain.Country;
import br.pucrio.olis.domain.EventsBoard;
import br.pucrio.olis.domain.User;
import br.pucrio.olis.domain.UserService;
import br.pucrio.olis.domain.UserServiceType;

/**
 * <p>
 * This class generates the database schema according to the hibernate
 * configuration file. It also populates the database with the initial data.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public abstract class SchemaGenerator {

	public enum SchemaType {
		ACADEMIC_EVENT, GENERIC_EVENT, TRAVEL_EVENT;
	}

	private static final SchemaType SCHEMA_TYPE = SchemaType.TRAVEL_EVENT;

	/**
	 * <p>
	 * Method executed when the SchemaGenerator application is runs. It creates
	 * the database schema and insert the initial data in the database.
	 * </p>
	 * 
	 * @param args
	 *            the application input arguments.
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {
		SchemaGenerator schemaGenerator = null;
		switch (SCHEMA_TYPE) {
		case ACADEMIC_EVENT:
			schemaGenerator = new AcademicSchemaGenerator();
			break;
		case TRAVEL_EVENT:
			schemaGenerator = new TravelSchemaGenerator();
			break;
		default:
			schemaGenerator = new GenericSchemaGenerator();
			break;
		}
		schemaGenerator.createSchema();
		schemaGenerator.run();
	}

	protected Map<String, City> cities;
	protected Configuration configuration;
	protected Map<String, Country> countries;
	protected DateFormat dateFormat = new SimpleDateFormat("ddMMyyyy");
	protected DateFormat dateTimeFormat = new SimpleDateFormat("ddMMyyyy HH:mm");
	protected EventsBoard eventsBoard;
	protected SessionFactory sessionFactory;
	protected DateFormat timeFormat = new SimpleDateFormat("HH:mm");
	protected Map<String, User> users;

	/**
	 * <p>
	 * Create a new instance of SchemaGenerator. It initializes the hibernate
	 * configuration and also build a session factory.
	 * </p>
	 */
	public SchemaGenerator() {
		configuration = new AnnotationConfiguration().configure();
		sessionFactory = configuration.buildSessionFactory();
	}

	protected void createCities(Session session) throws Exception {
		cities = new HashMap<String, City>();

		City city = new City();
		city.setName("Porto Alegre");
		city.setCountry(countries.get("br"));
		session.save(city);
		cities.put("poa", city);

		city = new City();
		city.setName("Juiz de Fora");
		city.setCountry(countries.get("br"));
		session.save(city);
		cities.put("juizFora", city);

		city = new City();
		city.setName("Alagoas");
		city.setCountry(countries.get("br"));
		session.save(city);
		cities.put("alagoas", city);

		city = new City();
		city.setName("Rio de Janeiro");
		city.setCountry(countries.get("br"));
		session.save(city);
		cities.put("rio", city);

		city = new City();
		city.setName("San Diego");
		city.setCountry(countries.get("usa"));
		session.save(city);
		cities.put("sanDiego", city);

		city = new City();
		city.setName("San Francisco");
		city.setCountry(countries.get("usa"));
		session.save(city);
		cities.put("sanFrancisco", city);

		city = new City();
		city.setName("Estoril");
		city.setCountry(countries.get("pt"));
		session.save(city);
		cities.put("estoril", city);

		city = new City();
		city.setName("Lisboa");
		city.setCountry(countries.get("pt"));
		session.save(city);
		cities.put("lisboa", city);

		city = new City();
		city.setName("Costa da Caparica");
		city.setCountry(countries.get("pt"));
		session.save(city);
		cities.put("caparica", city);

		city = new City();
		city.setName("Stuttgart");
		city.setCountry(countries.get("de"));
		session.save(city);
		cities.put("stuttgart", city);

		city = new City();
		city.setName("Munchen");
		city.setCountry(countries.get("de"));
		session.save(city);
		cities.put("munchen", city);

		city = new City();
		city.setName("Frankfurt");
		city.setCountry(countries.get("de"));
		session.save(city);
		cities.put("frankfurt", city);

		city = new City();
		city.setName("Berlin");
		city.setCountry(countries.get("de"));
		session.save(city);
		cities.put("berlin", city);
	}

	protected void createCountries(Session session) throws Exception {
		countries = new HashMap<String, Country>();

		Country country = new Country();
		country.setCode("BR");
		country.setName("Brazil");
		session.save(country);
		countries.put("br", country);

		country = new Country();
		country.setCode("US");
		country.setName("United States");
		session.save(country);
		countries.put("usa", country);

		country = new Country();
		country.setCode("PT");
		country.setName("Portugal");
		session.save(country);
		countries.put("pt", country);

		country = new Country();
		country.setCode("DE");
		country.setName("Germany");
		session.save(country);
		countries.put("de", country);
	}

	public abstract void createEvents(Session session) throws Exception;

	/**
	 * <p>
	 * Create the database schema according to the hibernate configuration.
	 * </p>
	 */
	public void createSchema() {
		try {
			SchemaExport schema = new SchemaExport(configuration);
			Class.forName(configuration.getProperty("connection.driver_class"));
			DriverManager.getConnection(configuration
					.getProperty("connection.url"), configuration
					.getProperty("connection.username"), configuration
					.getProperty("connection.password"));

			schema.create(true, true);

			Class.forName(configuration.getProperty("connection.driver_class"));
			Connection conn = DriverManager.getConnection(configuration
					.getProperty("connection.url"), configuration
					.getProperty("connection.username"), configuration
					.getProperty("connection.password"));

			Statement s = conn.createStatement();
			s
					.executeUpdate("ALTER TABLE calendaruserservice_calendarevent DROP FOREIGN KEY FK251EED09FCD69244");
			s
					.executeUpdate("ALTER TABLE calendaruserservice_calendarevent ADD CONSTRAINT FK251EED09FCD69244 FOREIGN KEY FK251EED09FCD69244 (events_id) REFERENCES calendarevent (id) ON DELETE CASCADE");
			s
					.executeUpdate("ALTER TABLE calendarevent DROP FOREIGN KEY FKA3350DDC9B9FD0C1");
			s
					.executeUpdate("ALTER TABLE calendarevent ADD CONSTRAINT FKA3350DDC9B9FD0C1 FOREIGN KEY FKA3350DDC9B9FD0C1 (event_id) REFERENCES event (id) ON DELETE CASCADE");
		} catch (Exception exc) {
			System.err.println(exc);
		}
	}

	protected void createServices(Session session, User user) {
		Iterator<UserServiceType> it = UserServiceType.getServices();
		while (it.hasNext()) {
			UserServiceType userServiceType = it.next();
			UserService service = userServiceType.createUserService(user);
			session.save(service);
			user.addServices(service);
		}
	}

	public abstract void createSpecificEntities(Session session)
			throws Exception;

	public abstract void createUserPreferences(Session session, User user)
			throws Exception;

	protected void createUsers(Session session) throws Exception {
		users = new HashMap<String, User>();

		User user = new User();
		user.setFirstName("Ingrid");
		user.setLastName("Nunes");
		user.setEmail("ingridnunes@gmx.net");
		user.setUsername("ingrid");
		user.setPassword("123");
		user.setDateOfBirth(dateFormat.parse("22011984"));
		user.setCity(cities.get("poa"));
		session.save(user);
		createServices(session, user);
		createUserPreferences(session, user);
		users.put("ingrid", user);

		user = new User();
		user.setFirstName("Daltro");
		user.setLastName("Nunes");
		user.setEmail("ingridnunes@gmail.com");
		user.setUsername("daltro");
		user.setPassword("123");
		user.setDateOfBirth(dateFormat.parse("28111943"));
		user.setCity(cities.get("poa"));
		session.save(user);
		createServices(session, user);
		createUserPreferences(session, user);
		users.put("daltro", user);

		user = new User();
		user.setFirstName("Suzana");
		user.setLastName("Nunes");
		user.setEmail("ingridnunes@gmail.com");
		user.setUsername("suzana");
		user.setPassword("123");
		user.setDateOfBirth(dateFormat.parse("02031948"));
		user.setCity(cities.get("poa"));
		session.save(user);
		createServices(session, user);
		createUserPreferences(session, user);
		users.put("suzana", user);

		user = new User();
		user.setFirstName("Camila");
		user.setLastName("Nunes");
		user.setEmail("ingridnunes@gmail.com");
		user.setUsername("camila");
		user.setPassword("123");
		user.setDateOfBirth(dateFormat.parse("23021985"));
		user.setCity(cities.get("alagoas"));
		session.save(user);
		createServices(session, user);
		createUserPreferences(session, user);
		users.put("camila", user);

		user = new User();
		user.setFirstName("Elder");
		user.setLastName("Cirilo");
		user.setEmail("ingridnunes@gmail.com");
		user.setUsername("elder");
		user.setPassword("123");
		user.setDateOfBirth(dateFormat.parse("23021985"));
		user.setCity(cities.get("juizFora"));
		session.save(user);
		createServices(session, user);
		createUserPreferences(session, user);
		users.put("elder", user);

		user = new User();
		user.setFirstName("Andrew");
		user.setLastName("Diniz");
		user.setEmail("ingridnunes@gmail.com");
		user.setUsername("andrew");
		user.setPassword("123");
		user.setDateOfBirth(dateFormat.parse("26041984"));
		user.setCity(cities.get("rio"));
		session.save(user);
		createServices(session, user);
		createUserPreferences(session, user);
		users.put("andrew", user);
	}

	protected CalendarUserService getCalendarUserService(String user) {
		return (CalendarUserService) users.get(user).getService(
				CalendarUserService.class);
	}

	public Date getDate(int day, int hour, int min) {
		Calendar calendar = GregorianCalendar.getInstance();
		calendar.setTime(new Date());
		calendar.set(Calendar.DAY_OF_MONTH, day);
		calendar.set(Calendar.HOUR_OF_DAY, hour);
		calendar.set(Calendar.MINUTE, min);
		calendar.set(Calendar.SECOND, 0);
		calendar.set(Calendar.MILLISECOND, 0);
		return calendar.getTime();
	}

	/**
	 * <p>
	 * Populate the database with the initial data. All data is inserted under a
	 * transaction.
	 * </p>
	 * 
	 * @throws Exception
	 */
	public synchronized void run() throws Exception {
		Session session = sessionFactory.getCurrentSession();
		session.beginTransaction().begin();

		/** ******* MANDATORY ********* */
		eventsBoard = new EventsBoard();
		session.save(eventsBoard);

		createCountries(session);
		createCities(session);

		createSpecificEntities(session);
		/** ******* MANDATORY ********* */

		createUsers(session);
		createEvents(session);

		session.getTransaction().commit();
	}

}
