/*
 * Created on 19/06/2008
 */
package br.pucrio.olis.agent.agentslayerfacade;

import java.io.Serializable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

/**
 * @author ingridnunes
 * @version 1.0
 * 
 */
public class AgentResponse implements Future<Object>, Serializable {

	public static final int INFINITE = -1;

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 7242507160124877995L;

	private boolean done;
	private boolean error;
	private int performative;
	private AgentRequest request;
	private Object ret;

	public AgentResponse(AgentRequest request) {
		this.request = request;
		this.done = false;
		this.ret = null;
		this.error = false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.util.concurrent.Future#cancel(boolean)
	 */
	@Override
	public boolean cancel(boolean mayInterruptIfRunning) {
		throw new RuntimeException("Not implemented yet!");
	}

	public synchronized void error() {
		if (!done) {
			this.done = true;
			this.error = true;
			notifyAll();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.util.concurrent.Future#get()
	 */
	@Override
	public Object get() throws InterruptedException, ExecutionException {
		try {
			return get(INFINITE, null);
		} catch (TimeoutException e) {
			throw new RuntimeException(e);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.util.concurrent.Future#get(long, java.util.concurrent.TimeUnit)
	 */
	@Override
	public synchronized Object get(long timeout, TimeUnit unit)
			throws InterruptedException, ExecutionException, TimeoutException {
		long start = System.currentTimeMillis();
		while (!done) {
			if (timeout != INFINITE
					&& (System.currentTimeMillis() - start) > unit
							.toMillis(timeout))
				throw new TimeoutException();
			try {
				if (unit == null) {
					wait();
				} else {
					wait(unit.toMillis(timeout));
				}
			} catch (Exception exc) {
			}
		}
		return ret;
	}

	public int getPerformative() {
		return performative;
	}

	public AgentRequest getRequest() {
		return request;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.util.concurrent.Future#isCancelled()
	 */
	@Override
	public boolean isCancelled() {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.util.concurrent.Future#isDone()
	 */
	@Override
	public synchronized boolean isDone() {
		return done;
	}

	public boolean isError() {
		return error;
	}

	public synchronized void set(int performative, Object ret) {
		if (!done) {
			this.performative = performative;
			this.ret = ret;
			this.done = true;
			notifyAll();
		}
	}

	public synchronized void set(Object ret) {
		set(0, ret);
	}

}
