/*
 * Created on 04/06/2008
 */
package br.pucrio.olis.agent.agentslayerfacade;

import jade.content.lang.sl.SLCodec;
import jade.core.Agent;

import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * <p>
 * This class represents the agent that composes the agents layer facade. It
 * receives requests from the business layer to get information from the agents.
 * </p>
 * 
 * @author ingridnunes
 * @version 1.0
 * 
 */
public class FacadeAgent extends Agent {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -5780233245952568410L;
	private static FacadeAgent singleton;

	public static synchronized FacadeAgent getFacadeAgent() {
		if (singleton == null) {
			throw new NullPointerException("Facade Agent not initialized!");
		}
		return singleton;
	}

	private String cidBase;
	private int cidCnt;
	private Log log;

	public String genCID() {
		if (cidBase == null) {
			cidBase = getLocalName() + hashCode() + System.currentTimeMillis()
					% 10000 + "_";
		}
		return cidBase + (cidCnt++);
	}

	public Object invoke(AgentRequest request) throws AgentInvocationException {
		return invoke(request, AgentResponse.INFINITE, null);
	}

	public Object invoke(AgentRequest request, long timeOut, TimeUnit timeUnit)
			throws AgentInvocationException {
		try {
			AgentResponse response = new AgentResponse(request);
			addBehaviour(new RequestBehavior(this, response));
			Object ret = response.get(timeOut, timeUnit);
			if (response.isError()) {
				throw new AgentInvocationException();
			}
			return ret;
		} catch (Exception e) {
			throw new AgentInvocationException(e);
		}
	}

	public AgentResponse invokeAsync(AgentRequest request)
			throws AgentInvocationException {
		AgentResponse response = new AgentResponse(request);
		addBehaviour(new RequestBehavior(this, response));
		return response;
	}

	public Future<Object> invokeAsync(AgentRequest request,
			final AsyncHandler handler) throws AgentInvocationException {
		final AgentResponse response = new AgentResponse(request);
		addBehaviour(new RequestBehavior(this, response));
		new Thread(new Runnable() {
			public void run() {
				try {
					while (!response.isDone()) {
						try {
							wait();
						} catch (Exception e) {
							log.warn(e);
						}
					}
					handler.handleResponse(response);
				} catch (Exception e) {
					log.error(e);
					e.printStackTrace();
				}
			}
		}).start();
		return response;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jade.core.Agent#setup()
	 */
	@Override
	protected synchronized void setup() {
		this.log = LogFactory.getLog(this.getClass());
		this.cidBase = getLocalName() + hashCode() + System.currentTimeMillis()
				% 10000 + "_";
		this.cidCnt = 0;
		getContentManager().registerLanguage(new SLCodec());
		if (singleton != null) {
			throw new IllegalStateException("Facade instance already up!");
		}
		FacadeAgent.singleton = this;
		super.setup();
		log.info("Facade Agent is up!");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jade.core.Agent#takeDown()
	 */
	@Override
	protected synchronized void takeDown() {
		FacadeAgent.singleton = null;
		super.takeDown();
		log.info("Facade Agent is prepared to die!");
	}

}
