/*
 * Created on Jun 9, 2008
 */
package br.pucrio.olis.agent.agentslayerfacade;

import jade.content.ContentElement;
import jade.content.abs.AbsContentElement;
import jade.core.behaviours.Behaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jadex.adapter.fipa.SFipa;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.pucrio.olis.ontology.OLISOntology;

/**
 * @author Ingrid Nunes
 * 
 */
public class RequestBehavior extends Behaviour {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 409827831921188232L;

	private Log log;
	private MessageTemplate mt; // The template to receive replies
	private final AgentResponse response;
	private int step;

	/**
	 * Creates a new instance of RequestBehavior.
	 * 
	 * @param agent
	 * @param operation
	 */
	public RequestBehavior(final FacadeAgent agent, final AgentResponse response) {
		super(agent);
		this.log = LogFactory.getLog(this.getClass());
		this.response = response;
		this.step = 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jade.core.behaviours.Behaviour#action()
	 */
	@Override
	public void action() {
		switch (step) {
		case 0:
			AgentRequest request = response.getRequest();
			String cid = ((FacadeAgent) myAgent).genCID();
			ACLMessage msg = new ACLMessage(request.getPerformative());
			msg.addReceiver(request.getAgent());
			msg.setConversationId(cid); // Unique
			msg.setReplyWith("request" + System.currentTimeMillis());
			msg.setOntology(request.getOntology());
			msg.setProtocol(request.getProtocol());
			msg.setLanguage(SFipa.FIPA_SL);

			try {
				if (request.getContent() instanceof AbsContentElement) {
					myAgent.getContentManager().fillContent(msg,
							(AbsContentElement) request.getContent());
				} else if (request.getContent() instanceof ContentElement) {
					myAgent.getContentManager().fillContent(msg,
							(ContentElement) request.getContent());
				} else {
					response.error();
					return;
				}
			} catch (Exception e) {
				log.error(e);
				e.printStackTrace();
				response.error();
				return;
			}

			myAgent.send(msg);
			// Prepare the template to get proposals
			mt = MessageTemplate.and(MessageTemplate.MatchConversationId(cid),
					MessageTemplate.MatchInReplyTo(msg.getReplyWith()));
			step = 1;
			break;
		case 1:
			// Receive answer
			ACLMessage reply = myAgent.receive(mt);
			if (reply != null) {
				// Reply received
				Object ret = null;
				if (reply.getContent() != null
						&& !"".equals(reply.getContent())) {
					try {
						if (response.getRequest().getContent() instanceof AbsContentElement) {
							ret = myAgent.getContentManager()
									.extractAbsContent(reply);
						} else if (response.getRequest().getContent() instanceof ContentElement) {
							ret = myAgent.getContentManager().extractContent(
									reply);
						}
					} catch (Exception e) {
						log.error(e);
						e.printStackTrace();
						response.error();
						return;
					}
				}
				response.set(reply.getPerformative(), ret);
				step = 2;
			} else {
				block();
			}
			break;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jade.core.behaviours.Behaviour#done()
	 */
	@Override
	public boolean done() {
		return response.isDone();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jade.core.behaviours.Behaviour#onStart()
	 */
	@Override
	public void onStart() {
		myAgent.getContentManager()
				.registerOntology(OLISOntology.getInstance());
		super.onStart();
	}

}
