/*
 * Created on 04/06/2008
 */
package br.pucrio.olis.agent.businesslayermonitor;

import jade.content.lang.sl.SLCodec;
import jade.core.AID;
import jade.core.Agent;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.domain.FIPANames.ContentLanguage;

import java.util.HashSet;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.pucrio.olis.business.BusinessOperation;
import br.pucrio.olis.business.ServiceLocator;
import br.pucrio.olis.business.observer.BusinessServiceObserver;
import br.pucrio.olis.business.observer.ObservableBusinessService;
import br.pucrio.olis.ontology.OLISOntology;

/**
 * <p>
 * This class represents the agent that composes the business layer monitor. It
 * receives notifications about the execution in the the business layer, and
 * propagates them to the other agents.
 * </p>
 * 
 * @author ingridnunes
 * @version 1.0
 * 
 */
public class EnvironmentAgent extends Agent implements BusinessServiceObserver {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 8719967874369360136L;

	private Log log;
	private Set<AID> agents;

	/*
	 * (non-Javadoc)
	 * 
	 * @see jade.core.Agent#setup()
	 */
	@Override
	protected void setup() {
		this.log = LogFactory.getLog(this.getClass());
		this.agents = new HashSet<AID>();

		ServiceLocator serviceLocator = ServiceLocator.getInstance();
		String[] services = serviceLocator.getServiceNames();

		for (String serviceName : services) {
			ObservableBusinessService service = (ObservableBusinessService) serviceLocator
					.getService(serviceName);
			service.addObserver(this);
		}

		// Register the service.
		DFAgentDescription dfd = new DFAgentDescription();
		dfd.setName(this.getAID());
		ServiceDescription sd = new ServiceDescription();
		sd.setType("environment");
		sd.setName(getLocalName());
		sd.addLanguages(ContentLanguage.FIPA_SL);
		sd.addOntologies(OLISOntology.ONTOLOGY_NAME);
		dfd.addServices(sd);

		try {
			DFService.register(this, dfd);
		} catch (FIPAException fe) {
			log.error(getLocalName()
					+ " registration with DF unsucceeded. Reason: "
					+ fe.getMessage());
			log.error(fe);
			fe.printStackTrace();
		}

		addBehaviour(new RegisterAgentBehavior(this));
		getContentManager().registerLanguage(new SLCodec());
		getContentManager().registerOntology(OLISOntology.getInstance());
		super.setup();
		log.info("Environment Agent is up!");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jade.core.Agent#takeDown()
	 */
	@Override
	protected void takeDown() {
		this.agents = null;

		ServiceLocator serviceLocator = ServiceLocator.getInstance();
		String[] services = serviceLocator.getServiceNames();

		for (String serviceName : services) {
			ObservableBusinessService service = (ObservableBusinessService) serviceLocator
					.getService(serviceName);
			service.deleteObserver(this);
		}

		try {
			DFService.deregister(this);
		} catch (FIPAException fe) {
			log.error(fe);
			fe.printStackTrace();
		}

		super.takeDown();

		log.info("Environment Agent is prepared to die!");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.observer.BusinessServiceObserver#operationPerformed(br.pucrio.olis.business.observer.ObservableBusinessService,
	 *      br.pucrio.olis.business.observer.BusinessOperation)
	 */
	@Override
	public void operationPerformed(ObservableBusinessService observable,
			BusinessOperation operation) {
		log.info("Business Operation performed... adding behavior");
		addBehaviour(new PropagateBusinessOperationBehavior(this, operation));
	}

	public Set<AID> getAgents() {
		return agents;
	}

}
