/*
 * Created on Jun 9, 2008
 */
package br.pucrio.olis.agent.businesslayermonitor;

import jade.core.AID;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import jadex.adapter.fipa.SFipa;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.pucrio.olis.business.BusinessOperation;
import br.pucrio.olis.ontology.OLISOntology;

/**
 * <p>
 * This class represents the behavior that propagates a business operation to
 * the other agents.
 * </p>
 * 
 * @author ingridnunes
 * @version 1.0
 * 
 */
public class PropagateBusinessOperationBehavior extends OneShotBehaviour {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 1141782364884981861L;

	private Log log;
	private BusinessOperation operation;

	/**
	 * Creates a new instance of PropagateBusinessOperationBehavior.
	 * 
	 * @param agent
	 * @param operation
	 */
	public PropagateBusinessOperationBehavior(final EnvironmentAgent agent,
			final BusinessOperation operation) {
		super(agent);
		this.log = LogFactory.getLog(this.getClass());
		this.operation = operation;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jade.core.behaviours.Behaviour#action()
	 */
	@Override
	public void action() {
		try {
			ACLMessage msg = new ACLMessage(ACLMessage.INFORM);
			for (AID agentId : ((EnvironmentAgent) myAgent).getAgents()) {
				msg.addReceiver(agentId);
			}
			msg.setLanguage(SFipa.FIPA_SL);
			msg.setOntology(OLISOntology.ONTOLOGY_NAME);
			myAgent.getContentManager().fillContent(msg, operation);
			myAgent.send(msg);
		} catch (Exception e) {
			log.error("Couldn't send business operation message.");
			log.error(e);
			e.printStackTrace();
		}
	}

}
