/*
 * Created on Jun 10, 2008
 */
package br.pucrio.olis.agent.businesslayermonitor;

import jade.core.behaviours.DataStore;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.lang.acl.MessageTemplate.MatchExpression;
import jade.proto.states.MsgReceiver;
import jadex.adapter.fipa.SFipa;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * <p>
 * This class represents the behavior that receives a registering message from
 * an agent and adds it to the set of agents that will receive business
 * operation notifications.
 * </p>
 * 
 * @author ingridnunes
 * @version 1.0
 * 
 */
public class RegisterAgentBehavior extends MsgReceiver {

	public static final Object MSG_KEY = "msgs";

	/**
	 * Determines if a de-serialized file is compatible with this class.
	 */
	private static final long serialVersionUID = 6031714374691140821L;

	private Log log;

	/**
	 * Creates a new instance of RegisterAgentBehavior.
	 * 
	 * @param a
	 */
	public RegisterAgentBehavior(EnvironmentAgent agent) {
		super(agent, new MessageTemplate(new MatchExpression() {
			private static final long serialVersionUID = -6254722003825383822L;

			public boolean match(ACLMessage arg0) {
				if (!(arg0.getProtocol().equals(SFipa.PROTOCOL_SUBSCRIBE)))
					return false;
				if (!(arg0.getPerformative() == ACLMessage.SUBSCRIBE))
					return false;
				return true;
			}
		}), INFINITE, new DataStore(), MSG_KEY);
		this.log = LogFactory.getLog(this.getClass());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jade.proto.states.MsgReceiver#done()
	 */
	@Override
	public boolean done() {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jade.proto.states.MsgReceiver#handleMessage(jade.lang.acl.ACLMessage)
	 */
	@Override
	protected void handleMessage(ACLMessage msg) {
		try {
			EnvironmentAgent env = (EnvironmentAgent) myAgent;
			ACLMessage reply = msg.createReply();
			if (env.getAgents().contains(msg.getSender())) {
				reply.setPerformative(ACLMessage.REFUSE);
			} else {
				reply.setPerformative(ACLMessage.AGREE);
				env.getAgents().add(msg.getSender());
			}
			env.send(reply);
		} catch (Exception exc) {
			log.error(exc);
			exc.printStackTrace();
		}
	}

}
