/*
 * Created on 13/06/2008
 */
package br.pucrio.olis.agent.eventremider;

import jadex.runtime.ICondition;
import jadex.runtime.IGoal;
import jadex.runtime.Plan;

import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.mail.EmailException;
import org.apache.commons.mail.SimpleEmail;

import br.pucrio.olis.domain.CalendarEvent;
import br.pucrio.olis.domain.Frequency;
import br.pucrio.olis.domain.User;

/**
 * <p>
 * This class represents the plan to remind an user about an event.
 * </p>
 * 
 * @author ingridnunes
 * @version 1.0
 * 
 */
public class RemindEventPlan extends Plan {

	public static final String SMTP_FROM_NAME = "OLIS";
	public static final String SMTP_FROM_ADDRESS = "ingrid_nunes@yahoo.com.br";
	public static final String SMTP_USERNAME = "ingrid_nunes";
	public static final String SMTP_PASSWORD = "ingrid123";
	public static final String SMTP_HOSTNAME = "smtp.mail.yahoo.com.br";
	public static final int SMTP_PORT = 25;
	public static final boolean SMTP_AUTHENTICATE = true;

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -6667022565345536180L;

	private Log log = LogFactory.getLog(this.getClass());

	/*
	 * (non-Javadoc)
	 * 
	 * @see jadex.runtime.Plan#body()
	 */
	@Override
	public void body() {
		User user = (User) getBeliefbase().getBelief("myself").getFact();
		CalendarEvent event = (CalendarEvent) getParameter("event").getValue();
		SimpleDateFormat sdf = new SimpleDateFormat(user.getDateFormat()
				.getFormat()
				+ " " + user.getTimeFormat().getFormat());
		SimpleDateFormat sdf2 = new SimpleDateFormat(user.getTimeFormat()
				.getFormat());
		Date nextTime = event.getEvent().getNextTime();

		// Wait for the alarm time
		log.info(event.getEvent().getSubject() + " at " + nextTime + " ("
				+ user.getFullName() + ")");
		Long timeBefore = (Long) getBeliefbase().getBelief("timeBefore")
				.getFact();
		if (timeBefore == null) {
			timeBefore = new Long(0);
		} else {
			timeBefore = new Long(timeBefore.longValue());
		}
		getBeliefbase().getBelief("alarmtime").setFact(
				nextTime.getTime() - timeBefore.longValue());
		ICondition condition = getCondition("alarmtime_reached");
		waitForCondition(condition);

		try {
			SimpleEmail email = new SimpleEmail();
			email.setHostName(SMTP_HOSTNAME);
			if (SMTP_AUTHENTICATE) {
				email.setAuthentication(SMTP_USERNAME, SMTP_PASSWORD);
			}
			email.setSmtpPort(SMTP_PORT);
			email.setFrom(SMTP_FROM_ADDRESS, SMTP_FROM_NAME);
			email.addTo(user.getEmail(), user.getFullName());

			email.setSubject(sdf.format(nextTime) + " - "
					+ event.getEvent().getSubject());
			StringBuffer sb = new StringBuffer();
			sb.append("Subject: ").append(event.getEvent().getSubject())
					.append("\n");
			sb.append("Description: ")
					.append(event.getEvent().getDescription()).append("\n");
			sb.append("Time: ").append(sdf.format(nextTime)).append(" - ");
			if (Frequency.ONCE.equals(event.getEvent().getFrequency())) {
				sb.append(sdf.format(event.getEvent().getEnd()));
			} else {
				sb.append(sdf2.format(event.getEvent().getEndTime()));
			}
			sb.append("\n");
			sb.append("Location: ").append(event.getEvent().getLocation())
					.append("(").append(event.getEvent().getCity().getName())
					.append("-").append(
							event.getEvent().getCity().getCountry().getCode())
					.append(")").append("\n");
			email.setMsg(sb.toString());
			log.info("Sending remind event at " + new Date() + " ("
					+ event.getEvent().getSubject() + " - "
					+ user.getFullName() + ")");
			email.send();
			log.info("Email sent!");
		} catch (EmailException e) {
			log.error(e);
			e.printStackTrace();
		}

		IGoal schedule = createGoal("schedule_next_event");
		dispatchTopLevelGoal(schedule);
	}
}
