/*
 * Created on Jun 21, 2008
 */
package br.pucrio.olis.agent.eventsuggestion.travelevent;

import jadex.runtime.IGoal;
import jadex.runtime.Plan;

import java.util.HashMap;
import java.util.Map;

import br.pucrio.olis.agent.eventsuggestion.travelevent.weather.AmusementParkRateCalculator;
import br.pucrio.olis.agent.eventsuggestion.travelevent.weather.BeachRateCalculator;
import br.pucrio.olis.agent.eventsuggestion.travelevent.weather.CityRateCalculator;
import br.pucrio.olis.agent.eventsuggestion.travelevent.weather.MountainRateCalculator;
import br.pucrio.olis.agent.eventsuggestion.travelevent.weather.SkyMountainCalculator;
import br.pucrio.olis.agent.eventsuggestion.travelevent.weather.WeatherRateCalculator;
import br.pucrio.olis.domain.TravelEvent;
import br.pucrio.olis.domain.weather.DayForecast;

/**
 * @author Ingrid Nunes
 * 
 */
public class WeatherGoodPlan extends Plan {

	private static final Map<String, WeatherRateCalculator> rateCalculators;

	static {
		rateCalculators = new HashMap<String, WeatherRateCalculator>();
		rateCalculators.put("Beach", new BeachRateCalculator());
		rateCalculators.put("Mountain", new MountainRateCalculator());
		rateCalculators.put("City", new CityRateCalculator());
		rateCalculators
				.put("Amusement Park", new AmusementParkRateCalculator());
		rateCalculators.put("Sky Montain", new SkyMountainCalculator());

	}

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -3799138352713764420L;

	/*
	 * (non-Javadoc)
	 * 
	 * @see jadex.runtime.Plan#body()
	 */
	@Override
	public void body() {
		TravelEvent event = (TravelEvent) getParameter("event").getValue();

		IGoal forecast = createGoal("get_forecast");
		forecast.getParameter("date").setValue(event.getStart());
		forecast.getParameter("city").setValue(event.getCity());
		dispatchSubgoalAndWait(forecast);
		DayForecast dayForecast = (DayForecast) forecast.getParameter("result")
				.getValue();

		double result = -1;
		WeatherRateCalculator weatherRateCalculator = rateCalculators.get(event
				.getPlaceType().getLabel());
		if (dayForecast != null && weatherRateCalculator != null) {
			result = weatherRateCalculator.calculateRate(dayForecast);
		}
		getParameter("result").setValue(result);
	}

}
