/*
 * Created on Jun 21, 2008
 */
package br.pucrio.olis.agent.eventsuggestion.travelevent.weather;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.pucrio.olis.agent.eventsuggestion.travelevent.Rain;
import br.pucrio.olis.agent.eventsuggestion.travelevent.Temperature;
import br.pucrio.olis.agent.eventsuggestion.travelevent.Wind;
import br.pucrio.olis.domain.weather.DayForecast;

/**
 * @author Ingrid Nunes
 * 
 */
public abstract class WeatherRateCalculator {

	public enum Classification {
		VERY_BAD(0), BAD(0.2), NOT_SO_BAD(0.36), OK(0.52), NOT_SO_GOOD(0.68), GOOD(
				0.84), VERY_GOOD(1);

		public final double rate;

		private Classification(double rate) {
			this.rate = rate;
		}
	}

	protected Log log = LogFactory.getLog(this.getClass());

	public double calculateRate(DayForecast dayForecast) {
		int count = 0;
		double sum = 0;

		if (dayForecast.getLowTemperature() != null
				|| dayForecast.getHighTemperature() != null) {
			Temperature temperature = new Temperature(dayForecast
					.getLowTemperature(), dayForecast.getHighTemperature());
			Classification temperatureClass = getClassification(temperature);
			log.debug("Temperature: " + temperature.getClassification() + " / "
					+ temperatureClass);

			count++;
			sum += temperatureClass.rate;
		}

		if (dayForecast.getDay().getChanceOfPrecipitation() != null
				|| dayForecast.getNight().getChanceOfPrecipitation() != null) {
			double mean = 0;
			if (dayForecast.getDay().getChanceOfPrecipitation() == null)
				mean = dayForecast.getNight().getChanceOfPrecipitation();
			else if (dayForecast.getNight().getChanceOfPrecipitation() == null)
				mean = dayForecast.getDay().getChanceOfPrecipitation();
			else
				mean = ((double) (dayForecast.getDay()
						.getChanceOfPrecipitation() + dayForecast.getNight()
						.getChanceOfPrecipitation())) / 2;
			Rain rain = new Rain(mean);
			Classification rainClass = getClassification(rain);
			log.debug("Rain: " + rain.getClassification() + " / " + rainClass);

			count++;
			sum += rainClass.rate;
		}

		if (dayForecast.getDay().getWind().getSpeed() != null
				|| dayForecast.getNight().getWind().getSpeed() != null) {
			double mean = 0;
			if (dayForecast.getDay().getWind().getSpeed() == null)
				mean = dayForecast.getNight().getWind().getSpeed();
			else if (dayForecast.getNight().getWind().getSpeed() == null)
				mean = dayForecast.getDay().getWind().getSpeed();
			else
				mean = ((double) (dayForecast.getDay().getWind().getSpeed() + dayForecast
						.getNight().getWind().getSpeed())) / 2;
			Wind wind = new Wind(mean);
			Classification windClass = getClassification(wind);
			log.debug("Wind: " + wind.getClassification() + " / " + windClass);

			count++;
			sum += windClass.rate;
		}

		double rate = -1;
		if (count > 0) {
			rate = sum / count;
		}
		log.debug("Rate: " + rate);
		return rate;
	}

	protected abstract Classification getClassification(Rain rain);

	protected abstract Classification getClassification(Temperature temperature);

	protected abstract Classification getClassification(Wind wind);

}
