/*
 * Created on Jun 10, 2008
 */
package br.pucrio.olis.agent.manager;

import jadex.planlib.StartAgentInfo;
import jadex.runtime.IGoal;
import jadex.runtime.Plan;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.pucrio.olis.business.ServiceLocator;
import br.pucrio.olis.business.service.UserService;
import br.pucrio.olis.domain.User;

/**
 * <p>
 * Plan for starting agents and remembering agent identifiers.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class StarterPlan extends Plan {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 6244093195209401930L;

	private Log log = LogFactory.getLog(this.getClass());

	/**
	 * The body method is called on the instantiated plan instance from the
	 * scheduler.
	 */
	@SuppressWarnings("unchecked")
	public void body() {
		try {
			IGoal start = createGoal("start_agents");

			UserService userService = (UserService) ServiceLocator
					.getInstance().getService(UserService.SERVICE_NAME);
			List<User> users = userService.loadAll();

			for (User user : users) {
				Map args = new HashMap<String, Object>();
				args.put("myId", user.getId());
				StartAgentInfo startAgentInfo = new StartAgentInfo(
						"br.pucrio.olis.agent.user.User", "user_"
								+ user.getId(), 0, args);
				start.getParameterSet("agentinfos").addValue(startAgentInfo);
			}

			log.info("Dispatching subgoal to start agents");
			dispatchSubgoalAndWait(start);
			getBeliefbase().getBeliefSet("startedagents").addFacts(
					start.getParameterSet("agentidentifiers").getValues());
		} catch (Exception e) {
			log.error(e);
			e.printStackTrace();
		}
	}

}
