/*
 * Created on Jun 22, 2008
 */
package br.pucrio.olis.agent.scheduler;

import jadex.adapter.fipa.AgentIdentifier;
import jadex.adapter.fipa.SFipa;
import jadex.runtime.IGoal;
import jadex.runtime.IMessageEvent;
import jadex.runtime.Plan;

import java.util.Date;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.ServiceLocator;
import br.pucrio.olis.business.service.UserService;
import br.pucrio.olis.domain.CalendarEvent;
import br.pucrio.olis.domain.EventSchedulerNotification;
import br.pucrio.olis.domain.User;

/**
 * @author Ingrid Nunes
 * 
 */
public class RescheduleEventPlan extends Plan {

	private static final long serialVersionUID = 3347023306541898112L;

	private Log log = LogFactory.getLog(this.getClass());

	/*
	 * (non-Javadoc)
	 * 
	 * @see jadex.runtime.Plan#body()
	 */
	@Override
	public void body() {
		CalendarEvent event = (CalendarEvent) getParameter("event").getValue();
		User problem = (User) getParameter("user").getValue();

		Long duration = calculateDurantion(event);
		Date alternativeTime = event.getEvent().getStart();
		boolean ok = false;
		while (!ok) {
			alternativeTime = queryAlternativeTime(alternativeTime, duration,
					problem);
			Date end = calculateEnd(alternativeTime, duration);
			problem = checkTimeOK(alternativeTime, end, problem, event
					.getParticipants());
			if (problem == null) {
				ok = true;
			}
		}

		try {
			User user = (User) getBeliefbase().getBelief("myself").getFact();
			EventSchedulerNotification eventSchedulerNotification = new EventSchedulerNotification();
			eventSchedulerNotification.setCalendarEvent(event);
			eventSchedulerNotification.setConflict((User) getParameter("user")
					.getValue());
			eventSchedulerNotification.setSuggestedStart(alternativeTime);
			eventSchedulerNotification.setSuggestedEnd(calculateEnd(
					alternativeTime, duration));
			UserService service = (UserService) ServiceLocator.getInstance()
					.getService(UserService.SERVICE_NAME);
			service.addNotification(user, eventSchedulerNotification);
			log.info("Notification added.");
		} catch (BusinessException e) {
			log.error(e);
			e.printStackTrace();
		}
	}

	private Long calculateDurantion(CalendarEvent event) {
		return event.getEvent().getEnd().getTime()
				- event.getEvent().getStart().getTime();
	}

	private Date calculateEnd(Date start, Long duration) {
		return new Date(start.getTime() + duration);
	}

	private User checkTimeOK(Date start, Date end, User problem,
			List<User> participants) {
		if (participants.isEmpty())
			return null;

		IGoal timeOK = createGoal("check_time_ok");
		timeOK.getParameter("start").setValue(start);
		timeOK.getParameter("end").setValue(end);
		for (User participant : participants) {
			// The user who proposed the time do not have to be checked
			if (problem.equals(participant))
				continue;
			timeOK.getParameterSet("participants").addValue(participant);
		}
		dispatchSubgoalAndWait(timeOK);

		Boolean result = (Boolean) timeOK.getParameter("result").getValue();
		if (!result) {
			return (User) timeOK.getParameter("user").getValue();
		}

		return null;
	}

	private Date queryAlternativeTime(Date start, Long duration, User user) {
		IMessageEvent msg = createMessageEvent("query_alternative_time");
		msg.getParameterSet(SFipa.RECEIVERS).addValue(
				new AgentIdentifier("user_" + user.getId(), true));
		AlternativeTimeQuery alternativeTimeQuery = new AlternativeTimeQuery();
		alternativeTimeQuery.setStart(start);
		alternativeTimeQuery.setDuration(duration);
		msg.setContent(alternativeTimeQuery);
		getWaitqueue().addReply(msg);
		sendMessage(msg);

		IMessageEvent reply = waitForReply(msg);
		AlternativeTimeAnswer alternativeTimeAnswer = (AlternativeTimeAnswer) reply
				.getContent();
		return alternativeTimeAnswer.getAlternativeTime();
	}

}
