/*
 * Created on Jun 15, 2008
 */
package br.pucrio.olis.agent.user;

import jadex.runtime.Plan;
import jadex.runtime.PlanFailureException;

import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.ServiceLocator;
import br.pucrio.olis.business.service.CalendarService;
import br.pucrio.olis.domain.CalendarEvent;
import br.pucrio.olis.domain.Frequency;
import br.pucrio.olis.domain.User;

/**
 * @author Ingrid Nunes
 * 
 */
public class CheckAvailabilityPlan extends Plan {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 2209237584618215103L;

	private Log log = LogFactory.getLog(this.getClass());

	/*
	 * (non-Javadoc)
	 * 
	 * @see jadex.runtime.Plan#body()
	 */
	@Override
	public void body() {
		User user = (User) getBeliefbase().getBelief("myself").getFact();
		Date start = (Date) getParameter("start").getValue();
		Date end = (Date) getParameter("end").getValue();
		getParameter("result").setValue(Boolean.TRUE);

		try {
			CalendarService calendarService = (CalendarService) ServiceLocator
					.getInstance().getService(CalendarService.SERVICE_NAME);
			Map<Frequency, List<CalendarEvent>> result = calendarService
					.loadByPeriod(start, end, user);

			for (Frequency frequency : Frequency.values()) {
				List<CalendarEvent> events = result.get(frequency);
				CalendarEvent conflict = null;

				if (Frequency.ONCE.equals(frequency)) {
					for (CalendarEvent event : events) {
						if (!event.getAvailable()) {
							conflict = event;
							break;
						}
					}
				} else {
					for (CalendarEvent event : events) {
						if (event.getAvailable()) {
							continue;
						}
						Date next = event.getEvent().getNextTime(start);
						if (next != null && next.before(end)) {
							conflict = event;
							break;
						}
						Date prev = event.getEvent().getPreviousTime(end);
						if (prev != null && prev.after(start)) {
							conflict = event;
							break;
						}
					}
				}

				if (conflict != null) {
					log.debug("Conflict with: " + start + " " + end);
					log.debug("* Event: " + conflict.getEvent().getSubject());
					log.debug("* Frequency: "
							+ conflict.getEvent().getFrequency());
					log.debug("* Date: " + conflict.getEvent().getStart() + "-"
							+ conflict.getEvent().getEnd());
					log.debug("* Time: " + conflict.getEvent().getStartTime()
							+ "-" + conflict.getEvent().getEndTime());
					getParameter("result").setValue(Boolean.FALSE);
					return;
				}
			}
		} catch (BusinessException ex) {
			ex.printStackTrace();
			throw new PlanFailureException(ex.getMessage());
		}

		getParameter("result").setValue(Boolean.TRUE);
	}

}
