/*
 * Created on 17/06/2008
 */
package br.pucrio.olis.agent.util;

import jade.content.ContentElement;
import jade.content.ContentManager;
import jade.content.abs.AbsContentElement;
import jade.content.lang.sl.SLCodec;
import jade.domain.FIPANames.ContentLanguage;
import jade.lang.acl.ACLMessage;
import jadex.runtime.ContentException;
import jadex.runtime.IContentCodec;

import java.io.Serializable;
import java.util.Properties;

import br.pucrio.olis.ontology.OLISOntology;

public class OLISFipaSLCodec implements IContentCodec, Serializable {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 3585362874324257571L;

	protected ContentManager contentManager;

	/**
	 * Create a OLISFipaSL content codec.
	 */
	public OLISFipaSLCodec() {
		this.contentManager = new ContentManager();
		contentManager.registerLanguage(new SLCodec());
		contentManager.registerOntology(OLISOntology.getInstance());
		//contentManager.setValidationMode(false);
	}

	private ACLMessage createACLMessage() {
		ACLMessage msg = new ACLMessage(ACLMessage.INFORM);
		msg.setLanguage(ContentLanguage.FIPA_SL);
		msg.setOntology(OLISOntology.ONTOLOGY_NAME);
		return msg;
	}

	/**
	 * Decode data with the codec.
	 * 
	 * @param val
	 *            The string value.
	 * @return The encoded object.
	 */
	public synchronized Object decode(String val) {
		try {
			ACLMessage msg = createACLMessage();
			msg.setContent(val);
			return contentManager.extractContent(msg);
		} catch (Exception e) {
			e.printStackTrace();
			throw new ContentException(e.getMessage(), e);
		}
	}

	/**
	 * Encode data with the codec.
	 * 
	 * @param val
	 *            The value.
	 * @return The encoded object.
	 */
	public synchronized String encode(Object val) {
		try {
			ACLMessage msg = createACLMessage();
			if (val instanceof AbsContentElement) {
				contentManager.fillContent(msg, (AbsContentElement) val);
			} else if (val instanceof ContentElement) {
				contentManager.fillContent(msg, (ContentElement) val);
			}
			return msg.getContent();
		} catch (Exception e) {
			e.printStackTrace();
			throw new ContentException(e.getMessage(), e);
		}
	}

	/**
	 * Test if the codec can be used with the provided meta information.
	 * 
	 * @param props
	 *            The meta information.
	 * @return True, if it can be used.
	 */
	public boolean match(Properties props) {
		return ContentLanguage.FIPA_SL.equals(props.getProperty("language"))
				&& OLISOntology.ONTOLOGY_NAME.equals(props
						.getProperty("ontology"));
	}

}
