/*
 * Created on Jun 10, 2008
 */
package br.pucrio.olis.agent.util;

import jadex.adapter.fipa.AgentIdentifier;
import jadex.adapter.fipa.SFipa;
import jadex.runtime.IGoal;
import jadex.runtime.IMessageEvent;
import jadex.runtime.Plan;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * <p>
 * This class represents the plan that sends a message to the environment agent
 * to register the agent to receive business notifications in the future.
 * </p>
 * 
 * @author ingridnunes
 * @version 1.0
 * 
 */
public class RegisterEnvironmentPlan extends Plan {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 5881617670809736929L;

	private Log log = LogFactory.getLog(this.getClass());

	/*
	 * (non-Javadoc)
	 * 
	 * @see jadex.runtime.Plan#body()
	 */
	@Override
	public void body() {
		IGoal search = createGoal("search_environment");
		dispatchSubgoalAndWait(search);

		// Search for environment.
		AgentIdentifier environment = (AgentIdentifier) getBeliefbase()
				.getBelief("environment").getFact();

		IMessageEvent msg = createMessageEvent("sub_subscribe");
		msg.getParameterSet(SFipa.RECEIVERS).addValue(environment);
		getWaitqueue().addReply(msg);
		sendMessage(msg);

		IMessageEvent reply = waitForReply(msg);
		if (SFipa.AGREE.equals(reply.getParameter(SFipa.PERFORMATIVE)
				.getValue())) {
			log.info("Agent " + getAgentName()
					+ " succesfully registered in the environment.");
		} else if (SFipa.REFUSE.equals(reply.getParameter(SFipa.PERFORMATIVE)
				.getValue())) {
			log.info("Agent " + getAgentName()
					+ " unsuccesfully registered in the environment.");
		}
	}

}
