/*
 * Created on 10/06/2008
 */
package br.pucrio.olis.business;

import org.springframework.context.ApplicationContext;

import br.pucrio.olis.business.observer.ObservableBusinessService;

/**
 * @author ingridnunes
 * @version 1.0
 * 
 */
public class ServiceLocator {

	private static ServiceLocator singleton;

	/**
	 * Get the unique instance of this class.
	 * 
	 * @return The unique instance of this class.
	 */
	public static ServiceLocator getInstance() {
		if (singleton == null) {
			throw new NullPointerException("ServiceLocator not initialized!");
		}
		return singleton;
	}

	private final ApplicationContext context;

	/**
	 * Creates a new instance of ServiceLocator.
	 */
	public ServiceLocator(ApplicationContext context) {
		if (singleton != null) {
			throw new IllegalStateException(
					"ServiceLocator already initialized!");
		}
		ServiceLocator.singleton = this;
		this.context = context;
	}

	/**
	 * Get a instance of the specified service.
	 * 
	 * @return An instance of the service.
	 * 
	 * @param name
	 *            the service name
	 */
	public Object getService(final String name) {
		return context.getBean(name);
	}

	/**
	 * Get the name of the services beans.
	 * 
	 * @return the array containing the service names.
	 */
	public String[] getServiceNames() {
		return context.getBeanNamesForType(ObservableBusinessService.class);
	}

}
