/*
 * Created on May 24, 2008
 */
package br.pucrio.olis.business.service;

import java.util.List;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.observer.ObservableBusinessService;
import br.pucrio.olis.domain.AcademicPreferences;
import br.pucrio.olis.domain.AreaOfInterest;
import br.pucrio.olis.domain.User;

/**
 * <p>
 * This interface defines the services provided by the Academic Event Service.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public interface AcademicEventService extends ObservableBusinessService {

	/**
	 * The String indicates the service name.
	 */
	public static final String SERVICE_NAME = AcademicEventService.class
			.getSimpleName();

	/**
	 * <p>
	 * Load all areas of interest.
	 * </p>
	 * 
	 * @return the list containing all areas of interest.
	 * @throws BusinessException
	 */
	public List<AreaOfInterest> loadAllAreasOfInterest()
			throws BusinessException;

	/**
	 * <p>
	 * Update user preferences.
	 * </p>
	 * 
	 * @param academicPreferences
	 *            the user academic preferences.
	 * @return the update user.
	 * @throws BusinessException
	 */
	public User updatePreferences(AcademicPreferences academicPreferences)
			throws BusinessException;

}
