/*
 * Created on May 22, 2008
 */
package br.pucrio.olis.business.service;

import java.util.List;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.observer.ObservableBusinessService;
import br.pucrio.olis.domain.Event;
import br.pucrio.olis.domain.EventsAnnouncementUserService;
import br.pucrio.olis.domain.User;

/**
 * <p>
 * This interface defines the services provided by the Events Announcement
 * Service.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public interface EventsAnnouncementService extends ObservableBusinessService {

	/**
	 * The String indicates the service name.
	 */
	public static final String SERVICE_NAME = EventsAnnouncementService.class
			.getSimpleName();

	/**
	 * <p>
	 * Delete an event in the database.
	 * </p>
	 * 
	 * @param event
	 *            the event to be delete.
	 * @throws BusinessException
	 */
	public void delete(Event event) throws BusinessException;

	/**
	 * <p>
	 * Retrieve an event from the database according to its id.
	 * </p>
	 * 
	 * @param id
	 *            the event identifier.
	 * @return the event.
	 * @throws BusinessException
	 */
	public Event getEvent(Long id) throws BusinessException;

	/**
	 * <p>
	 * Load a list of events the occurs in a specific month/year.
	 * </p>
	 * 
	 * @param month
	 *            the month that the events occur.
	 * @param year
	 *            the year that the events occur.
	 * @return the list of events.
	 * @throws BusinessException
	 */
	public List<Event> loadEventsInMonth(Integer month, Integer year)
			throws BusinessException;

	/**
	 * <p>
	 * Load a list of periodic events the occurs in a specific month/year.
	 * Periodic events are the ones whose frequency is not Frequency.ONCE.
	 * </p>
	 * 
	 * @param month
	 *            the month that the events occur.
	 * @param year
	 *            the year that the events occur.
	 * @return the list of events.
	 * @throws BusinessException
	 */
	public List<Event> loadPeriodicEventsInMonth(Integer month, Integer year)
			throws BusinessException;

	/**
	 * <p>
	 * Save an event in the database.
	 * </p>
	 * 
	 * @param event
	 *            the event to be saved.
	 * @return the event saved.
	 * @throws BusinessException
	 */
	public Event save(Event event) throws BusinessException;

	/**
	 * <p>
	 * Update an event in the database.
	 * </p>
	 * 
	 * @param event
	 *            the event to be updated.
	 * @return the event updated.
	 * @throws BusinessException
	 */
	public Event update(Event event) throws BusinessException;

	/**
	 * <p>
	 * Update a events user announcement service in the database.
	 * </p>
	 * 
	 * @param eventsAnnouncementUserService
	 *            the events announcement service to be updated.
	 * @return the user whose events announcement service was updated.
	 * @throws BusinessException
	 */
	public User update(
			EventsAnnouncementUserService eventsAnnouncementUserService)
			throws BusinessException;

}
