/*
 * Created on May 25, 2008
 */
package br.pucrio.olis.business.service;

import java.util.List;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.observer.ObservableBusinessService;
import br.pucrio.olis.domain.ActivityType;
import br.pucrio.olis.domain.PlaceType;
import br.pucrio.olis.domain.TravelPreferences;
import br.pucrio.olis.domain.User;

/**
 * <p>
 * This interface defines the services provided by the Travel Event Service.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public interface TravelEventService extends ObservableBusinessService {

	/**
	 * The String indicates the service name.
	 */
	public static final String SERVICE_NAME = TravelEventService.class
			.getSimpleName();

	/**
	 * <p>
	 * Load all activity types.
	 * </p>
	 * 
	 * @return the list containing all activity types.
	 * @throws BusinessException
	 */
	public List<ActivityType> loadAllActivityTypes() throws BusinessException;

	/**
	 * <p>
	 * Load all place types.
	 * </p>
	 * 
	 * @return the list containing all place types.
	 * @throws BusinessException
	 */
	public List<PlaceType> loadAllPlaceTypes() throws BusinessException;

	/**
	 * <p>
	 * Update user preferences.
	 * </p>
	 * 
	 * @param travelPreferences
	 *            the user travel preferences.
	 * @return the updated user.
	 * @throws BusinessException
	 */
	public User updatePreferences(TravelPreferences travelPreferences)
			throws BusinessException;

}
