/*
 * Created on Apr 4, 2008
 */
package br.pucrio.olis.business.service;

import java.util.List;
import java.util.Set;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.observer.ObservableBusinessService;
import br.pucrio.olis.domain.Notification;
import br.pucrio.olis.domain.User;

/**
 * <p>
 * This interface defines the services provided by the User Service.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public interface UserService extends ObservableBusinessService {

	/**
	 * The String indicates the service name.
	 */
	public static final String SERVICE_NAME = UserService.class.getSimpleName();

	/**
	 * <p>
	 * Add a user notification.
	 * </p>
	 * 
	 * @param user
	 * @param notification
	 * @throws BusinessException
	 */
	public void addNotification(User user, Notification notification)
			throws BusinessException;

	/**
	 * <p>
	 * Delete the user notification.
	 * </p>
	 * 
	 * @param user
	 * @param notification
	 * @throws BusinessException
	 */
	public void deleteNotification(User user, Notification notification)
			throws BusinessException;

	/**
	 * <p>
	 * Load all the users from the database.
	 * </p>
	 * 
	 * @return the list containg all the users.
	 * @throws BusinessException
	 */
	public List<User> loadAll() throws BusinessException;

	/**
	 * <p>
	 * Load all user notifications.
	 * </p>
	 * 
	 * @param user
	 * @return the user notifications.
	 * @throws BusinessException
	 */
	public Set<Notification> loadNotifications(User user)
			throws BusinessException;

	/**
	 * <p>
	 * Load user by him/her id.
	 * </p>
	 * 
	 * @return the user.
	 * @throws BusinessException
	 */
	public User loadUserById(Long id) throws BusinessException;

	/**
	 * <p>
	 * Load all users.
	 * </p>
	 * 
	 * @return the List containing all the users.
	 * @throws BusinessException
	 */
	public List<User> loadUsers() throws BusinessException;

	/**
	 * <p>
	 * Login a user. Look an user by his/her user name and, if he/she was found,
	 * check if the provided password is correct.
	 * </p>
	 * 
	 * @param username
	 *            the username of the user.
	 * @param password
	 *            the password of the user.
	 * @return the user or null if no user with the username provided is found.
	 * @throws BusinessException
	 */
	public User login(String username, String password)
			throws BusinessException;

	/**
	 * <p>
	 * Save an user in the database.
	 * </p>
	 * 
	 * @param user
	 *            the user to be saved.
	 * @return the user saved.
	 * @throws BusinessException
	 */
	public User save(User user) throws BusinessException;

	/**
	 * <p>
	 * Update an user in the database.
	 * </p>
	 * 
	 * @param user
	 *            the user to be updated.
	 * @return the user updated.
	 * @throws BusinessException
	 */
	public User update(User user) throws BusinessException;

	/**
	 * <p>
	 * Update a user advanced options in the database.
	 * </p>
	 * 
	 * @param user
	 *            the user to be updated.
	 * @return the user updated.
	 * @throws BusinessException
	 */
	public User updateAdvancedOptions(User user) throws BusinessException;

}
