/*
 * Created on May 24, 2008
 */
package br.pucrio.olis.business.service.impl;

import java.util.List;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.observer.ObservableBusinessServiceImpl;
import br.pucrio.olis.business.service.AcademicEventService;
import br.pucrio.olis.data.dao.AcademicPreferencesDAO;
import br.pucrio.olis.data.dao.AreaOfInterestDAO;
import br.pucrio.olis.data.dao.UserDAO;
import br.pucrio.olis.domain.AcademicPreferences;
import br.pucrio.olis.domain.AreaOfInterest;
import br.pucrio.olis.domain.User;

/**
 * <p>
 * This class implements the services provided by the Academic Event Service.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class AcademicEventServiceImpl extends ObservableBusinessServiceImpl
		implements AcademicEventService {

	private AcademicPreferencesDAO academicPreferencesDAO;
	private AreaOfInterestDAO areaOfInterestDAO;
	private UserDAO userDAO;

	/**
	 * <p>
	 * Create an instance of CalendarServiceImpl.
	 * </p>
	 * 
	 * @param areaOfInterestDAO
	 *            the area of interest DAO.
	 * @param academicPreferencesDAO
	 *            the academic preferences DAO.
	 * @param userDAO
	 *            the user DAO.
	 */
	public AcademicEventServiceImpl(AreaOfInterestDAO areaOfInterestDAO,
			AcademicPreferencesDAO academicPreferencesDAO, UserDAO userDAO) {
		this.areaOfInterestDAO = areaOfInterestDAO;
		this.academicPreferencesDAO = academicPreferencesDAO;
		this.userDAO = userDAO;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.AcademicEventService#loadAllAreasOfInterest()
	 */
	@Override
	public List<AreaOfInterest> loadAllAreasOfInterest()
			throws BusinessException {
		return areaOfInterestDAO.loadAll();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.AcademicEventService#updatePreferences(br.pucrio.olis.domain.AcademicPreferences)
	 */
	@Override
	public User updatePreferences(AcademicPreferences academicPreferences)
			throws BusinessException {
		academicPreferencesDAO.update(academicPreferences);
		return userDAO.get(academicPreferences.getUser().getId());
	}

}
