/*
 * Created on May 22, 2008
 */
package br.pucrio.olis.business.service.impl;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.BusinessOperation;
import br.pucrio.olis.business.BusinessOperationType;
import br.pucrio.olis.business.observer.ObservableBusinessServiceImpl;
import br.pucrio.olis.business.service.EventsAnnouncementService;
import br.pucrio.olis.data.dao.EventDAO;
import br.pucrio.olis.data.dao.EventsAnnouncementUserServiceDAO;
import br.pucrio.olis.data.dao.EventsBoardDAO;
import br.pucrio.olis.domain.Event;
import br.pucrio.olis.domain.EventsAnnouncementUserService;
import br.pucrio.olis.domain.User;
import br.pucrio.olis.util.OlisUtils;

/**
 * <p>
 * This class implements the services provided by the Events Announcement
 * Service.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class EventsAnnouncementServiceImpl extends
		ObservableBusinessServiceImpl implements EventsAnnouncementService {

	private EventDAO eventDAO;
	private EventsAnnouncementUserServiceDAO eventsAnnouncementUserServiceDAO;
	private EventsBoardDAO eventsBoardDAO;

	public EventsAnnouncementServiceImpl(
			EventsAnnouncementUserServiceDAO eventsAnnouncementUserServiceDAO,
			EventsBoardDAO eventsBoardDAO, EventDAO eventDAO) {
		this.eventsAnnouncementUserServiceDAO = eventsAnnouncementUserServiceDAO;
		this.eventDAO = eventDAO;
		this.eventsBoardDAO = eventsBoardDAO;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.EventsAnnouncementService#delete(br.pucrio.olis.domain.Event)
	 */
	@Override
	public void delete(Event event) throws BusinessException {
		eventDAO.delete(event);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.EventsAnnouncementService#getEvent(java.lang.Long)
	 */
	@Override
	public Event getEvent(Long id) throws BusinessException {
		return eventDAO.get(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.EventsAnnouncementService#loadEventsInMonth(java.lang.Integer,
	 *      java.lang.Integer)
	 */
	@Override
	public List<Event> loadEventsInMonth(Integer month, Integer year)
			throws BusinessException {
		Date start = OlisUtils.getMonthBegin(month, year);
		Date end = OlisUtils.getMonthEnd(month, year);
		return eventsAnnouncementUserServiceDAO.loadEventsBetweenDates(start,
				end);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.EventsAnnouncementService#loadPeriodicEventsInMonth(java.lang.Integer,
	 *      java.lang.Integer)
	 */
	@Override
	public List<Event> loadPeriodicEventsInMonth(Integer month, Integer year)
			throws BusinessException {
		Date start = OlisUtils.getMonthBegin(month, year);
		Date end = OlisUtils.getMonthEnd(month, year);
		return eventsAnnouncementUserServiceDAO.loadPeriodicEventBetweenDates(
				start, end);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.EventsAnnouncementService#save(br.pucrio.olis.domain.Event)
	 */
	@Override
	public Event save(Event event) throws BusinessException {
		event.checkBusinessRules();
		Serializable id = eventDAO.save(event);
		Event e = eventDAO.get(id);
		eventsBoardDAO.addEvent(e);

		BusinessOperation businessOperation = new BusinessOperation();
		businessOperation.setOperationType(BusinessOperationType.CREATE);
		businessOperation.setParam(eventDAO.get(id));
		notifyObservers(businessOperation);

		return e;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.EventsAnnouncementService#update(br.pucrio.olis.domain.Event)
	 */
	@Override
	public Event update(Event event) throws BusinessException {
		event.checkBusinessRules();
		eventDAO.update(event);
		return eventDAO.get(event.getId());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.EventsAnnouncementService#update(br.pucrio.olis.domain.EventsAnnouncementUserService)
	 */
	@Override
	public User update(
			EventsAnnouncementUserService eventsAnnouncementUserService)
			throws BusinessException {
		eventsAnnouncementUserServiceDAO.update(eventsAnnouncementUserService);
		return eventsAnnouncementUserServiceDAO.get(
				eventsAnnouncementUserService.getId()).getUser();
	}

}
