/*
 * Created on May 25, 2008
 */
package br.pucrio.olis.business.service.impl;

import java.util.List;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.observer.ObservableBusinessServiceImpl;
import br.pucrio.olis.business.service.TravelEventService;
import br.pucrio.olis.data.dao.ActivityTypeDAO;
import br.pucrio.olis.data.dao.PlaceTypeDAO;
import br.pucrio.olis.data.dao.TravelPreferencesDAO;
import br.pucrio.olis.data.dao.UserDAO;
import br.pucrio.olis.domain.ActivityType;
import br.pucrio.olis.domain.PlaceType;
import br.pucrio.olis.domain.TravelPreferences;
import br.pucrio.olis.domain.User;

/**
 * @author Ingrid Nunes
 * 
 */
public class TravelEventServiceImpl extends ObservableBusinessServiceImpl
		implements TravelEventService {

	private ActivityTypeDAO activityTypeDAO;
	private PlaceTypeDAO placeTypeDAO;
	private TravelPreferencesDAO travelPreferencesDAO;
	private UserDAO userDAO;

	/**
	 * <p>
	 * Create an instance of TravelEventServiceImpl.
	 * </p>
	 * 
	 * @param activityTypeDAO
	 *            the activity type DAO.
	 * @param placeTypeDAO
	 *            the place type DAO.
	 * @param travelPreferencesDAO
	 *            the travel preferences DAO.
	 * @param userDAO
	 *            the user DAO.
	 */
	public TravelEventServiceImpl(ActivityTypeDAO activityTypeDAO,
			PlaceTypeDAO placeTypeDAO,
			TravelPreferencesDAO travelPreferencesDAO, UserDAO userDAO) {
		this.activityTypeDAO = activityTypeDAO;
		this.placeTypeDAO = placeTypeDAO;
		this.travelPreferencesDAO = travelPreferencesDAO;
		this.userDAO = userDAO;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.TravelEventService#loadAllActivityTypes()
	 */
	@Override
	public List<ActivityType> loadAllActivityTypes() throws BusinessException {
		return activityTypeDAO.loadAll();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.TravelEventService#loadAllPlaceTypes()
	 */
	@Override
	public List<PlaceType> loadAllPlaceTypes() throws BusinessException {
		return placeTypeDAO.loadAll();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.TravelEventService#updatePreferences(br.pucrio.olis.domain.TravelPreferences)
	 */
	@Override
	public User updatePreferences(TravelPreferences travelPreferences)
			throws BusinessException {
		travelPreferencesDAO.update(travelPreferences);
		return userDAO.get(travelPreferences.getUser().getId());
	}

}
