/*
 * Created on 20/06/2008
 */
package br.pucrio.olis.business.service.impl;

import jade.core.AID;
import jade.lang.acl.ACLMessage;
import br.pucrio.olis.agent.agentslayerfacade.AgentInvocationException;
import br.pucrio.olis.agent.agentslayerfacade.AgentRequest;
import br.pucrio.olis.agent.agentslayerfacade.FacadeAgent;
import br.pucrio.olis.agent.weather.WeatherRequest;
import br.pucrio.olis.agent.weather.WeatherResponse;
import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.observer.ObservableBusinessServiceImpl;
import br.pucrio.olis.business.service.WeatherService;
import br.pucrio.olis.data.dao.CityDAO;
import br.pucrio.olis.data.dao.WeatherUserServiceDAO;
import br.pucrio.olis.domain.City;
import br.pucrio.olis.domain.DegreeUnit;
import br.pucrio.olis.domain.User;
import br.pucrio.olis.domain.WeatherUserService;
import br.pucrio.olis.domain.weather.Weather;
import br.pucrio.olis.ontology.OLISOntology;

/**
 * <p>
 * This class implements the services provided by the Weather Service.
 * </p>
 * 
 * @author ingridnunes
 * @version 1.0
 * 
 */
public class WeatherServiceImpl extends ObservableBusinessServiceImpl implements
		WeatherService {

	private CityDAO cityDAO;
	private WeatherUserServiceDAO weatherUserServiceDAO;

	public WeatherServiceImpl(WeatherUserServiceDAO weatherUserServiceDAO,
			CityDAO cityDAO) {
		this.weatherUserServiceDAO = weatherUserServiceDAO;
		this.cityDAO = cityDAO;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.WeatherService#getWeather(br.pucrio.olis.domain.City)
	 */
	@Override
	public Weather getWeather(City city, DegreeUnit unit)
			throws BusinessException {
		WeatherRequest weatherRequest = new WeatherRequest();
		weatherRequest.setCity(cityDAO.get(city.getId()));
		weatherRequest.setCurrentConditions(true);
		weatherRequest.setDayForecasts(5);
		weatherRequest.setDegreeUnit(unit);

		AgentRequest request = new AgentRequest();
		request.setAgent(new AID("weather", AID.ISLOCALNAME));
		request.setContent(weatherRequest);
		request.setOntology(OLISOntology.ONTOLOGY_NAME);
		request.setPerformative(ACLMessage.REQUEST);

		try {
			WeatherResponse response = (WeatherResponse) FacadeAgent
					.getFacadeAgent().invoke(request);
			return response.getWeather();
		} catch (AgentInvocationException e) {
			log.error(e);
			throw new BusinessException(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.business.service.WeatherService#update(br.pucrio.olis.domain.WeatherUserService)
	 */
	@Override
	public User update(WeatherUserService weatherUserService)
			throws BusinessException {
		weatherUserServiceDAO.update(weatherUserService);
		return (weatherUserServiceDAO.get(weatherUserService.getId()))
				.getUser();
	}

}
