/*
 * Created on May 22, 2008
 */
package br.pucrio.olis.data.dao;

import java.util.Date;
import java.util.List;

import br.pucrio.olis.domain.CalendarEvent;
import br.pucrio.olis.domain.CalendarUserService;
import br.pucrio.olis.domain.Frequency;
import br.pucrio.olis.domain.User;

/**
 * <p>
 * This class represents the data access object of the CalendarUserService
 * domain entity.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public interface CalendarUserServiceDAO extends GenericDAO<CalendarUserService> {

	/**
	 * <p>
	 * Load the calendar events that occur in a certain month. The events start
	 * in the provided month. The events are also filtered by the frequency and
	 * the user.
	 * </p>
	 * 
	 * @param start
	 *            the start time that the events occur.
	 * @param end
	 *            the end time that the events occur.
	 * @param frequency
	 *            the frequency of the event.
	 * @param user
	 *            the user that participates from the events.
	 * @return
	 */
	public List<CalendarEvent> loadCalendarEventsBetweenDate(Date start,
			Date end, Frequency frequency, User user);

	/**
	 * <p>
	 * Load the calendar events that are valid in a certain month. The events
	 * should not start after the month ends and not end before the month
	 * begins. in the provided month. The events are also filtered by the
	 * frequency and the user.
	 * </p>
	 * 
	 * @param start
	 *            the start time that the events occur.
	 * @param end
	 *            the end time that the events occur.
	 * @param frequency
	 *            the frequency of the event.
	 * @param user
	 *            the user that participates from the events.
	 * @return
	 */
	public List<CalendarEvent> loadPeriodicCalendarEventsBetweenDate(
			Date start, Date end, Frequency frequency, User user);

}
