/*
 * Created on Apr 4, 2008
 */
package br.pucrio.olis.data.dao;

import java.io.Serializable;
import java.util.List;

import org.springframework.dao.DataAccessException;

/**
 * <p>
 * This interface defines the common methods of the data access objects. It
 * defines the methods that correspond to the CRUD methods.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 * @param <T>
 *            the entity type that is being accessed by this data access object.
 */
public interface GenericDAO<T> {

	/**
	 * <p>
	 * Delete an entity.
	 * </p>
	 * 
	 * @param entity
	 *            the entity to be deleted.
	 * @throws DataAccessException
	 */
	public void delete(T entity) throws DataAccessException;

	/**
	 * <p>
	 * Read the entity by its id.
	 * </p>
	 * 
	 * @param id
	 *            the entity id.
	 * @return the entity.
	 * @throws DataAccessException
	 */
	public T get(final Serializable id) throws DataAccessException;

	/**
	 * <p>
	 * List all entities.
	 * </p>
	 * 
	 * @return the list containing all the entities.
	 * @throws DataAccessException
	 */
	public List<T> loadAll() throws DataAccessException;

	/**
	 * <p>
	 * Save an entity.
	 * </p>
	 * 
	 * @param entity
	 *            the entity to be saved.
	 * @return the generated identifier.
	 * @throws DataAccessException
	 */
	public Serializable save(T entity) throws DataAccessException;

	/**
	 * <p>
	 * Update an entity.
	 * </p>
	 * 
	 * @param entity
	 *            the entity to be updated.
	 * @throws DataAccessException
	 */
	public void update(T entity) throws DataAccessException;

}
