/*
 * Created on Apr 4, 2008
 */
package br.pucrio.olis.data.dao;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.domain.Notification;
import br.pucrio.olis.domain.User;

/**
 * <p>
 * This class represents the data access object of the User domain entity.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public interface UserDAO extends GenericDAO<User> {

	/**
	 * <p>
	 * Add a user notification.
	 * </p>
	 * 
	 * @param user
	 * @param notification
	 * @throws BusinessException
	 */
	public void addNotification(User user, Notification notification)
			throws BusinessException;

	/**
	 * <p>
	 * Delete the user notification.
	 * </p>
	 * 
	 * @param user
	 * @param notification
	 * @throws BusinessException
	 */
	public void deleteNotification(User user, Notification notification)
			throws BusinessException;

	/**
	 * <p>
	 * Load a user by its username.
	 * </p>
	 * 
	 * @param username
	 *            the username.
	 * @return the user whose username is the one provided.
	 */
	public User loadUserByUsername(String username);

	/**
	 * <p>
	 * Update the user advanced options.
	 * </p>
	 * 
	 * @param user
	 *            the user to be updated.
	 */
	public void updateAdvancedOptions(User user);

}
