/*
 * Created on May 25, 2008
 */
package br.pucrio.olis.data.hibernate.dao;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

import org.springframework.dao.DataAccessException;

import br.pucrio.olis.data.dao.EventDAO;
import br.pucrio.olis.domain.AcademicEvent;
import br.pucrio.olis.domain.AreaOfInterest;
import br.pucrio.olis.domain.DeadlineDate;
import br.pucrio.olis.domain.Event;

/**
 * <p>
 * This class is the hibernate implementation of the academic event data access
 * object.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class AcademicEventDAOHibernate extends EventDAOHibernate implements
		EventDAO {

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.hibernate.dao.GenericDAOHibernate#getEntityClass()
	 */
	@SuppressWarnings("unchecked")
	@Override
	public Class getEntityClass() {
		return AcademicEvent.class;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.hibernate.dao.GenericDAOHibernate#save(java.lang.Object)
	 */
	@Override
	public Serializable save(Event entity) throws DataAccessException {
		AcademicEvent academicEvent = (AcademicEvent) entity;

		Set<AreaOfInterest> areasOfInterest = academicEvent
				.getAreasOfInterest();
		Serializable id = super.save(academicEvent);
		for (AreaOfInterest areaOfInterest : areasOfInterest) {
			academicEvent
					.addAreasOfInterest((AreaOfInterest) getHibernateTemplate()
							.get(AreaOfInterest.class, areaOfInterest.getId()));
		}

		return id;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.hibernate.dao.EventDAOHibernate#update(br.pucrio.olis.domain.Event)
	 */
	@Override
	public void update(Event entity) throws DataAccessException {
		AcademicEvent temp = (AcademicEvent) entity;
		AcademicEvent academicEvent = (AcademicEvent) get(entity.getId());

		academicEvent.setEventType(temp.getEventType());

		for (DeadlineDate deadlineDate : temp.getDeadlines()) {
			academicEvent.putDeadline(deadlineDate.getDeadline(), deadlineDate
					.getDate());
		}

		Set<AreaOfInterest> newAreasOfInterest = new HashSet<AreaOfInterest>(
				temp.getAreasOfInterest().size());
		for (AreaOfInterest areaOfInterest : temp.getAreasOfInterest()) {
			newAreasOfInterest.add((AreaOfInterest) getHibernateTemplate().get(
					AreaOfInterest.class, areaOfInterest.getId()));
		}
		Set<AreaOfInterest> deletedAreasOfInterest = new HashSet<AreaOfInterest>(
				academicEvent.getAreasOfInterest());
		deletedAreasOfInterest.removeAll(newAreasOfInterest);
		newAreasOfInterest.removeAll(academicEvent.getAreasOfInterest());

		for (AreaOfInterest areaOfInterest : newAreasOfInterest) {
			academicEvent.addAreasOfInterest(areaOfInterest);
		}
		for (AreaOfInterest areaOfInterest : deletedAreasOfInterest) {
			academicEvent.removeAreaOfInterests(areaOfInterest);
		}

		super.update(entity);
	}

}
