/*
 * Created on May 25, 2008
 */
package br.pucrio.olis.data.hibernate.dao;

import java.util.HashSet;
import java.util.Set;

import org.springframework.dao.DataAccessException;

import br.pucrio.olis.data.dao.AcademicPreferencesDAO;
import br.pucrio.olis.domain.AcademicEventType;
import br.pucrio.olis.domain.AcademicPreferences;
import br.pucrio.olis.domain.AreaOfInterest;

/**
 * <p>
 * This class is the hibernate implementation of the academic preferences data
 * access object.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class AcademicPreferencesDAOHibernate extends
		GenericDAOHibernate<AcademicPreferences> implements
		AcademicPreferencesDAO {

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.hibernate.dao.GenericDAOHibernate#getEntityClass()
	 */
	@SuppressWarnings("unchecked")
	@Override
	public Class getEntityClass() {
		return AcademicPreferences.class;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.hibernate.dao.GenericDAOHibernate#update(java.lang.Object)
	 */
	@Override
	public void update(AcademicPreferences entity) throws DataAccessException {
		AcademicPreferences academicPreferences = super.get(entity.getId());

		academicPreferences.setForeignEvents(entity.getForeignEvents());

		Set<AreaOfInterest> newAreasOfInterest = new HashSet<AreaOfInterest>(
				entity.getAreasOfInterest().size());
		for (AreaOfInterest areaOfInterest : entity.getAreasOfInterest()) {
			newAreasOfInterest.add((AreaOfInterest) getHibernateTemplate().get(
					AreaOfInterest.class, areaOfInterest.getId()));
		}
		Set<AreaOfInterest> deletedAreasOfInterest = new HashSet<AreaOfInterest>(
				academicPreferences.getAreasOfInterest());
		deletedAreasOfInterest.removeAll(newAreasOfInterest);
		newAreasOfInterest.removeAll(academicPreferences.getAreasOfInterest());

		for (AreaOfInterest areaOfInterest : newAreasOfInterest) {
			academicPreferences.addAreasOfInterest(areaOfInterest);
		}
		for (AreaOfInterest areaOfInterest : deletedAreasOfInterest) {
			academicPreferences.removeAreasOfInterest(areaOfInterest);
		}

		Set<AcademicEventType> newEventTypes = new HashSet<AcademicEventType>(
				entity.getEventTypes());
		Set<AcademicEventType> deletedEventTypes = new HashSet<AcademicEventType>(
				academicPreferences.getEventTypes());
		deletedEventTypes.removeAll(entity.getEventTypes());
		newEventTypes.removeAll(academicPreferences.getEventTypes());

		for (AcademicEventType academicEventType : newEventTypes) {
			academicPreferences.addEventTypes(academicEventType);
		}
		for (AcademicEventType academicEventType : deletedEventTypes) {
			academicPreferences.removeEventTypes(academicEventType);
		}
	}

}
