/*
 * Created on May 20, 2008
 */
package br.pucrio.olis.data.hibernate.dao;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.springframework.dao.DataAccessException;

import br.pucrio.olis.data.dao.CalendarEventDAO;
import br.pucrio.olis.domain.CalendarEvent;
import br.pucrio.olis.domain.CalendarUserService;
import br.pucrio.olis.domain.User;

/**
 * <p>
 * This class is the hibernate implementation of the calendar event data access
 * object.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class CalendarEventDAOHibernate extends
		GenericDAOHibernate<CalendarEvent> implements CalendarEventDAO {

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.hibernate.dao.GenericDAOHibernate#get(java.io.Serializable)
	 */
	@Override
	public CalendarEvent get(Serializable id) throws DataAccessException {
		CalendarEvent calendarEvent = super.get(id);
		// Iteration to load the participants from the database.
		Iterator<User> it = calendarEvent.getAllParticipants();
		while (it.hasNext()) {
			it.next().getFullName();
		}
		return calendarEvent;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.hibernate.dao.GenericDAOHibernate#getEntityClass()
	 */
	@SuppressWarnings("unchecked")
	@Override
	public Class getEntityClass() {
		return CalendarEvent.class;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.hibernate.dao.GenericDAOHibernate#save(java.lang.Object)
	 */
	@Override
	public Serializable save(CalendarEvent entity) throws DataAccessException {
		CalendarEvent calendarEvent = new CalendarEvent();
		calendarEvent.setAvailable(entity.getAvailable());
		calendarEvent.setEvent(entity.getEvent());
		Serializable id = super.save(calendarEvent);

		for (User user : entity.getParticipants()) {
			User u = (User) getHibernateTemplate()
					.get(User.class, user.getId());

			CalendarUserService calendarUserService = (CalendarUserService) u
					.getService(CalendarUserService.class);
			if (calendarUserService != null) {
				calendarUserService.addEvents(calendarEvent);
			} else {
				calendarEvent.addParticipants(u);
			}
		}

		calendarEvent.setOwner(entity.getOwner());
		return id;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.hibernate.dao.GenericDAOHibernate#update(java.lang.Object)
	 */
	@Override
	public void update(CalendarEvent entity) throws DataAccessException {
		CalendarEvent calendarEvent = super.get(entity.getId());
		calendarEvent.setAvailable(entity.getAvailable());

		Set<User> newUsers = new HashSet<User>(entity.getParticipants().size());
		for (User user : entity.getParticipants()) {
			newUsers.add((User) getHibernateTemplate().get(User.class,
					user.getId()));
		}
		Set<User> deletedUsers = new HashSet<User>(calendarEvent
				.getParticipants());
		deletedUsers.removeAll(newUsers);
		newUsers.removeAll(calendarEvent.getParticipants());

		for (User user : newUsers) {
			CalendarUserService calendarUserService = (CalendarUserService) user
					.getService(CalendarUserService.class);
			if (calendarUserService != null) {
				calendarUserService.addEvents(calendarEvent);
			} else {
				calendarEvent.addParticipants(user);
			}
		}
		for (User user : deletedUsers) {
			CalendarUserService calendarUserService = (CalendarUserService) user
					.getService(CalendarUserService.class);
			if (calendarUserService != null) {
				calendarUserService.removeEvents(calendarEvent);
			} else {
				calendarEvent.removeParticipants(user);
			}
		}

		calendarEvent.setOwner(entity.getOwner());
	}

}
