package br.pucrio.olis.data.hibernate.dao;

import java.util.Date;
import java.util.List;

import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.springframework.dao.DataAccessException;
import org.springframework.orm.hibernate3.HibernateCallback;

import br.pucrio.olis.data.dao.CalendarUserServiceDAO;
import br.pucrio.olis.domain.CalendarEvent;
import br.pucrio.olis.domain.CalendarUserService;
import br.pucrio.olis.domain.Frequency;
import br.pucrio.olis.domain.User;

/**
 * <p>
 * This class is the hibernate implementation of the calendar user service data
 * access object.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class CalendarUserServiceDAOHibernate extends
		GenericDAOHibernate<CalendarUserService> implements
		CalendarUserServiceDAO {

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.hibernate.dao.GenericDAOHibernate#getEntityClass()
	 */
	@SuppressWarnings("unchecked")
	@Override
	public Class getEntityClass() {
		return CalendarUserService.class;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.dao.CalendarUserServiceDAO#loadCalendarEventsBetweenDate(java.util.Date,
	 *      java.util.Date, br.pucrio.olis.domain.Frequency,
	 *      br.pucrio.olis.domain.User)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public List<CalendarEvent> loadCalendarEventsBetweenDate(final Date start,
			final Date end, final Frequency frequency, final User user) {
		return (List<CalendarEvent>) getHibernateTemplate().execute(
				new HibernateCallback() {
					public Object doInHibernate(Session session)
							throws HibernateException {
						return session.getNamedQuery(
								"calendarUserService.eventsBetweenDates")
								.setParameter("user", user).setParameter(
										"frequency", frequency).setParameter(
										"start", start)
								.setParameter("end", end).list();
					}
				});
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.dao.CalendarUserServiceDAO#loadPeriodicCalendarEventsBetweenDate(java.util.Date,
	 *      java.util.Date, br.pucrio.olis.domain.Frequency,
	 *      br.pucrio.olis.domain.User)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public List<CalendarEvent> loadPeriodicCalendarEventsBetweenDate(
			final Date start, final Date end, final Frequency frequency,
			final User user) {
		return (List<CalendarEvent>) getHibernateTemplate().execute(
				new HibernateCallback() {
					public Object doInHibernate(Session session)
							throws HibernateException {
						return session
								.getNamedQuery(
										"calendarUserService.periodicEventsBetweenDates")
								.setParameter("user", user).setParameter(
										"frequency", frequency).setParameter(
										"start", start)
								.setParameter("end", end).list();
					}
				});
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.hibernate.dao.GenericDAOHibernate#update(java.lang.Object)
	 */
	@Override
	public void update(CalendarUserService entity) throws DataAccessException {
		CalendarUserService calendarUserService = super.get(entity.getId());
		calendarUserService.setRemindEvents(entity.getRemindEvents());
		calendarUserService.setRemindMinutesBefore(entity
				.getRemindMinutesBefore());
		calendarUserService.setShowWeekends(entity.getShowWeekends());
	}

}
