/*
 * Created on May 24, 2008
 */
package br.pucrio.olis.data.hibernate.dao;

import java.util.Date;
import java.util.List;

import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.springframework.dao.DataAccessException;
import org.springframework.orm.hibernate3.HibernateCallback;

import br.pucrio.olis.data.dao.EventsAnnouncementUserServiceDAO;
import br.pucrio.olis.domain.Event;
import br.pucrio.olis.domain.EventsAnnouncementUserService;
import br.pucrio.olis.domain.Frequency;

/**
 * <p>
 * This class is the hibernate implementation of the events announcement user
 * service data access object.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class EventsAnnouncementUserServiceDAOHibernate extends
		GenericDAOHibernate<EventsAnnouncementUserService> implements
		EventsAnnouncementUserServiceDAO {

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.hibernate.dao.GenericDAOHibernate#getEntityClass()
	 */
	@SuppressWarnings("unchecked")
	@Override
	public Class getEntityClass() {
		return EventsAnnouncementUserService.class;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.dao.EventsAnnouncementUserServiceDAO#loadEventsBetweenDates(java.util.Date,
	 *      java.util.Date)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public List<Event> loadEventsBetweenDates(final Date start, final Date end) {
		return (List<Event>) getHibernateTemplate().execute(
				new HibernateCallback() {
					public Object doInHibernate(Session session)
							throws HibernateException {
						return session
								.getNamedQuery(
										"eventsAnnouncementUserService.eventsBetweenDates")
								.setParameter("frequency", Frequency.ONCE)
								.setParameter("start", start).setParameter(
										"end", end).list();
					}
				});
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.dao.EventsAnnouncementUserServiceDAO#loadPeriodicEventBetweenDates(java.util.Date,
	 *      java.util.Date)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public List<Event> loadPeriodicEventBetweenDates(final Date start,
			final Date end) {
		return (List<Event>) getHibernateTemplate().execute(
				new HibernateCallback() {
					public Object doInHibernate(Session session)
							throws HibernateException {
						return session
								.getNamedQuery(
										"eventsAnnouncementUserService.periodicEventsBetweenDates")
								.setParameter("frequency", Frequency.ONCE)
								.setParameter("start", start).setParameter(
										"end", end).list();
					}
				});
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.hibernate.dao.GenericDAOHibernate#update(java.lang.Object)
	 */
	@Override
	public void update(EventsAnnouncementUserService entity)
			throws DataAccessException {
		EventsAnnouncementUserService eventsAnnouncementUserService = super
				.get(entity.getId());
		eventsAnnouncementUserService.setSuggestEvents(entity
				.getSuggestEvents());
		eventsAnnouncementUserService.setWorkTimeBegin(entity
				.getWorkTimeBegin());
		eventsAnnouncementUserService.setWorkTimeEnd(entity.getWorkTimeEnd());
	}

}
