/*
 * Created on Apr 4, 2008
 */
package br.pucrio.olis.data.hibernate.dao;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.dao.DataAccessException;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;

import br.pucrio.olis.data.dao.GenericDAO;

/**
 * <p>
 * This class is an abstract class that contains the default implementation of
 * the methods of the generic data access object.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public abstract class GenericDAOHibernate<T> extends HibernateDaoSupport
		implements GenericDAO<T> {

	protected Log log;

	/**
	 * Create a new instance of GenericDAOHibernate.
	 */
	public GenericDAOHibernate() {
		this.log = LogFactory.getLog(this.getClass());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.dao.GenericDAO#delete(java.lang.Object)
	 */
	@Override
	public void delete(T entity) throws DataAccessException {
		getHibernateTemplate().delete(entity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.dao.GenericDAO#get(java.lang.Object)
	 */
	@Override
	@SuppressWarnings("unchecked")
	public T get(Serializable id) throws DataAccessException {
		T t = (T) getHibernateTemplate().get(getEntityClass(), id);

		return t;
	}

	/**
	 * Gets the entity class.
	 * 
	 * @return the entity class managed by the DAO.
	 */
	@SuppressWarnings("unchecked")
	public abstract Class getEntityClass();

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.dao.GenericDAO#loadAll()
	 */
	@Override
	@SuppressWarnings("unchecked")
	public List<T> loadAll() throws DataAccessException {
		// Corrects the errors of loading duplicates entities when the fetch
		// type is EAGER
		return new ArrayList(new LinkedHashSet(getHibernateTemplate().loadAll(
				getEntityClass())));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.dao.GenericDAO#save(java.lang.Object)
	 */
	@Override
	public Serializable save(T entity) throws DataAccessException {
		Serializable id = getHibernateTemplate().save(entity);
		getHibernateTemplate().refresh(entity);
		getHibernateTemplate().flush();
		getHibernateTemplate().refresh(entity);
		return id;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.dao.GenericDAO#update(java.lang.Object)
	 */
	@Override
	public void update(T entity) throws DataAccessException {
		getHibernateTemplate().update(entity);
		getHibernateTemplate().flush();
		getHibernateTemplate().refresh(entity);
	}

}
