/*
 * Created on May 25, 2008
 */
package br.pucrio.olis.data.hibernate.dao;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

import org.springframework.dao.DataAccessException;

import br.pucrio.olis.data.dao.EventDAO;
import br.pucrio.olis.domain.ActivityType;
import br.pucrio.olis.domain.Event;
import br.pucrio.olis.domain.TravelEvent;

/**
 * <p>
 * This class is the hibernate implementation of the travel event data access
 * object.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class TravelEventDAOHibernate extends EventDAOHibernate implements
		EventDAO {

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.hibernate.dao.GenericDAOHibernate#getEntityClass()
	 */
	@SuppressWarnings("unchecked")
	@Override
	public Class getEntityClass() {
		return TravelEvent.class;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.hibernate.dao.GenericDAOHibernate#save(java.lang.Object)
	 */
	@Override
	public Serializable save(Event entity) throws DataAccessException {
		TravelEvent travelEvent = (TravelEvent) entity;

		Set<ActivityType> activities = travelEvent.getActivities();
		Serializable id = super.save(travelEvent);
		for (ActivityType activityType : activities) {
			travelEvent.addActivities((ActivityType) getHibernateTemplate()
					.get(ActivityType.class, activityType.getId()));
		}

		return id;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.hibernate.dao.EventDAOHibernate#update(br.pucrio.olis.domain.Event)
	 */
	@Override
	public void update(Event entity) throws DataAccessException {
		TravelEvent temp = (TravelEvent) entity;
		TravelEvent travelEvent = (TravelEvent) get(entity.getId());

		travelEvent.setPlaceType(temp.getPlaceType());

		Set<ActivityType> newActivities = new HashSet<ActivityType>(temp
				.getActivities().size());
		for (ActivityType activityType : temp.getActivities()) {
			newActivities.add((ActivityType) getHibernateTemplate().get(
					ActivityType.class, activityType.getId()));
		}
		Set<ActivityType> deletedActivities = new HashSet<ActivityType>(
				travelEvent.getActivities());
		deletedActivities.removeAll(newActivities);
		newActivities.removeAll(travelEvent.getActivities());

		for (ActivityType activityType : newActivities) {
			travelEvent.addActivities(activityType);
		}
		for (ActivityType activityType : deletedActivities) {
			travelEvent.removeActivities(activityType);
		}

		super.update(entity);
	}

}
