/*
 * Created on May 25, 2008
 */
package br.pucrio.olis.data.hibernate.dao;

import java.util.HashSet;
import java.util.Set;

import org.springframework.dao.DataAccessException;

import br.pucrio.olis.data.dao.TravelPreferencesDAO;
import br.pucrio.olis.domain.ActivityType;
import br.pucrio.olis.domain.PlaceType;
import br.pucrio.olis.domain.TravelPreferences;

/**
 * <p>
 * This class is the hibernate implementation of the travel preferences data
 * access object.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class TravelPreferencesDAOHibernate extends
		GenericDAOHibernate<TravelPreferences> implements TravelPreferencesDAO {

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.hibernate.dao.GenericDAOHibernate#getEntityClass()
	 */
	@SuppressWarnings("unchecked")
	@Override
	public Class getEntityClass() {
		return TravelPreferences.class;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.hibernate.dao.GenericDAOHibernate#update(java.lang.Object)
	 */
	@Override
	public void update(TravelPreferences entity) throws DataAccessException {
		TravelPreferences travelPreferences = super.get(entity.getId());

		Set<ActivityType> newActivities = new HashSet<ActivityType>(entity
				.getActivities().size());
		for (ActivityType activityType : entity.getActivities()) {
			newActivities.add((ActivityType) getHibernateTemplate().get(
					ActivityType.class, activityType.getId()));
		}
		Set<ActivityType> deletedActivities = new HashSet<ActivityType>(
				travelPreferences.getActivities());
		deletedActivities.removeAll(newActivities);
		newActivities.removeAll(travelPreferences.getActivities());

		for (ActivityType activityType : newActivities) {
			travelPreferences.addActivities(activityType);
		}
		for (ActivityType activityType : deletedActivities) {
			travelPreferences.removeActivities(activityType);
		}

		Set<PlaceType> newPlaceTypes = new HashSet<PlaceType>(entity
				.getPlaceTypes().size());
		for (PlaceType placeType : entity.getPlaceTypes()) {
			newPlaceTypes.add((PlaceType) getHibernateTemplate().get(
					PlaceType.class, placeType.getId()));
		}
		Set<PlaceType> deletedPlaceTypes = new HashSet<PlaceType>(
				travelPreferences.getPlaceTypes());
		deletedPlaceTypes.removeAll(newPlaceTypes);
		newPlaceTypes.removeAll(travelPreferences.getPlaceTypes());

		for (PlaceType placeType : newPlaceTypes) {
			travelPreferences.addPlaceTypes(placeType);
		}
		for (PlaceType placeType : deletedPlaceTypes) {
			travelPreferences.removePlaceTypes(placeType);
		}
	}

}
