/*
 * Created on Apr 4, 2008
 */
package br.pucrio.olis.data.hibernate.dao;

import java.io.Serializable;
import java.util.Iterator;

import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.criterion.Restrictions;
import org.springframework.dao.DataAccessException;
import org.springframework.orm.hibernate3.HibernateCallback;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.data.dao.UserDAO;
import br.pucrio.olis.data.dao.factory.UserPreferencesAbstractFactory;
import br.pucrio.olis.domain.Notification;
import br.pucrio.olis.domain.User;
import br.pucrio.olis.domain.UserPreferences;
import br.pucrio.olis.domain.UserService;
import br.pucrio.olis.domain.UserServiceType;

/**
 * <p>
 * This class is the hibernate implementation of the user data access object.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class UserDAOHibernate extends GenericDAOHibernate<User> implements
		UserDAO {

	private UserPreferencesAbstractFactory userPreferencesAbstractFactory;

	/**
	 * <p>
	 * Create an instance of UserDAOHibernate.
	 * </p>
	 * 
	 * @param userPreferencesAbstractFactory
	 *            the user preferences abstract factory.
	 */
	public UserDAOHibernate(
			UserPreferencesAbstractFactory userPreferencesAbstractFactory) {
		this.userPreferencesAbstractFactory = userPreferencesAbstractFactory;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.dao.UserDAO#addNotification(br.pucrio.olis.domain.User,
	 *      br.pucrio.olis.domain.Notification)
	 */
	@Override
	public void addNotification(User user, Notification notification)
			throws BusinessException {
		User u = get(user.getId());
		getHibernateTemplate().save(notification);
		u.getNotifications().add(notification);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.dao.UserDAO#deleteNotification(br.pucrio.olis.domain.User,
	 *      br.pucrio.olis.domain.Notification)
	 */
	@Override
	public void deleteNotification(User user, Notification notification)
			throws BusinessException {
		User u = get(user.getId());
		Notification n = (Notification) getHibernateTemplate().get(
				Notification.class, notification.getId());
		u.getNotifications().remove(n);
		getHibernateTemplate().delete(n);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.hibernate.dao.GenericDAOHibernate#getEntityClass()
	 */
	@Override
	@SuppressWarnings("unchecked")
	public Class getEntityClass() {
		return User.class;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.dao.UserDAO#loadUserByUsername(java.lang.String)
	 */
	@Override
	public User loadUserByUsername(final String username) {
		return (User) getHibernateTemplate().execute(new HibernateCallback() {
			public Object doInHibernate(Session session)
					throws HibernateException {
				return (User) session.createCriteria(User.class).add(
						Restrictions.eq("username", username)).uniqueResult();
			}
		});
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.hibernate.dao.GenericDAOHibernate#save(java.lang.Object)
	 */
	@Override
	public Serializable save(User entity) throws DataAccessException {
		Serializable id = super.save(entity);

		Iterator<UserServiceType> it = UserServiceType.getServices();
		while (it.hasNext()) {
			UserServiceType userServiceType = it.next();
			UserService service = userServiceType.createUserService(entity);
			getHibernateTemplate().save(service);
			entity.addServices(service);
		}

		UserPreferences preferences = userPreferencesAbstractFactory
				.createUserPreferences();
		if (preferences != null) {
			getHibernateTemplate().save(preferences);
			entity.setPreferences(preferences);
		}

		return id;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.hibernate.dao.GenericDAOHibernate#update(java.lang.Object)
	 */
	@Override
	public void update(User entity) throws DataAccessException {
		User user = get(entity.getId());
		user.setCity(entity.getCity());
		user.setDateOfBirth(entity.getDateOfBirth());
		user.setEmail(entity.getEmail());
		user.setFirstName(entity.getFirstName());
		user.setLastName(entity.getLastName());
		user.setPassword(entity.getPassword());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.data.dao.UserDAO#updateAdvancedOptions(br.pucrio.olis.domain.User)
	 */
	@Override
	public void updateAdvancedOptions(User entity) {
		User user = super.get(entity.getId());
		user.setDateFormat(entity.getDateFormat());
		user.setTimeFormat(entity.getTimeFormat());
		super.update(user);
	}

}
