/*
 * Created on May 1, 2008
 */
package br.pucrio.olis.domain;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.ManyToMany;
import javax.persistence.Transient;

import org.hibernate.annotations.CollectionOfElements;
import org.hibernate.annotations.Type;

/**
 * <p>
 * This class represents the user academic preferences.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
@Entity
public class AcademicPreferences extends UserPreferences {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 4281974406843215861L;

	/**
	 * The Set represents the set of areas of interest that a user is interested
	 * to.
	 */
	private Set<AreaOfInterest> areasOfInterest;
	/**
	 * The Set represents the set of event types that a user is interested to.
	 */
	private Set<AcademicEventType> eventTypes;
	/**
	 * The Boolean represents if the user wants to receive suggestions about
	 * foreign events.
	 */
	private Boolean foreignEvents;

	/**
	 * <p>
	 * Create an instance of AcademicPreferences.
	 * </p>
	 */
	public AcademicPreferences() {
		this.areasOfInterest = new HashSet<AreaOfInterest>();
		this.eventTypes = new HashSet<AcademicEventType>();
	}

	/**
	 * <p>
	 * Add an area of interest.
	 * </p>
	 * 
	 * @param areaOfInterest
	 *            the area of interest to be added.
	 */
	public void addAreasOfInterest(AreaOfInterest areaOfInterest) {
		this.areasOfInterest.add(areaOfInterest);
	}

	/**
	 * <p>
	 * Add an academic event type.
	 * </p>
	 * 
	 * @param academicEventType
	 *            the academic event type to be added.
	 */
	public void addEventTypes(AcademicEventType academicEventType) {
		this.eventTypes.add(academicEventType);
	}
	
	@Transient
	public Iterator<AreaOfInterest> getAllAreasOfInterest() {
		return areasOfInterest.iterator();
	}

	@Transient
	public Iterator<AcademicEventType> getAllEventTypes() {
		return eventTypes.iterator();
	}
	
	@ManyToMany(fetch = FetchType.EAGER)
	public Set<AreaOfInterest> getAreasOfInterest() {
		return areasOfInterest;
	}

	@CollectionOfElements(fetch = FetchType.EAGER)
	public Set<AcademicEventType> getEventTypes() {
		return eventTypes;
	}

	@Type(type = "yes_no")
	public Boolean getForeignEvents() {
		return foreignEvents;
	}

	/**
	 * <p>
	 * Remove an academic event type.
	 * </p>
	 * 
	 * @param academicEventType
	 *            the academic event type to be removed.
	 */
	public boolean removeEventTypes(AcademicEventType academicEventType) {
		return this.eventTypes.remove(academicEventType);
	}

	/**
	 * <p>
	 * Remove an area of interest.
	 * </p>
	 * 
	 * @param areaOfInterest
	 *            the area of interest to be removed.
	 */
	public boolean removeAreasOfInterest(AreaOfInterest areaOfInterest) {
		return this.areasOfInterest.remove(areaOfInterest);
	}

	public void setAreasOfInterest(Set<AreaOfInterest> areasOfInterest) {
		this.areasOfInterest = areasOfInterest;
	}

	public void setEventTypes(Set<AcademicEventType> eventTypes) {
		this.eventTypes = eventTypes;
	}

	public void setForeignEvents(Boolean foreignEvents) {
		this.foreignEvents = foreignEvents;
	}

}
