/*
 * Created on May 1, 2008
 */
package br.pucrio.olis.domain;

import jade.content.Concept;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.Transient;

import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;
import org.hibernate.annotations.Type;

/**
 * <p>
 * This class represents an event that is part of the user calendar. It is
 * associated with an event, but have some particular properties, such as the
 * participants of the event. It is shared among all the participants of the
 * event.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
@Entity
public class CalendarEvent implements Serializable, Concept {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 875619742611128207L;

	/**
	 * The Boolean represents if the time of the user schedule is available to
	 * other calendar events.
	 */
	private Boolean available;
	/**
	 * The Event represents the event that this calendar event is associated to.
	 */
	private Event event;
	/**
	 * The Long represents the calendar event unique identifier.
	 */
	private Long id;
	/**
	 * The Set represents the group of users that participates of this event.
	 */
	private List<User> participants;

	/**
	 * <p>
	 * Create an instance of CalendarEvent.
	 * </p>
	 */
	public CalendarEvent() {
		this.participants = new ArrayList<User>();
	}

	/**
	 * <p>
	 * Add a participant to this calendar event.
	 * </p>
	 * 
	 * @param user
	 *            the user to be added.
	 */
	public void addParticipants(User user) {
		this.participants.add(user);
	}

	@Transient
	public Iterator<User> getAllParticipants() {
		return participants.iterator();
	}

	@Type(type = "yes_no")
	public Boolean getAvailable() {
		return available;
	}

	@ManyToOne
	@OnDelete(action = OnDeleteAction.CASCADE)
	public Event getEvent() {
		return event;
	}

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	public Long getId() {
		return id;
	}

	@Transient
	public User getOwner() {
		if (participants.isEmpty())
			return null;
		return participants.get(0);
	}

	@ManyToMany
	public List<User> getParticipants() {
		return participants;
	}

	/**
	 * <p>
	 * Remove a participant from this calendar event.
	 * </p>
	 * 
	 * @param user
	 *            the user to be removed.
	 */
	public boolean removeParticipants(User user) {
		return this.participants.remove(user);
	}

	public void setAvailable(Boolean available) {
		this.available = available;
	}

	public void setEvent(Event event) {
		this.event = event;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public void setOwner(User user) {
		if (participants.contains(user)) {
			participants.remove(user);
		}
		participants.add(0, user);
	}

	public void setParticipants(List<User> participants) {
		this.participants = participants;
	}

}
