/*
 * Created on May 1, 2008
 */
package br.pucrio.olis.domain;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.ManyToMany;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Transient;

import org.hibernate.annotations.Type;

/**
 * <p>
 * This class represents the calendar user service. It keeps the information of
 * the service, besides the user specific configuration of the service.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
@Entity
@NamedQueries( {
		@NamedQuery(name = "calendarUserService.eventsBetweenDates", query = "select ev from CalendarUserService calendarUserService join calendarUserService.events as ev where calendarUserService.user = :user and ev.event.frequency = :frequency and ((ev.event.start >= :start and ev.event.start <= :end) or (ev.event.end >= :start and ev.event.end <= :end))"),
		@NamedQuery(name = "calendarUserService.periodicEventsBetweenDates", query = "select ev from CalendarUserService calendarUserService join calendarUserService.events as ev where calendarUserService.user = :user and ev.event.frequency = :frequency and not(ev.event.end < :start) and not(ev.event.start > :end)") })
public class CalendarUserService extends UserService {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 7477967937931834863L;

	public static final UserServiceType SERVICE_TYPE;

	static {
		SERVICE_TYPE = new UserServiceType();
		SERVICE_TYPE.setServiceLabel("service.calendar");
		List<UserServiceType.MenuOption> menuOptions = new ArrayList<UserServiceType.MenuOption>();
		menuOptions.add(SERVICE_TYPE.new MenuOption("Calendar_open",
				"message.calendar"));
		menuOptions.add(SERVICE_TYPE.new MenuOption("CalendarEvent_input",
				"message.create.event"));
		SERVICE_TYPE.setServiceMenuOptions(menuOptions);
		SERVICE_TYPE.setServiceNameSpace("calendar");

		CalendarUserService calendarUserService = new CalendarUserService();
		calendarUserService.setRemindEvents(Boolean.TRUE);
		calendarUserService.setRemindMinutesBefore(5);
		calendarUserService.setShowWeekends(Boolean.TRUE);
		SERVICE_TYPE.setDefaultInstance(calendarUserService);

		UserServiceType.addUserServiceType(SERVICE_TYPE);
	}

	/**
	 * The Set represents the calendar events that are part from the user
	 * calendar.
	 */
	private Set<CalendarEvent> events;

	/**
	 * The Boolean determines if the user wants to be remembered about events.
	 */
	private Boolean remindEvents;
	/**
	 * The Integer represents how many minutes before the event starts the user
	 * wants to be remembered.
	 */
	private Integer remindMinutesBefore;
	/**
	 * The Boolean determines if the user wants that weekends should appear in
	 * the calendar.
	 */
	private Boolean showWeekends;

	/**
	 * <p>
	 * Create an instance of CalendarUserService.
	 * </p>
	 */
	public CalendarUserService() {
		this.events = new HashSet<CalendarEvent>();
	}

	/**
	 * <p>
	 * Add a calendar event to this service.
	 * </p>
	 * 
	 * @param calendarEvent
	 *            the calendar event to be added.
	 */
	public void addEvents(CalendarEvent calendarEvent) {
		calendarEvent.addParticipants(user);
		this.events.add(calendarEvent);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.domain.UserService#clone(br.pucrio.olis.domain.User)
	 */
	@Override
	public UserService clone(User user) {
		CalendarUserService calendarUserService = new CalendarUserService();
		calendarUserService.remindEvents = this.remindEvents;
		calendarUserService.remindMinutesBefore = this.remindMinutesBefore;
		calendarUserService.showWeekends = this.showWeekends;
		return calendarUserService;
	}

	@Transient
	public Iterator<CalendarEvent> getAllEvents() {
		return events.iterator();
	}

	@ManyToMany(fetch = FetchType.EAGER)
	public Set<CalendarEvent> getEvents() {
		return events;
	}

	@Type(type = "yes_no")
	public Boolean getRemindEvents() {
		return remindEvents;
	}
	
	public Integer getRemindMinutesBefore() {
		return remindMinutesBefore;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.domain.UserService#getServiceType()
	 */
	@Override
	@Transient
	public UserServiceType getServiceType() {
		return SERVICE_TYPE;
	}

	@Type(type = "yes_no")
	public Boolean getShowWeekends() {
		return showWeekends;
	}

	/**
	 * <p>
	 * Remove a calendar event from this service.
	 * </p>
	 * 
	 * @param calendarEvent
	 *            the calendar event to be removed.
	 */
	public boolean removeEvents(CalendarEvent calendarEvent) {
		calendarEvent.removeParticipants(user);
		return this.events.remove(calendarEvent);
	}

	public void setEvents(Set<CalendarEvent> events) {
		this.events = events;
	}

	public void setRemindEvents(Boolean remindEvents) {
		this.remindEvents = remindEvents;
	}

	public void setRemindMinutesBefore(Integer remindMinutesBefore) {
		this.remindMinutesBefore = remindMinutesBefore;
	}

	public void setShowWeekends(Boolean showWeekends) {
		this.showWeekends = showWeekends;
	}

}
