/*
 * Created on May 25, 2008
 */
package br.pucrio.olis.domain;

import jade.content.Concept;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

/**
 * <p>
 * This class represents a deadline date of an academic event.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
@Entity
public class DeadlineDate implements Serializable, Concept {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 2480325077946091270L;
	/**
	 * The AcademicEvent represents the academic event that the deadline belongs
	 * to.
	 */
	private AcademicEvent academicEvent;
	/**
	 * The Date represents the deadline date.
	 */
	private Date date;
	/**
	 * The Deadline represents the deadline type of the deadline date.
	 */
	private Deadline deadline;
	/**
	 * The Long represents the event unique identifier.
	 */
	protected Long id;

	/**
	 * <p>
	 * Create an instance of DeadlineDate.
	 * </p>
	 */
	public DeadlineDate() {

	}

	/**
	 * <p>
	 * Create an instance of DeadlineDate.
	 * </p>
	 * 
	 * @param deadline
	 *            the deadline.
	 * @param date
	 *            the deadline date.
	 * @param academicEvent
	 *            the academic event.
	 */
	public DeadlineDate(Deadline deadline, Date date,
			AcademicEvent academicEvent) {
		this.deadline = deadline;
		this.date = date;
		this.academicEvent = academicEvent;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof DeadlineDate))
			return false;
		return id.equals(((DeadlineDate) obj).getId());
	}

	@ManyToOne
	public AcademicEvent getAcademicEvent() {
		return academicEvent;
	}

	@Temporal(value = TemporalType.DATE)
	public Date getDate() {
		return date;
	}

	@Enumerated(EnumType.STRING)
	public Deadline getDeadline() {
		return deadline;
	}

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	public Long getId() {
		return id;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		return id.hashCode();
	}

	public void setAcademicEvent(AcademicEvent academicEvent) {
		this.academicEvent = academicEvent;
	}

	public void setDate(Date date) {
		this.date = date;
	}

	public void setDeadline(Deadline deadline) {
		this.deadline = deadline;
	}

	public void setId(Long id) {
		this.id = id;
	}

}
