/*
 * Created on May 1, 2008
 */
package br.pucrio.olis.domain;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.Entity;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.annotations.Type;

/**
 * <p>
 * This class represents the events announcement user service. It keeps the
 * information of the service, besides the user specific configuration of the
 * service.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
@Entity
@NamedQueries( {
		@NamedQuery(name = "eventsAnnouncementUserService.eventsBetweenDates", query = "select event from EventsBoard eventsBoard join eventsBoard.events as event where event.frequency = :frequency and ((event.start >= :start and event.start <= :end) or (event.end >= :start and event.end <= :end))"),
		@NamedQuery(name = "eventsAnnouncementUserService.periodicEventsBetweenDates", query = "select event from EventsBoard eventsBoard join eventsBoard.events as event where event.frequency != :frequency and not(event.end < :start) and not(event.start > :end)") })
public class EventsAnnouncementUserService extends UserService {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 2050851658063724868L;

	public static final UserServiceType SERVICE_TYPE;

	static {
		SERVICE_TYPE = new UserServiceType();
		SERVICE_TYPE.setServiceLabel("service.events.announcement");
		List<UserServiceType.MenuOption> menuOptions = new ArrayList<UserServiceType.MenuOption>();
		menuOptions.add(SERVICE_TYPE.new MenuOption("EventsBoard_open",
				"message.events.board"));
		menuOptions.add(SERVICE_TYPE.new MenuOption("Event_input",
				"message.announce.event"));
		SERVICE_TYPE.setServiceMenuOptions(menuOptions);
		SERVICE_TYPE.setServiceNameSpace("eventsannouncement");

		EventsAnnouncementUserService eventsAnnouncementUserService = new EventsAnnouncementUserService();
		eventsAnnouncementUserService.setSuggestEvents(Boolean.TRUE);
		eventsAnnouncementUserService.setWorkTimeBegin(null);
		eventsAnnouncementUserService.setWorkTimeEnd(null);
		SERVICE_TYPE.setDefaultInstance(eventsAnnouncementUserService);

		UserServiceType.addUserServiceType(SERVICE_TYPE);
	}

	/**
	 * The Boolean represents if the user should receive events suggestions.
	 */
	private Boolean suggestEvents;

	/**
	 * The Date represents the time that the user starts to work.
	 */
	private Date workTimeBegin;

	/**
	 * The Date represents the time that the user ends to work.
	 */
	private Date workTimeEnd;

	/**
	 * <p>
	 * Create an instance of EventsAnnouncementUserService.
	 * </p>
	 */
	public EventsAnnouncementUserService() {
		suggestEvents = Boolean.TRUE;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.domain.UserService#clone(br.pucrio.olis.domain.User)
	 */
	@Override
	public UserService clone(User user) {
		EventsAnnouncementUserService eventsAnnouncementUserService = new EventsAnnouncementUserService();
		eventsAnnouncementUserService.suggestEvents = this.suggestEvents;
		eventsAnnouncementUserService.workTimeBegin = this.workTimeBegin;
		eventsAnnouncementUserService.workTimeEnd = this.workTimeEnd;
		return eventsAnnouncementUserService;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.domain.UserService#getServiceType()
	 */
	@Override
	@Transient
	public UserServiceType getServiceType() {
		return SERVICE_TYPE;
	}

	@Type(type = "yes_no")
	public Boolean getSuggestEvents() {
		return suggestEvents;
	}

	@Temporal(TemporalType.TIMESTAMP)
	public Date getWorkTimeBegin() {
		return workTimeBegin;
	}

	@Temporal(TemporalType.TIMESTAMP)
	public Date getWorkTimeEnd() {
		return workTimeEnd;
	}

	public void setSuggestEvents(Boolean suggestEvents) {
		this.suggestEvents = suggestEvents;
	}

	public void setWorkTimeBegin(Date workTimeBegin) {
		this.workTimeBegin = workTimeBegin;
	}

	public void setWorkTimeEnd(Date workTimeEnd) {
		this.workTimeEnd = workTimeEnd;
	}

}
