/*
 * Created on May 1, 2008
 */
package br.pucrio.olis.domain;

import jade.content.Concept;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.Transient;

/**
 * <p>
 * This class represents the board where the events are announced. There is a
 * single event board in the system.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
@Entity
public class EventsBoard implements Serializable, Concept {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 3469724701191863191L;

	/**
	 * The Set represents the set of events that makes part of this events
	 * board.
	 */
	private Set<Event> events;
	/**
	 * The Long represents the city unique identifier.
	 */
	private Long id;

	/**
	 * <p>
	 * Create an instance of EventsBoard.
	 * </p>
	 */
	public EventsBoard() {
		this.events = new HashSet<Event>();
	}

	/**
	 * <p>
	 * Add an event to the events board.
	 * </p>
	 * 
	 * @param event
	 *            the event to be added.
	 */
	public void addEvents(Event event) {
		event.setEventsBoard(this);
		this.events.add(event);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof EventsBoard))
			return false;
		return id.equals(((EventsBoard) obj).getId());
	}

	@Transient
	public Iterator<Event> getAllEvents() {
		return events.iterator();
	}

	@OneToMany(mappedBy = "eventsBoard", fetch = FetchType.EAGER)
	@OrderBy("start")
	public Set<Event> getEvents() {
		return events;
	}

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	public Long getId() {
		return id;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		return id.hashCode();
	}

	/**
	 * <p>
	 * Remove an event from the event board.
	 * </p>
	 * 
	 * @param event
	 *            the event to be removed.
	 * 
	 * @return true if the event was removed.
	 */
	public boolean removeEvents(Event event) {
		event.setEventsBoard(null);
		return this.events.remove(event);
	}

	public void setEvents(Set<Event> events) {
		this.events = events;
	}

	public void setId(Long id) {
		this.id = id;
	}

}
