/*
 * Created on May 1, 2008
 */
package br.pucrio.olis.domain;

import java.util.Calendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * <p>
 * This enumeration contains the possible frequencies of an event.
 * </p>
 * 
 * <ul>
 * <li> EVERY_WEEKDAY: every Monday, Tuesday, Wednesday, Thursday and Friday.</li>
 * <li> EVERY_WEEKEND: every Saturday and Sunday.</li>
 * <li> DAILY: every day.</li>
 * <li> EVERY_ODD_DAY: every Tuesday and Thursday.</li>
 * <li> EVERY_EVEN_DAY: every Monday, Wednesday, and Friday.</li>
 * </ul>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public enum Frequency {

	DAILY("frequency.daily"), EVERY_EVEN_DAY("frequency.every.even.day"), EVERY_ODD_DAY(
			"frequency.every.odd.day"), EVERY_WEEKDAY("frequency.every.weekday"), EVERY_WEEKEND(
			"frequency.every.weekend"), FRIDAY("frequency.friday"), MONDAY(
			"frequency.monday"), ONCE("frequency.once"), SATURDAY(
			"frequency.saturday"), SUNDAY("frequency.sunday"), THURSDAY(
			"frequency.thurday"), TUESDAY("frequency.tuesday"), WEDNESDAY(
			"frequency.wednesday");

	/**
	 * The Map indicates which days of the week make part of each one of the
	 * frequencies.
	 */
	public static final Map<Frequency, Set<Integer>> DAYS;

	static {
		DAYS = new HashMap<Frequency, Set<Integer>>();
		Set<Integer> days = new HashSet<Integer>();
		days.add(Calendar.MONDAY);
		days.add(Calendar.TUESDAY);
		days.add(Calendar.WEDNESDAY);
		days.add(Calendar.THURSDAY);
		days.add(Calendar.FRIDAY);
		days.add(Calendar.SATURDAY);
		days.add(Calendar.SUNDAY);
		DAYS.put(Frequency.DAILY, days);
		days = new HashSet<Integer>();
		days.add(Calendar.MONDAY);
		days.add(Calendar.WEDNESDAY);
		days.add(Calendar.FRIDAY);
		DAYS.put(Frequency.EVERY_EVEN_DAY, days);
		days = new HashSet<Integer>();
		days.add(Calendar.TUESDAY);
		days.add(Calendar.THURSDAY);
		DAYS.put(Frequency.EVERY_ODD_DAY, days);
		days = new HashSet<Integer>();
		days.add(Calendar.MONDAY);
		days.add(Calendar.TUESDAY);
		days.add(Calendar.WEDNESDAY);
		days.add(Calendar.THURSDAY);
		days.add(Calendar.FRIDAY);
		DAYS.put(Frequency.EVERY_WEEKDAY, days);
		days = new HashSet<Integer>();
		days.add(Calendar.SATURDAY);
		days.add(Calendar.SUNDAY);
		DAYS.put(Frequency.EVERY_WEEKEND, days);
		days = new HashSet<Integer>();
		DAYS.put(Frequency.ONCE, days);
		
		days = new HashSet<Integer>();
		days.add(Calendar.SUNDAY);
		DAYS.put(Frequency.SUNDAY, days);
		
		days = new HashSet<Integer>();
		days.add(Calendar.MONDAY);
		DAYS.put(Frequency.MONDAY, days);
		
		days = new HashSet<Integer>();
		days.add(Calendar.TUESDAY);
		DAYS.put(Frequency.TUESDAY, days);
		
		days = new HashSet<Integer>();
		days.add(Calendar.WEDNESDAY);
		DAYS.put(Frequency.WEDNESDAY, days);
		
		days = new HashSet<Integer>();
		days.add(Calendar.THURSDAY);
		DAYS.put(Frequency.THURSDAY, days);
		
		days = new HashSet<Integer>();
		days.add(Calendar.FRIDAY);
		DAYS.put(Frequency.FRIDAY, days);
		
		days = new HashSet<Integer>();
		days.add(Calendar.SATURDAY);
		DAYS.put(Frequency.SATURDAY, days);
		
	}

	/**
	 * The String represents the label of the frequency.
	 */
	private String label;

	/**
	 * <p>
	 * Create an instance of Frequency.
	 * </p>
	 */
	private Frequency(String label) {
		this.label = label;
	}

	public String getLabel() {
		return label;
	}

}
