/*
 * Created on May 1, 2008
 */
package br.pucrio.olis.domain;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.Transient;

import br.pucrio.olis.business.BusinessException;

/**
 * <p>
 * This class represents a travel event, which has some travel specific
 * properties, besides the common properties of the Event class.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
@Entity
public class TravelEvent extends Event {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 4699031969652868891L;

	/**
	 * The Set represent the set of activities that can be done in this travel
	 * event.
	 */
	private Set<ActivityType> activities;
	/**
	 * The PlaceType represent the type of the place of this travel event.
	 */
	private PlaceType placeType;

	/**
	 * <p>
	 * Create an instance of TravelEvent.
	 * </p>
	 */
	public TravelEvent() {
		this.activities = new HashSet<ActivityType>();
	}

	/**
	 * <p>
	 * Add an activity type.
	 * </p>
	 * 
	 * @param activityType
	 *            the activity type to be added.
	 */
	public void addActivities(ActivityType activityType) {
		this.activities.add(activityType);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.domain.Event#checkBusinessRules()
	 */
	@Override
	public void checkBusinessRules() throws BusinessException {
		super.checkBusinessRules();
		if (placeType == null) {
			throw new BusinessException("errors.required", "placeType");
		}
	}

	@ManyToMany(fetch = FetchType.EAGER)
	public Set<ActivityType> getActivities() {
		return activities;
	}
	
	@Transient
	public Iterator<ActivityType> getAllActivities() {
		return activities.iterator();
	}

	@ManyToOne
	public PlaceType getPlaceType() {
		return placeType;
	}

	/**
	 * <p>
	 * Remove an area of interest.
	 * </p>
	 * 
	 * @param activityType
	 *            the activity type to be removed.
	 */
	public boolean removeActivities(ActivityType activityType) {
		return this.activities.remove(activityType);
	}

	public void setActivities(Set<ActivityType> activities) {
		this.activities = activities;
	}

	public void setPlaceType(PlaceType placeType) {
		this.placeType = placeType;
	}

}
