/*
 * Created on May 1, 2008
 */
package br.pucrio.olis.domain;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.ManyToMany;
import javax.persistence.Transient;

/**
 * <p>
 * This class represents the user travel preferences.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
@Entity
public class TravelPreferences extends UserPreferences {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -3153081061051364176L;

	/**
	 * The Set represents the set of activity types that a user is interested
	 * to.
	 */
	private Set<ActivityType> activities;
	/**
	 * The Set represents the set of place types that a user is interested to.
	 */
	private Set<PlaceType> placeTypes;

	/**
	 * <p>
	 * Create an instance of TravelPreferences.
	 * </p>
	 */
	public TravelPreferences() {
		this.activities = new HashSet<ActivityType>();
		this.placeTypes = new HashSet<PlaceType>();
	}

	/**
	 * <p>
	 * Add an activity type.
	 * </p>
	 * 
	 * @param activityType
	 *            the activity type to be added.
	 */
	public void addActivities(ActivityType activityType) {
		this.activities.add(activityType);
	}

	/**
	 * <p>
	 * Add a place type.
	 * </p>
	 * 
	 * @param placeType
	 *            the place type to be added.
	 */
	public void addPlaceTypes(PlaceType placeType) {
		this.placeTypes.add(placeType);
	}

	@ManyToMany(fetch = FetchType.EAGER)
	public Set<ActivityType> getActivities() {
		return activities;
	}
	
	@Transient
	public Iterator<ActivityType> getAllActivities() {
		return activities.iterator();
	}

	@Transient
	public Iterator<PlaceType> getAllPlaceTypes() {
		return placeTypes.iterator();
	}
	
	@ManyToMany(fetch = FetchType.EAGER)
	public Set<PlaceType> getPlaceTypes() {
		return placeTypes;
	}

	/**
	 * <p>
	 * Remove an activity type.
	 * </p>
	 * 
	 * @param activityType
	 *            the activity type to be removed.
	 */
	public boolean removeActivities(ActivityType activityType) {
		return this.activities.remove(activityType);
	}

	/**
	 * <p>
	 * Remove a place type.
	 * </p>
	 * 
	 * @param placeType
	 *            the place type to be removed.
	 */
	public boolean removePlaceTypes(PlaceType placeType) {
		return this.placeTypes.remove(placeType);
	}

	public void setActivities(Set<ActivityType> activities) {
		this.activities = activities;
	}

	public void setPlaceTypes(Set<PlaceType> placeTypes) {
		this.placeTypes = placeTypes;
	}

}
