/*
 * Created on May 1, 2008
 */
package br.pucrio.olis.domain;

import jade.content.Concept;

import java.io.Serializable;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.ManyToOne;

/**
 * <p>
 * This class represents the generic user preferences. There are specific
 * classes for the preferences of each event type.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
@Entity
@Inheritance(strategy = InheritanceType.JOINED)
public abstract class UserPreferences implements Serializable, Concept {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 7268483770732419582L;

	/**
	 * The Long represents the user preferences unique identifier.
	 */
	protected Long id;
	/**
	 * The User represents the user which these preferences belongs to.
	 */
	protected User user;

	/**
	 * <p>
	 * Create a new instance of UserPreferences.
	 * </p>
	 */
	public UserPreferences() {

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof UserPreferences))
			return false;
		return id.equals(((UserPreferences) obj).getId());
	}

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	public Long getId() {
		return id;
	}

	@ManyToOne
	public User getUser() {
		return user;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		return id.hashCode();
	}

	public void setId(Long id) {
		this.id = id;
	}

	public void setUser(User user) {
		this.user = user;
	}

}
