/*
 * Created on May 1, 2008
 */
package br.pucrio.olis.domain;

import jade.content.Concept;

import java.io.Serializable;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.ManyToOne;
import javax.persistence.Transient;

/**
 * <p>
 * This class represents a generic user service. The OLIS system provides
 * different services for the user.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
@Entity
@Inheritance(strategy = InheritanceType.JOINED)
public abstract class UserService implements Serializable, Cloneable, Concept {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -5729451473047574880L;

	/**
	 * The Long represents the user service unique identifier.
	 */
	protected Long id;
	/**
	 * The User represents the user which this service belongs to.
	 */
	protected User user;

	/**
	 * <p>
	 * Create a new instance of UserService.
	 * </p>
	 */
	public UserService() {

	}

	public abstract UserService clone(User user);

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof UserService))
			return false;
		return id.equals(((UserService) obj).getId());
	}

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	public Long getId() {
		return id;
	}

	/**
	 * <p>
	 * Return the type of the service.
	 * </p>
	 * 
	 * @return the service type.
	 */
	@Transient
	public abstract UserServiceType getServiceType();

	@ManyToOne
	public User getUser() {
		return user;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		return id.hashCode();
	}

	public void setId(Long id) {
		this.id = id;
	}

	public void setUser(User user) {
		this.user = user;
	}

}
