/*
 * Created on Jun 1, 2008
 */
package br.pucrio.olis.domain;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

/**
 * <p>
 * This class represents the type of the user service. The OLIS system provides
 * different service types for the user.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class UserServiceType implements Serializable,
		Comparable<UserServiceType> {

	public class MenuOption {
		private String action;
		private String label;

		public MenuOption(String action) {
			this(action, action);
		}

		public MenuOption(String action, String label) {
			this.action = action;
			this.label = label;
		}

		public String getAction() {
			return action;
		}

		public String getLabel() {
			return label;
		}

		public void setAction(String action) {
			this.action = action;
		}

		public void setLabel(String label) {
			this.label = label;
		}
	}

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -2225623936335147615L;

	private static List<UserServiceType> services = new ArrayList<UserServiceType>();

	public static void addUserServiceType(UserServiceType userServiceType) {
		services.add(userServiceType);
		Collections.sort(services);
	}

	public static Iterator<UserServiceType> getServices() {
		return services.iterator();
	}

	public static void removeUserServiceType(UserServiceType userServiceType) {
		services.remove(userServiceType);
	}

	/**
	 * The UserService represents the user service instance with the default
	 * fields set.
	 */
	private UserService defaultInstance;

	/**
	 * The String represents the label of the service type.
	 */
	private String serviceLabel;
	/**
	 * The List represent the menu options of the the service type.
	 */
	private List<MenuOption> serviceMenuOptions;
	/**
	 * The String represents the name space of the service type.
	 */
	private String serviceNameSpace;

	/**
	 * <p>
	 * Create a new instance of UserServiceType.
	 * </p>
	 */
	public UserServiceType() {
		this.serviceMenuOptions = new ArrayList<MenuOption>();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(UserServiceType o) {
		return serviceNameSpace.compareTo(o.serviceNameSpace);
	}

	public UserService createUserService(User user) {
		return defaultInstance.clone(user);
	}

	public UserService getDefaultInstance() {
		return defaultInstance;
	}

	public String getServiceLabel() {
		return serviceLabel;
	}

	public List<MenuOption> getServiceMenuOptions() {
		return serviceMenuOptions;
	}

	public String getServiceNameSpace() {
		return serviceNameSpace;
	}

	public void setDefaultInstance(UserService defaultInstance) {
		this.defaultInstance = defaultInstance;
	}

	public void setServiceLabel(String serviceLabel) {
		this.serviceLabel = serviceLabel;
	}

	public void setServiceMenuOptions(List<MenuOption> serviceMenuOptions) {
		this.serviceMenuOptions = serviceMenuOptions;
	}

	public void setServiceNameSpace(String serviceNameSpace) {
		this.serviceNameSpace = serviceNameSpace;
	}

}
