/*
 * Created on May 1, 2008
 */
package br.pucrio.olis.domain;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.ManyToOne;
import javax.persistence.Transient;

/**
 * <p>
 * This class represents the weather user service. It keeps the information of
 * the service, besides the user specific configuration of the service.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
@Entity
public class WeatherUserService extends UserService {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 4640516507336757335L;

	public static final UserServiceType SERVICE_TYPE;

	static {
		SERVICE_TYPE = new UserServiceType();
		SERVICE_TYPE.setServiceLabel("service.weather");
		List<UserServiceType.MenuOption> menuOptions = new ArrayList<UserServiceType.MenuOption>();
		menuOptions.add(SERVICE_TYPE.new MenuOption("Weather_input",
				"message.weather"));
		SERVICE_TYPE.setServiceMenuOptions(menuOptions);
		SERVICE_TYPE.setServiceNameSpace("weather");

		WeatherUserService weatherUserService = new WeatherUserService();
		weatherUserService.setDegreeUnit(DegreeUnit.CELSIUS);
		SERVICE_TYPE.setDefaultInstance(weatherUserService);

		UserServiceType.addUserServiceType(SERVICE_TYPE);
	}

	/**
	 * The City represents the default city to be displayed to the user.
	 */
	private City defaultCity;

	/**
	 * The DegreeUnit represents the temperature unit that the temperature will
	 * be displayed to the user.
	 */
	private DegreeUnit degreeUnit;

	/**
	 * <p>
	 * Create an instance of WeatherUserService.
	 * </p>
	 */
	public WeatherUserService() {

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.domain.UserService#clone(br.pucrio.olis.domain.User)
	 */
	@Override
	public UserService clone(User user) {
		WeatherUserService weatherUserService = new WeatherUserService();
		weatherUserService.degreeUnit = this.degreeUnit;
		weatherUserService.defaultCity = user.getCity();
		return weatherUserService;
	}

	@ManyToOne
	public City getDefaultCity() {
		return defaultCity;
	}

	@Enumerated(EnumType.STRING)
	@Column(nullable = false)
	public DegreeUnit getDegreeUnit() {
		return degreeUnit;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.domain.UserService#getServiceType()
	 */
	@Override
	@Transient
	public UserServiceType getServiceType() {
		return SERVICE_TYPE;
	}

	public void setDefaultCity(City defaultCity) {
		this.defaultCity = defaultCity;
	}

	public void setDegreeUnit(DegreeUnit degreeUnit) {
		this.degreeUnit = degreeUnit;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.domain.UserService#setUser(br.pucrio.olis.domain.User)
	 */
	@Override
	public void setUser(User user) {
		this.defaultCity = user.getCity();
		super.setUser(user);
	}

}
