/*
 * Created on Jun 20, 2008
 */
package br.pucrio.olis.domain.weather;

import jade.content.Concept;

import java.io.Serializable;
import java.text.ParseException;
import java.util.Date;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author Ingrid Nunes
 * 
 */
public class CurrentConditions implements Serializable, Concept {

	private static final long serialVersionUID = -7228565143723694164L;

	private Barometric barometric;
	private Date dateOfObservation;
	private String description;
	private Integer dewPoint;
	private Integer feelLikeTemperature;
	private Integer humidity;
	private Integer icon;
	private Moon moon;
	private String observationStation;
	private Integer temperature;
	private UV uv;
	private Double visibility;
	private Wind wind;

	public CurrentConditions() {
	}

	public CurrentConditions(Node node) throws ParseException {
		NodeList list = node.getChildNodes();
		for (int i = 0; i < list.getLength(); i++) {
			Node child = list.item(i);
			if (child.getNodeName().equals("lsup")) {
				String lsup = child.getTextContent().replace(" Local Time", "");
				this.dateOfObservation = Weather.DATE_FORMAT_LONG.parse(lsup);
			} else if (child.getNodeName().equals("obst")) {
				this.observationStation = child.getTextContent();
			} else if (child.getNodeName().equals("tmp")) {
				try {
					this.temperature = Integer.parseInt(child.getTextContent());
				} catch (NumberFormatException e) {
					this.temperature = null;
				}
			} else if (child.getNodeName().equals("flik")) {
				try {
					this.feelLikeTemperature = Integer.parseInt(child
							.getTextContent());
				} catch (NumberFormatException e) {
					this.feelLikeTemperature = null;
				}
			} else if (child.getNodeName().equals("t")) {
				this.description = child.getTextContent();
			} else if (child.getNodeName().equals("icon")) {
				try {
					this.icon = Integer.parseInt(child.getTextContent());
				} catch (NumberFormatException e) {
					this.icon = null;
				}
			} else if (child.getNodeName().equals("bar")) {
				this.barometric = new Barometric(child);
			} else if (child.getNodeName().equals("wind")) {
				this.wind = new Wind(child);
			} else if (child.getNodeName().equals("hmid")) {
				try {
					this.humidity = Integer.parseInt(child.getTextContent());
				} catch (NumberFormatException e) {
					this.humidity = null;
				}
			} else if (child.getNodeName().equals("vis")) {
				try {
					this.visibility = Double
							.parseDouble(child.getTextContent());
				} catch (NumberFormatException e) {
					this.visibility = null;
				}
			} else if (child.getNodeName().equals("uv")) {
				this.uv = new UV(child);
			} else if (child.getNodeName().equals("dewp")) {
				try {
					this.dewPoint = Integer.parseInt(child.getTextContent());
				} catch (NumberFormatException e) {
					this.dewPoint = null;
				}
			} else if (child.getNodeName().equals("moon")) {
				this.moon = new Moon(child);
			}
		}
	}

	public Barometric getBarometric() {
		return barometric;
	}

	public Date getDateOfObservation() {
		return dateOfObservation;
	}

	public String getDescription() {
		return description;
	}

	public Integer getDewPoint() {
		return dewPoint;
	}

	public Integer getFeelLikeTemperature() {
		return feelLikeTemperature;
	}

	public Integer getHumidity() {
		return humidity;
	}

	public Integer getIcon() {
		return icon;
	}

	public Moon getMoon() {
		return moon;
	}

	public String getObservationStation() {
		return observationStation;
	}

	public Integer getTemperature() {
		return temperature;
	}

	public UV getUv() {
		return uv;
	}

	public Double getVisibility() {
		return visibility;
	}

	public Wind getWind() {
		return wind;
	}

	public void setBarometric(Barometric barometric) {
		this.barometric = barometric;
	}

	public void setDateOfObservation(Date dateOfObservation) {
		this.dateOfObservation = dateOfObservation;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public void setDewPoint(Integer dewPoint) {
		this.dewPoint = dewPoint;
	}

	public void setFeelLikeTemperature(Integer feelLikeTemperature) {
		this.feelLikeTemperature = feelLikeTemperature;
	}

	public void setHumidity(Integer humidity) {
		this.humidity = humidity;
	}

	public void setIcon(Integer icon) {
		this.icon = icon;
	}

	public void setMoon(Moon moon) {
		this.moon = moon;
	}

	public void setObservationStation(String observationStation) {
		this.observationStation = observationStation;
	}

	public void setTemperature(Integer temperature) {
		this.temperature = temperature;
	}

	public void setUv(UV uv) {
		this.uv = uv;
	}

	public void setVisibility(Double visibility) {
		this.visibility = visibility;
	}

	public void setWind(Wind wind) {
		this.wind = wind;
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("dateOfObservation: ").append(
				Weather.DATE_FORMAT_LONG.format(dateOfObservation))
				.append("\n");
		sb.append("observationStation: ").append(observationStation).append(
				"\n");
		sb.append("temperature: ").append(temperature).append("\n");
		sb.append("feelLikeTemperature: ").append(feelLikeTemperature).append(
				"\n");
		sb.append("description: ").append(description).append("\n");
		sb.append("icon: ").append(icon).append("\n");
		sb.append("barometric:\n").append(barometric).append("\n");
		sb.append("wind:\n").append(wind).append("\n");
		sb.append("humidity: ").append(humidity).append("\n");
		sb.append("visibility: ").append(visibility).append("\n");
		sb.append("uv:\n").append(uv).append("\n");
		sb.append("dewPoint: ").append(dewPoint).append("\n");
		sb.append("moon:\n").append(moon);
		return sb.toString();
	}

}
