/*
 * Created on Jun 20, 2008
 */
package br.pucrio.olis.domain.weather;

import jade.content.Concept;

import java.io.Serializable;
import java.text.ParseException;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author Ingrid Nunes
 * 
 */
public class DayForecast implements Serializable, Concept {

	private static final long serialVersionUID = -6212665892865059472L;

	private DayPart day;
	private Integer dayNumber;
	private Date dayOfMonth;
	private Integer highTemperature;
	private Integer lowTemperature;
	private DayPart night;
	private Date sunrise;
	private Date sunset;

	public DayForecast() {
	}

	public DayForecast(Node node) throws ParseException {
		try {
			this.dayNumber = Integer.parseInt(node.getAttributes()
					.getNamedItem("d").getTextContent());
		} catch (NumberFormatException e) {
			this.dayNumber = null;
		}
		Calendar calendar = new GregorianCalendar();
		calendar.setTime(new Date());
		String date = node.getAttributes().getNamedItem("dt").getTextContent()
				+ " " + calendar.get(Calendar.YEAR);
		this.dayOfMonth = Weather.DATE_FORMAT.parse(date);

		NodeList list = node.getChildNodes();
		for (int i = 0; i < list.getLength(); i++) {
			Node child = list.item(i);
			if (child.getNodeName().equals("hi")) {
				try {
					this.highTemperature = Integer.parseInt(child
							.getTextContent());
				} catch (NumberFormatException e) {
					this.highTemperature = null;
				}
			} else if (child.getNodeName().equals("low")) {
				try {
					this.lowTemperature = Integer.parseInt(child
							.getTextContent());
				} catch (NumberFormatException e) {
					this.lowTemperature = null;
				}
			} else if (child.getNodeName().equals("sunr")) {
				this.sunrise = Weather.TIME_FORMAT
						.parse(child.getTextContent());
			} else if (child.getNodeName().equals("suns")) {
				this.sunset = Weather.TIME_FORMAT.parse(child.getTextContent());
			} else if (child.getNodeName().equals("part")) {
				if (child.getAttributes().getNamedItem("p").getTextContent()
						.equals("d")) {
					this.day = new DayPart(child);
				} else {
					this.night = new DayPart(child);
				}
			}
		}
	}

	public DayPart getDay() {
		return day;
	}

	public Integer getDayNumber() {
		return dayNumber;
	}

	public Date getDayOfMonth() {
		return dayOfMonth;
	}

	public Integer getHighTemperature() {
		return highTemperature;
	}

	public Integer getLowTemperature() {
		return lowTemperature;
	}

	public DayPart getNight() {
		return night;
	}

	public Date getSunrise() {
		return sunrise;
	}

	public Date getSunset() {
		return sunset;
	}

	public void setDay(DayPart day) {
		this.day = day;
	}

	public void setDayNumber(Integer dayNumber) {
		this.dayNumber = dayNumber;
	}

	public void setDayOfMonth(Date dayOfMonth) {
		this.dayOfMonth = dayOfMonth;
	}

	public void setHighTemperature(Integer highTemperature) {
		this.highTemperature = highTemperature;
	}

	public void setLowTemperature(Integer lowTemperature) {
		this.lowTemperature = lowTemperature;
	}

	public void setNight(DayPart night) {
		this.night = night;
	}

	public void setSunrise(Date sunrise) {
		this.sunrise = sunrise;
	}

	public void setSunset(Date sunset) {
		this.sunset = sunset;
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("* dayNumber: ").append(dayNumber).append("\n");
		sb.append("dayOfMonth: ")
				.append(Weather.DATE_FORMAT.format(dayOfMonth)).append("\n");
		sb.append("highTemperature: ").append(highTemperature).append("\n");
		sb.append("lowTemperature: ").append(lowTemperature).append("\n");
		sb.append("sunrise: ").append(Weather.TIME_FORMAT.format(sunrise))
				.append("\n");
		sb.append("sunset: ").append(Weather.TIME_FORMAT.format(sunset))
				.append("\n");
		sb.append("day:\n").append(day);
		sb.append("\nnight:\n").append(night);
		return sb.toString();
	}

}
