/*
 * Created on 20/06/2008
 */
package br.pucrio.olis.domain.weather;

import jade.content.Concept;

import java.io.Serializable;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author ingridnunes
 * @version 1.0
 * 
 */
public class DayPart implements Serializable, Concept {

	public enum DayPartType {
		DAY, NIGHT
	};

	private static final long serialVersionUID = -5585034667827385733L;

	private Integer chanceOfPrecipitation;
	private String description;
	private String descriptionShort;
	private Integer humidity;
	private Integer icon;
	private DayPartType type;
	private Wind wind;

	public DayPart() {
	}

	public DayPart(Node node) {
		this.type = node.getAttributes().getNamedItem("p").getTextContent()
				.equals("d") ? DayPartType.DAY : DayPartType.NIGHT;
		NodeList list = node.getChildNodes();
		for (int i = 0; i < list.getLength(); i++) {
			Node child = list.item(i);
			if (child.getNodeName().equals("icon")) {
				try {
					this.icon = Integer.parseInt(child.getTextContent());
				} catch (NumberFormatException e) {
					this.icon = null;
				}
			} else if (child.getNodeName().equals("t")) {
				this.description = child.getTextContent();
			} else if (child.getNodeName().equals("wind")) {
				this.wind = new Wind(child);
			} else if (child.getNodeName().equals("bt")) {
				this.descriptionShort = child.getTextContent();
			} else if (child.getNodeName().equals("ppcp")) {
				try {
					this.chanceOfPrecipitation = Integer.parseInt(child
							.getTextContent());
				} catch (NumberFormatException e) {
					this.chanceOfPrecipitation = null;
				}
			} else if (child.getNodeName().equals("hmid")) {
				try {
					this.humidity = Integer.parseInt(child.getTextContent());
				} catch (NumberFormatException e) {
					this.humidity = null;
				}
			}
		}
	}

	public Integer getChanceOfPrecipitation() {
		return chanceOfPrecipitation;
	}

	public String getDescription() {
		return description;
	}

	public String getDescriptionShort() {
		return descriptionShort;
	}

	public Integer getHumidity() {
		return humidity;
	}

	public Integer getIcon() {
		return icon;
	}

	public DayPartType getType() {
		return type;
	}

	public Wind getWind() {
		return wind;
	}

	public void setChanceOfPrecipitation(Integer chanceOfPrecipitation) {
		this.chanceOfPrecipitation = chanceOfPrecipitation;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public void setDescriptionShort(String descriptionShort) {
		this.descriptionShort = descriptionShort;
	}

	public void setHumidity(Integer humidity) {
		this.humidity = humidity;
	}

	public void setIcon(Integer icon) {
		this.icon = icon;
	}

	public void setType(DayPartType type) {
		this.type = type;
	}

	public void setWind(Wind wind) {
		this.wind = wind;
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("type: ").append(type).append("\n");
		sb.append("icon: ").append(icon).append("\n");
		sb.append("description: ").append(description).append("\n");
		sb.append("wind:\n").append(wind).append("\n");
		sb.append("descriptionShort: ").append(descriptionShort).append("\n");
		sb.append("chanceOfPrecipitation: ").append(chanceOfPrecipitation)
				.append("\n");
		sb.append("humidity: ").append(humidity);
		return sb.toString();
	}

}
