/*
 * Created on Jun 20, 2008
 */
package br.pucrio.olis.domain.weather;

import jade.content.Concept;

import java.io.Serializable;
import java.text.ParseException;
import java.util.Date;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author Ingrid Nunes
 * 
 */
public class Location implements Serializable, Concept {

	private static final long serialVersionUID = 5509560272296837334L;

	private String id;
	private Double latitude;
	private Double longitude;
	private String name;
	private Date sunrise;
	private Date sunset;
	private Date time;
	private Integer zone;

	public Location() {
	}

	public Location(Node node) throws ParseException {
		this.id = node.getAttributes().getNamedItem("id").getTextContent();
		NodeList list = node.getChildNodes();
		for (int i = 0; i < list.getLength(); i++) {
			Node child = list.item(i);
			if (child.getNodeName().equals("dnam")) {
				this.name = child.getTextContent();
			} else if (child.getNodeName().equals("tm")) {
				this.time = Weather.TIME_FORMAT.parse(child.getTextContent());
			} else if (child.getNodeName().equals("lat")) {
				try {
					this.latitude = Double.parseDouble(child.getTextContent());
				} catch (NumberFormatException e) {
					this.latitude = null;
				}
			} else if (child.getNodeName().equals("lon")) {
				try {
					this.longitude = Double.parseDouble(child.getTextContent());
				} catch (NumberFormatException e) {
					this.longitude = null;
				}
			} else if (child.getNodeName().equals("sunr")) {
				this.sunrise = Weather.TIME_FORMAT
						.parse(child.getTextContent());
			} else if (child.getNodeName().equals("suns")) {
				this.sunset = Weather.TIME_FORMAT.parse(child.getTextContent());
			} else if (child.getNodeName().equals("zone")) {
				try {
					this.zone = Integer.parseInt(child.getTextContent());
				} catch (NumberFormatException e) {
					this.zone = null;
				}
			}
		}
	}

	public String getId() {
		return id;
	}

	public Double getLatitude() {
		return latitude;
	}

	public Double getLongitude() {
		return longitude;
	}

	public String getName() {
		return name;
	}

	public Date getSunrise() {
		return sunrise;
	}

	public Date getSunset() {
		return sunset;
	}

	public Date getTime() {
		return time;
	}

	public Integer getZone() {
		return zone;
	}

	public void setId(String id) {
		this.id = id;
	}

	public void setLatitude(Double latitude) {
		this.latitude = latitude;
	}

	public void setLongitude(Double longitude) {
		this.longitude = longitude;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void setSunrise(Date sunrise) {
		this.sunrise = sunrise;
	}

	public void setSunset(Date sunset) {
		this.sunset = sunset;
	}

	public void setTime(Date time) {
		this.time = time;
	}

	public void setZone(Integer zone) {
		this.zone = zone;
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("[").append(id).append("]").append(name).append("\n");
		sb.append("time: ").append(Weather.TIME_FORMAT.format(time)).append(
				"\n");
		sb.append("latitude: ").append(latitude).append("\n");
		sb.append("longitude: ").append(longitude).append("\n");
		sb.append("sunrise: ").append(Weather.TIME_FORMAT.format(sunrise))
				.append("\n");
		sb.append("sunset: ").append(Weather.TIME_FORMAT.format(sunset))
				.append("\n");
		sb.append("zone: ").append(zone);
		return sb.toString();
	}

}
