/*
 * Created on Jun 22, 2008
 */
package br.pucrio.olis.ontology;

import jade.content.onto.BasicOntology;
import jade.content.onto.Ontology;
import jade.content.onto.OntologyException;
import jade.content.schema.ConceptSchema;
import jade.content.schema.ObjectSchema;
import jade.content.schema.PrimitiveSchema;
import br.pucrio.olis.domain.AcademicEvent;
import br.pucrio.olis.domain.AcademicEventType;
import br.pucrio.olis.domain.AcademicPreferences;
import br.pucrio.olis.domain.AreaOfInterest;
import br.pucrio.olis.domain.Deadline;
import br.pucrio.olis.domain.DeadlineDate;

/**
 * @author Ingrid Nunes
 *
 */
public class OLISAcademicOntology extends Ontology implements OLISAcademicVocabulary {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 6113260660006024268L;

	public static final String ONTOLOGY_NAME = "OLIS";
	private static Ontology theInstance = new OLISAcademicOntology();

	/**
	 * This method grants access to the unique instance of the ontology.
	 * 
	 * @return An <code>Ontology</code> object, containing the concepts of the
	 *         ontology.
	 */
	public static Ontology getInstance() {
		return theInstance;
	}

	public OLISAcademicOntology() {
		super(ONTOLOGY_NAME, new Ontology[] { OLISBasicOntology.getInstance() },
				new OLISIntrospector());

		try {
			add(new EnumerationSchema(ACADEMICEVENTTYPE), AcademicEventType.class);
			add(new EnumerationSchema(DEADLINE), Deadline.class);
			
			add(new ConceptSchema(ACADEMICEVENT), AcademicEvent.class);
			add(new ConceptSchema(ACADEMICPREFERENCES), AcademicPreferences.class);
			add(new ConceptSchema(AREAOFINTEREST), AreaOfInterest.class);
			add(new ConceptSchema(DEADLINEDATE), DeadlineDate.class);
			
			ConceptSchema cs = (ConceptSchema) getSchema(ACADEMICEVENT);
			cs.add(ACADEMICEVENT_AREASOFINTEREST, (ConceptSchema) getSchema(AREAOFINTEREST), 0, ObjectSchema.UNLIMITED, BasicOntology.SET);
			cs.add(ACADEMICEVENT_DEADLINES, (ConceptSchema) getSchema(DEADLINEDATE), 0, ObjectSchema.UNLIMITED, BasicOntology.SEQUENCE);
			cs.add(ACADEMICEVENT_EVENTTYPE, (PrimitiveSchema) getSchema(ACADEMICEVENTTYPE));
			cs.addSuperSchema((ConceptSchema) getSchema(OLISBasicOntology.EVENT));
			
			cs = (ConceptSchema) getSchema(ACADEMICPREFERENCES);
			cs.add(ACADEMICPREFERENCES_AREASOFINTEREST, (ConceptSchema) getSchema(AREAOFINTEREST), 0, ObjectSchema.UNLIMITED, BasicOntology.SET);
			cs.add(ACADEMICPREFERENCES_EVENTTYPE, (PrimitiveSchema) getSchema(ACADEMICEVENTTYPE), 0, ObjectSchema.UNLIMITED, BasicOntology.SET);
			cs.add(ACADEMICPREFERENCES_FOREIGNEVENTS, (PrimitiveSchema) getSchema(BasicOntology.BOOLEAN));
			cs.addSuperSchema((ConceptSchema) getSchema(OLISBasicOntology.USERPREFERENCES));
			
			cs = (ConceptSchema) getSchema(AREAOFINTEREST);
			cs.add(AREAOFINTEREST_ID, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
			cs.add(AREAOFINTEREST_LABEL, (PrimitiveSchema) getSchema(BasicOntology.STRING));
			cs.addSuperSchema((ConceptSchema) getSchema(OLISBasicOntology.OBJECT));
			
			cs = (ConceptSchema) getSchema(DEADLINEDATE);
			//cs.add(DEADLINEDATE_ACADEMICEVENT, (ConceptSchema) getSchema(ACADEMICEVENT));
			cs.add(DEADLINEDATE_DATE, (PrimitiveSchema)getSchema(BasicOntology.DATE), ObjectSchema.OPTIONAL);
			cs.add(DEADLINEDATE_DEADLINE, (PrimitiveSchema) getSchema(DEADLINE));
			cs.add(DEADLINEDATE_ID, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
			cs.addSuperSchema((ConceptSchema) getSchema(OLISBasicOntology.OBJECT));
		} catch (OntologyException oe) {
			oe.printStackTrace();
		}
	}

}
