/*
 * Created on Jun 22, 2008
 */
package br.pucrio.olis.ontology;

import jade.content.onto.BasicOntology;
import jade.content.onto.Ontology;
import jade.content.onto.OntologyException;
import jade.content.schema.ConceptSchema;
import jade.content.schema.ObjectSchema;
import jade.content.schema.PredicateSchema;
import jade.content.schema.PrimitiveSchema;
import br.pucrio.olis.business.BusinessOperation;
import br.pucrio.olis.business.BusinessOperationType;
import br.pucrio.olis.domain.CalendarEvent;
import br.pucrio.olis.domain.CalendarUserService;
import br.pucrio.olis.domain.City;
import br.pucrio.olis.domain.Country;
import br.pucrio.olis.domain.DateFormat;
import br.pucrio.olis.domain.Event;
import br.pucrio.olis.domain.EventsAnnouncementUserService;
import br.pucrio.olis.domain.EventsBoard;
import br.pucrio.olis.domain.Frequency;
import br.pucrio.olis.domain.Notification;
import br.pucrio.olis.domain.TimeFormat;
import br.pucrio.olis.domain.User;
import br.pucrio.olis.domain.UserPreferences;
import br.pucrio.olis.domain.UserService;

/**
 * @author Ingrid Nunes
 *
 */
public class OLISBasicOntology extends Ontology implements OLISBasicVocabulary {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 6113260660006024268L;

	public static final String ONTOLOGY_NAME = "OLISBasicOntology";
	private static Ontology theInstance = new OLISBasicOntology();

	/**
	 * This method grants access to the unique instance of the ontology.
	 * 
	 * @return An <code>Ontology</code> object, containing the concepts of the
	 *         ontology.
	 */
	public static Ontology getInstance() {
		return theInstance;
	}

	public OLISBasicOntology() {
		super(ONTOLOGY_NAME, new Ontology[] { BasicOntology.getInstance() },
				new OLISIntrospector());

		try {
			add(new ConceptSchema(OBJECT), Object.class);
			
			add(new EnumerationSchema(DATEFORMAT), DateFormat.class);
			add(new EnumerationSchema(FREQUENCY), Frequency.class);
			add(new EnumerationSchema(BUSINESSOPERATIONTYPE), BusinessOperationType.class);
			add(new EnumerationSchema(TIMEFORMAT), TimeFormat.class);
			
			add(new ConceptSchema(CALENDAREVENT), CalendarEvent.class);
			add(new ConceptSchema(CALENDARUSERSERVICE), CalendarUserService.class);
			add(new ConceptSchema(CITY), City.class);
			add(new ConceptSchema(COUNTRY), Country.class);
			add(new ConceptSchema(EVENT), Event.class);
			add(new ConceptSchema(EVENTSANNOUNCEMENTUSERSERVICE), EventsAnnouncementUserService.class);
			add(new ConceptSchema(EVENTSBOARD), EventsBoard.class);
			add(new ConceptSchema(NOTIFICATION), Notification.class);
			add(new ConceptSchema(USER), User.class);
			add(new ConceptSchema(USERPREFERENCES), UserPreferences.class);
			add(new ConceptSchema(USERSERVICE), UserService.class);
			
			ConceptSchema cs = (ConceptSchema) getSchema(CALENDAREVENT);
			cs.add(CALENDAREVENT_AVAILABLE, (PrimitiveSchema) getSchema(BasicOntology.BOOLEAN));
			cs.add(CALENDAREVENT_EVENT, (ConceptSchema) getSchema(EVENT));
			cs.add(CALENDAREVENT_ID, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
			cs.add(CALENDAREVENT_PARTICIPANTS, (ConceptSchema) getSchema(USER), 0, ObjectSchema.UNLIMITED, BasicOntology.SET);
			cs.addSuperSchema((ConceptSchema) getSchema(OBJECT));
			
			cs = (ConceptSchema) getSchema(CALENDARUSERSERVICE);
			//cs.add(CALENDARUSERSERVICE_EVENTS, (ConceptSchema) getSchema(CALENDAREVENT), 0, ObjectSchema.UNLIMITED, BasicOntology.SET);
			cs.add(CALENDARUSERSERVICE_REMINDEVENT, (PrimitiveSchema) getSchema(BasicOntology.BOOLEAN));
			cs.add(CALENDARUSERSERVICE_REMINDMINUTESBEFORE, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
			cs.add(CALENDARUSERSERVICE_SHOWWEEKENDS, (PrimitiveSchema) getSchema(BasicOntology.BOOLEAN));
			cs.addSuperSchema((ConceptSchema) getSchema(USERSERVICE));
			
			cs = (ConceptSchema) getSchema(CITY);
			cs.add(CITY_COUNTRY, (ConceptSchema) getSchema(COUNTRY));
			cs.add(CITY_ID, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
			cs.add(CITY_NAME, (PrimitiveSchema) getSchema(BasicOntology.STRING));
			cs.addSuperSchema((ConceptSchema) getSchema(OBJECT));
			
			cs = (ConceptSchema) getSchema(COUNTRY);
			//cs.add(COUNTRY_CITIES, (ConceptSchema) getSchema(CITY), 0, ObjectSchema.UNLIMITED, BasicOntology.SET);
			cs.add(COUNTRY_CODE, (PrimitiveSchema) getSchema(BasicOntology.STRING));
			cs.add(COUNTRY_NAME, (PrimitiveSchema) getSchema(BasicOntology.STRING));
			cs.addSuperSchema((ConceptSchema) getSchema(OBJECT));
			
			cs = (ConceptSchema) getSchema(EVENT);
			cs.add(EVENT_ALLDAY, (PrimitiveSchema) getSchema(BasicOntology.BOOLEAN));
			cs.add(EVENT_CITY, (ConceptSchema) getSchema(CITY));
			cs.add(EVENT_DESCRIPTION, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			cs.add(EVENT_END, (PrimitiveSchema)getSchema(BasicOntology.DATE));
			cs.add(EVENT_ENDTIME, (PrimitiveSchema)getSchema(BasicOntology.DATE), ObjectSchema.OPTIONAL);
			cs.add(EVENT_EVENTSBOARD, (ConceptSchema) getSchema(EVENTSBOARD), ObjectSchema.OPTIONAL);
			cs.add(EVENT_FREQUENCY, (PrimitiveSchema) getSchema(FREQUENCY));
			cs.add(EVENT_ID, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
			cs.add(EVENT_LOCATION, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			cs.add(EVENT_OWNER, (ConceptSchema) getSchema(USER));
			cs.add(EVENT_START, (PrimitiveSchema)getSchema(BasicOntology.DATE));
			cs.add(EVENT_STARTTIME, (PrimitiveSchema)getSchema(BasicOntology.DATE), ObjectSchema.OPTIONAL);
			cs.add(EVENT_SUBJECT, (PrimitiveSchema) getSchema(BasicOntology.STRING));
			cs.addSuperSchema((ConceptSchema) getSchema(OBJECT));
			
			cs = (ConceptSchema) getSchema(EVENTSANNOUNCEMENTUSERSERVICE);
			cs.add(EVENTSANNOUNCEMENTUSERSERVICE_SUGGESTEVENTS, (PrimitiveSchema) getSchema(BasicOntology.BOOLEAN));
			cs.add(EVENTSANNOUNCEMENTUSERSERVICE_WORKTIMEBEGIN, (PrimitiveSchema)getSchema(BasicOntology.DATE), ObjectSchema.OPTIONAL);
			cs.add(EVENTSANNOUNCEMENTUSERSERVICE_WORKTIMEEND, (PrimitiveSchema)getSchema(BasicOntology.DATE), ObjectSchema.OPTIONAL);
			cs.addSuperSchema((ConceptSchema) getSchema(USERSERVICE));
			
			cs = (ConceptSchema) getSchema(EVENTSBOARD);
			//cs.add(EVENTSBOARD_EVENTS, (ConceptSchema) getSchema(EVENT), 0, ObjectSchema.UNLIMITED, BasicOntology.SET);
			cs.add(EVENTSBOARD_ID, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
			cs.addSuperSchema((ConceptSchema) getSchema(OBJECT));
			
			cs = (ConceptSchema) getSchema(NOTIFICATION);
			cs.add(NOTIFICATION_ID, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
			cs.add(NOTIFICATION_DATE, (PrimitiveSchema) getSchema(BasicOntology.DATE));
			cs.addSuperSchema((ConceptSchema) getSchema(OBJECT));
			
			cs = (ConceptSchema) getSchema(USER);
			cs.add(USER_CITY, (ConceptSchema) getSchema(CITY));
			cs.add(USER_DATEFORMAT, (PrimitiveSchema) getSchema(DATEFORMAT));
			cs.add(USER_DATEOFBIRTH, (PrimitiveSchema)getSchema(BasicOntology.DATE));
			cs.add(USER_EMAIL, (PrimitiveSchema) getSchema(BasicOntology.STRING));
			cs.add(USER_FIRSTNAME, (PrimitiveSchema) getSchema(BasicOntology.STRING));
			cs.add(USER_ID, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
			cs.add(USER_LASTNAME, (PrimitiveSchema) getSchema(BasicOntology.STRING));
			//cs.add(USER_NOTIFICATIONS, (ConceptSchema) getSchema(NOTIFICATION), 0, ObjectSchema.UNLIMITED, BasicOntology.SET);
			cs.add(USER_PASSWORD, (PrimitiveSchema) getSchema(BasicOntology.STRING));
			cs.add(USER_PREFERENCES, (ConceptSchema) getSchema(USERPREFERENCES), ObjectSchema.OPTIONAL);
			//cs.add(USER_SERVICES, (ConceptSchema) getSchema(USERSERVICE), 0, ObjectSchema.UNLIMITED, BasicOntology.SET);
			cs.add(USER_TIMEFORMAT, (PrimitiveSchema) getSchema(TIMEFORMAT));
			cs.add(USER_USERNAME, (PrimitiveSchema) getSchema(BasicOntology.STRING));
			cs.addSuperSchema((ConceptSchema) getSchema(OBJECT));
			
			cs = (ConceptSchema) getSchema(USERPREFERENCES);
			cs.add(USERPREFERENCES_ID, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
			//cs.add(USERPREFERENCES_USER, (ConceptSchema) getSchema(USER));
			cs.addSuperSchema((ConceptSchema) getSchema(OBJECT));
			
			cs = (ConceptSchema) getSchema(USERSERVICE);
			cs.add(USERSERVICE_ID, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
			cs.add(USERSERVICE_USER, (ConceptSchema) getSchema(USER));
			cs.addSuperSchema((ConceptSchema) getSchema(OBJECT));

			add(new PredicateSchema(BUSINESSOPERATION), BusinessOperation.class);

			PredicateSchema ps = (PredicateSchema) getSchema(BUSINESSOPERATION);
			ps.add(BUSINESSOPERATION_OPERATIONTYPE, (PrimitiveSchema) getSchema(BUSINESSOPERATIONTYPE));
			ps.add(BUSINESSOPERATION_PARAM, getSchema(OBJECT));
			ps.add(BUSINESSOPERATION_OLDVALUE, getSchema(OBJECT), ObjectSchema.OPTIONAL);
		} catch (OntologyException oe) {
			oe.printStackTrace();
		}
	}

}
