/*
 * Created on Jun 22, 2008
 */
package br.pucrio.olis.ontology;

import br.pucrio.olis.agent.scheduler.AlternativeTimeAnswer;
import br.pucrio.olis.agent.scheduler.AlternativeTimeQuery;
import br.pucrio.olis.agent.scheduler.AvailabilityAnswer;
import br.pucrio.olis.agent.scheduler.AvailabilityQuery;
import br.pucrio.olis.domain.EventSchedulerNotification;
import jade.content.onto.BasicOntology;
import jade.content.onto.Ontology;
import jade.content.onto.OntologyException;
import jade.content.schema.ConceptSchema;
import jade.content.schema.PredicateSchema;
import jade.content.schema.PrimitiveSchema;

/**
 * @author Ingrid Nunes
 * 
 */
public class OLISEventSchedulerOntology extends Ontology implements
		OLISEventSchedulerVocabulary {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 6113260660006024268L;

	public static final String ONTOLOGY_NAME = "OLISEventSchedulerOntology";
	private static Ontology theInstance = new OLISEventSchedulerOntology();

	/**
	 * This method grants access to the unique instance of the ontology.
	 * 
	 * @return An <code>Ontology</code> object, containing the concepts of the
	 *         ontology.
	 */
	public static Ontology getInstance() {
		return theInstance;
	}

	public OLISEventSchedulerOntology() {
		super(ONTOLOGY_NAME, new Ontology[] { OLISBasicOntology.getInstance() },
				new OLISIntrospector());

		try {
			add(new ConceptSchema(EVENTSCHEDULERNOTIFICATION), EventSchedulerNotification.class);
			
			ConceptSchema cs = (ConceptSchema) getSchema(EVENTSCHEDULERNOTIFICATION);
			cs.add(EVENTSCHEDULERNOTIFICATION_CALENDAREVENT, (ConceptSchema) getSchema(OLISBasicOntology.CALENDAREVENT));
			cs.add(EVENTSCHEDULERNOTIFICATION_CONFLICT, (ConceptSchema) getSchema(OLISBasicOntology.USER));
			cs.add(EVENTSCHEDULERNOTIFICATION_SUGGESTEDSTART, (PrimitiveSchema) getSchema(BasicOntology.DATE));
			cs.add(EVENTSCHEDULERNOTIFICATION_SUGGESTEDEND, (PrimitiveSchema) getSchema(BasicOntology.DATE));
			
			add(new PredicateSchema(AVAILABILITYQUERY), AvailabilityQuery.class);
			add(new PredicateSchema(AVAILABILITYANSWER), AvailabilityAnswer.class);
			add(new PredicateSchema(ALTERNATIVETIMEQUERY), AlternativeTimeQuery.class);
			add(new PredicateSchema(ALTERNATIVETIMEANSWER), AlternativeTimeAnswer.class);

			PredicateSchema ps = (PredicateSchema) getSchema(AVAILABILITYQUERY);
			ps.add(AVAILABILITYQUERY_START, (PrimitiveSchema) getSchema(BasicOntology.DATE));
			ps.add(AVAILABILITYQUERY_END, (PrimitiveSchema) getSchema(BasicOntology.DATE));
			
			ps = (PredicateSchema) getSchema(AVAILABILITYANSWER);
			ps.add(AVAILABILITYANSWER_AVAILABLE, (PrimitiveSchema) getSchema(BasicOntology.BOOLEAN));
			
			ps = (PredicateSchema) getSchema(ALTERNATIVETIMEQUERY);
			ps.add(ALTERNATIVETIMEQUERY_START, (PrimitiveSchema) getSchema(BasicOntology.DATE));
			ps.add(ALTERNATIVETIMEQUERY_DURATION, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
			
			ps = (PredicateSchema) getSchema(ALTERNATIVETIMEANSWER);
			ps.add(ALTERNATIVETIMEANSWER_ALTERNATIVETIME, (PrimitiveSchema) getSchema(BasicOntology.DATE));
		} catch (OntologyException oe) {
			oe.printStackTrace();
		}
	}
	
}
