/*
 * Created on Jun 22, 2008
 */
package br.pucrio.olis.ontology;

import jade.content.onto.BasicOntology;
import jade.content.onto.Ontology;
import jade.content.onto.OntologyException;
import jade.content.schema.ConceptSchema;
import jade.content.schema.ObjectSchema;
import jade.content.schema.PrimitiveSchema;
import br.pucrio.olis.domain.ActivityType;
import br.pucrio.olis.domain.PlaceType;
import br.pucrio.olis.domain.TravelEvent;
import br.pucrio.olis.domain.TravelPreferences;

/**
 * @author Ingrid Nunes
 *
 */
public class OLISTravelOntology extends Ontology implements OLISTravelVocabulary {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 6113260660006024268L;

	public static final String ONTOLOGY_NAME = "OLISTravelOntology";
	private static Ontology theInstance = new OLISTravelOntology();

	/**
	 * This method grants access to the unique instance of the ontology.
	 * 
	 * @return An <code>Ontology</code> object, containing the concepts of the
	 *         ontology.
	 */
	public static Ontology getInstance() {
		return theInstance;
	}

	public OLISTravelOntology() {
		super(ONTOLOGY_NAME, new Ontology[] { OLISBasicOntology.getInstance() },
				new OLISIntrospector());

		try {
			add(new ConceptSchema(ACTIVITYTYPE), ActivityType.class);			
			add(new ConceptSchema(PLACETYPE), PlaceType.class);
			add(new ConceptSchema(TRAVELEVENT), TravelEvent.class);
			add(new ConceptSchema(TRAVELPREFERENCES), TravelPreferences.class);
			
			ConceptSchema cs = (ConceptSchema) getSchema(ACTIVITYTYPE);
			cs.add(ACTIVITYTYPE_ID, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
			cs.add(ACTIVITYTYPE_LABEL, (PrimitiveSchema) getSchema(BasicOntology.STRING));
			cs.addSuperSchema((ConceptSchema) getSchema(OLISBasicOntology.OBJECT));
			
			cs = (ConceptSchema) getSchema(PLACETYPE);
			cs.add(PLACETYPE_ID, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
			cs.add(PLACETYPE_LABEL, (PrimitiveSchema) getSchema(BasicOntology.STRING));
			cs.addSuperSchema((ConceptSchema) getSchema(OLISBasicOntology.OBJECT));
			
			cs = (ConceptSchema) getSchema(TRAVELEVENT);
			cs.add(TRAVELEVENT_ACTIVITIES, (ConceptSchema) getSchema(ACTIVITYTYPE), 0, ObjectSchema.UNLIMITED, BasicOntology.SET);
			cs.add(TRAVELEVENT_PLACETYPE, (ConceptSchema) getSchema(PLACETYPE));
			cs.addSuperSchema((ConceptSchema) getSchema(OLISBasicOntology.EVENT));
			
			cs = (ConceptSchema) getSchema(TRAVELPREFERENCES);
			cs.add(TRAVELPREFERENCES_ACTIVITIES, (ConceptSchema) getSchema(ACTIVITYTYPE), 0, ObjectSchema.UNLIMITED, BasicOntology.SET);
			cs.add(TRAVELPREFERENCES_PLACETYPES, (ConceptSchema) getSchema(PLACETYPE), 0, ObjectSchema.UNLIMITED, BasicOntology.SET);
			cs.addSuperSchema((ConceptSchema) getSchema(OLISBasicOntology.USERPREFERENCES));
		} catch (OntologyException oe) {
			oe.printStackTrace();
		}
	}

}
