/*
 * Created on Jun 22, 2008
 */
package br.pucrio.olis.ontology;

import jade.content.onto.BasicOntology;
import jade.content.onto.Ontology;
import jade.content.onto.OntologyException;
import jade.content.schema.ConceptSchema;
import jade.content.schema.ObjectSchema;
import jade.content.schema.PredicateSchema;
import jade.content.schema.PrimitiveSchema;
import br.pucrio.olis.agent.weather.ForecastRequest;
import br.pucrio.olis.agent.weather.ForecastResponse;
import br.pucrio.olis.agent.weather.WeatherRequest;
import br.pucrio.olis.agent.weather.WeatherResponse;
import br.pucrio.olis.domain.DegreeUnit;
import br.pucrio.olis.domain.WeatherUserService;
import br.pucrio.olis.domain.weather.Barometric;
import br.pucrio.olis.domain.weather.CurrentConditions;
import br.pucrio.olis.domain.weather.DayForecast;
import br.pucrio.olis.domain.weather.DayPart;
import br.pucrio.olis.domain.weather.Head;
import br.pucrio.olis.domain.weather.Location;
import br.pucrio.olis.domain.weather.LocationSimple;
import br.pucrio.olis.domain.weather.Moon;
import br.pucrio.olis.domain.weather.Weather;
import br.pucrio.olis.domain.weather.Wind;

/**
 * @author Ingrid Nunes
 *
 */
public class OLISWeatherOntology extends Ontology implements OLISWeatherVocabulary {
	
	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 6113260660006024268L;

	public static final String ONTOLOGY_NAME = "OLISWeatherOntology";
	private static Ontology theInstance = new OLISWeatherOntology();

	/**
	 * This method grants access to the unique instance of the ontology.
	 * 
	 * @return An <code>Ontology</code> object, containing the concepts of the
	 *         ontology.
	 */
	public static Ontology getInstance() {
		return theInstance;
	}

	public OLISWeatherOntology() {
		super(ONTOLOGY_NAME, new Ontology[] { OLISBasicOntology.getInstance() },
				new OLISIntrospector());

		try {
			add(new EnumerationSchema(DEGREEUNIT), DegreeUnit.class);
			add(new EnumerationSchema(DAYPARTTYPE), DayPart.DayPartType.class);
			
			add(new ConceptSchema(WEATHERUSERSERVICE), WeatherUserService.class);
			
			add(new ConceptSchema(BAROMETRIC), Barometric.class);
			add(new ConceptSchema(CURRENTCONDITIONS), CurrentConditions.class);
			add(new ConceptSchema(DAYFORECAST), DayForecast.class);
			add(new ConceptSchema(DAYPART), DayPart.class);
			add(new ConceptSchema(HEAD), Head.class);
			add(new ConceptSchema(LOCATION), Location.class);
			add(new ConceptSchema(LOCATIONSIMPLE), LocationSimple.class);
			add(new ConceptSchema(MOON), Moon.class);
			add(new ConceptSchema(UV), br.pucrio.olis.domain.weather.UV.class);
			add(new ConceptSchema(WEATHER), Weather.class);
			add(new ConceptSchema(WIND), Wind.class);
			
			ConceptSchema cs = (ConceptSchema) getSchema(WEATHERUSERSERVICE);
			cs.add(WEATHERUSERSERVICE_DEFAULTCITY, (ConceptSchema) getSchema(OLISBasicOntology.CITY));
			cs.add(WEATHERUSERSERVICE_DEGREEUNIT, (PrimitiveSchema) getSchema(DEGREEUNIT));
			cs.addSuperSchema((ConceptSchema) getSchema(OLISBasicOntology.USERSERVICE));
			
			cs = (ConceptSchema) getSchema(BAROMETRIC);
			cs.add(BAROMETRIC_PRESSURE, (PrimitiveSchema) getSchema(BasicOntology.FLOAT), ObjectSchema.OPTIONAL);
			cs.add(BAROMETRIC_TREND, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			
			cs = (ConceptSchema) getSchema(CURRENTCONDITIONS);
			cs.add(CURRENTCONDITIONS_BAROMETRIC, (ConceptSchema) getSchema(BAROMETRIC), ObjectSchema.OPTIONAL);
			cs.add(CURRENTCONDITIONS_DATEOFOBSERVATION, (PrimitiveSchema) getSchema(BasicOntology.DATE), ObjectSchema.OPTIONAL);
			cs.add(CURRENTCONDITIONS_DESCRIPTION, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			cs.add(CURRENTCONDITIONS_DEWPOINT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
			cs.add(CURRENTCONDITIONS_FEELLIKETEMPERATURE, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
			cs.add(CURRENTCONDITIONS_HUMIDITY, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
			cs.add(CURRENTCONDITIONS_ICON, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
			cs.add(CURRENTCONDITIONS_MOON, (ConceptSchema) getSchema(MOON), ObjectSchema.OPTIONAL);
			cs.add(CURRENTCONDITIONS_OBSERVATIONSTATION, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			cs.add(CURRENTCONDITIONS_TEMPERATURE, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
			cs.add(CURRENTCONDITIONS_UV, (ConceptSchema) getSchema(UV), ObjectSchema.OPTIONAL);
			cs.add(CURRENTCONDITIONS_VISIBILITY, (PrimitiveSchema) getSchema(BasicOntology.FLOAT), ObjectSchema.OPTIONAL);
			cs.add(CURRENTCONDITIONS_WIND, (ConceptSchema) getSchema(WIND), ObjectSchema.OPTIONAL);
			
			cs = (ConceptSchema) getSchema(DAYFORECAST);
			cs.add(DAYFORECAST_DAY, (ConceptSchema) getSchema(DAYPART), ObjectSchema.OPTIONAL);
			cs.add(DAYFORECAST_DAYNUMBER, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
			cs.add(DAYFORECAST_DAYOFMONTH, (PrimitiveSchema) getSchema(BasicOntology.DATE), ObjectSchema.OPTIONAL);
			cs.add(DAYFORECAST_HIGHTEMPERATURE, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
			cs.add(DAYFORECAST_LOWTEMPERATURE, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
			cs.add(DAYFORECAST_NIGHT, (ConceptSchema) getSchema(DAYPART), ObjectSchema.OPTIONAL);
			cs.add(DAYFORECAST_SUNRISE, (PrimitiveSchema) getSchema(BasicOntology.DATE), ObjectSchema.OPTIONAL);
			cs.add(DAYFORECAST_SUNSET, (PrimitiveSchema) getSchema(BasicOntology.DATE), ObjectSchema.OPTIONAL);
			
			cs = (ConceptSchema) getSchema(DAYPART);
			cs.add(DAYPART_CHANCEOFPRECIPITATION, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
			cs.add(DAYPART_DESCRIPTION, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			cs.add(DAYPART_DESCRIPTIONSHORT, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			cs.add(DAYPART_HUMIDITY, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
			cs.add(DAYPART_ICON, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
			cs.add(DAYPART_TYPE, (PrimitiveSchema) getSchema(DAYPARTTYPE), ObjectSchema.OPTIONAL);
			cs.add(DAYPART_WIND, (ConceptSchema) getSchema(WIND), ObjectSchema.OPTIONAL);
			
			cs = (ConceptSchema) getSchema(HEAD);
			cs.add(HEAD_DEGREEUNIT, (PrimitiveSchema) getSchema(DEGREEUNIT), ObjectSchema.OPTIONAL);
			cs.add(HEAD_DISTANCEUNIT, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			cs.add(HEAD_FORM, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			cs.add(HEAD_LOCALE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			cs.add(HEAD_PRESSUREUNIT, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			cs.add(HEAD_RAINUNIT, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			cs.add(HEAD_SPEEDUNIT, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			
			cs = (ConceptSchema) getSchema(LOCATION);
			cs.add(LOCATION_ID, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			cs.add(LOCATION_LATITUDE, (PrimitiveSchema) getSchema(BasicOntology.FLOAT), ObjectSchema.OPTIONAL);
			cs.add(LOCATION_LONGITUDE, (PrimitiveSchema) getSchema(BasicOntology.FLOAT), ObjectSchema.OPTIONAL);
			cs.add(LOCATION_NAME, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			cs.add(LOCATION_SUNRISE, (PrimitiveSchema) getSchema(BasicOntology.DATE), ObjectSchema.OPTIONAL);
			cs.add(LOCATION_SUNSET, (PrimitiveSchema) getSchema(BasicOntology.DATE), ObjectSchema.OPTIONAL);
			cs.add(LOCATION_TIME, (PrimitiveSchema) getSchema(BasicOntology.DATE), ObjectSchema.OPTIONAL);
			cs.add(LOCATION_ZONE, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
			
			cs = (ConceptSchema) getSchema(LOCATIONSIMPLE);
			cs.add(LOCATIONSIMPLE_ID, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			cs.add(LOCATIONSIMPLE_TYPE, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
			cs.add(LOCATIONSIMPLE_NAME, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			
			cs = (ConceptSchema) getSchema(MOON);
			cs.add(MOON_DESCRIPTION, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			cs.add(MOON_ICON, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
			
			cs = (ConceptSchema) getSchema(UV);
			cs.add(UV_DESCRIPTION, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			cs.add(UV_INDEX, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
			
			cs = (ConceptSchema) getSchema(WEATHER);
			cs.add(WEATHER_CURRENTCONDITIONS, (ConceptSchema) getSchema(CURRENTCONDITIONS), ObjectSchema.OPTIONAL);
			cs.add(WEATHER_DAYFORCASTLIST, (ConceptSchema) getSchema(DAYFORECAST), 0, ObjectSchema.UNLIMITED, BasicOntology.SEQUENCE);
			cs.add(WEATHER_HEAD, (ConceptSchema) getSchema(HEAD), ObjectSchema.OPTIONAL);
			cs.add(WEATHER_LOCATION, (ConceptSchema) getSchema(LOCATION), ObjectSchema.OPTIONAL);
			
			cs = (ConceptSchema) getSchema(WIND);
			cs.add(WIND_DIRECTION, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
			cs.add(WIND_DIRECTIONPHRASE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			cs.add(WIND_GUST, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			cs.add(WIND_SPEED, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);

			add(new PredicateSchema(WEATHERREQUEST), WeatherRequest.class);
			add(new PredicateSchema(WEATHERRESPONSE), WeatherResponse.class);
			add(new PredicateSchema(FORECASTREQUEST), ForecastRequest.class);
			add(new PredicateSchema(FORECASTRESPONSE), ForecastResponse.class);

			PredicateSchema ps = (PredicateSchema) getSchema(WEATHERREQUEST);
			ps.add(WEATHERREQUEST_CITY, (ConceptSchema) getSchema(OLISBasicOntology.CITY));
			ps.add(WEATHERREQUEST_CURRENTCONDITIONS, (PrimitiveSchema) getSchema(BasicOntology.BOOLEAN));
			ps.add(WEATHERREQUEST_DAYFORECASTS, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
			ps.add(WEATHERREQUEST_DEGREEUNIT, (PrimitiveSchema) getSchema(DEGREEUNIT));
			
			ps = (PredicateSchema) getSchema(WEATHERRESPONSE);
			ps.add(WEATHERRESPONSE_WEATHER, getSchema(WEATHER), ObjectSchema.OPTIONAL);
			
			ps = (PredicateSchema) getSchema(FORECASTREQUEST);
			ps.add(FORECASTREQUEST_CITY, (ConceptSchema) getSchema(OLISBasicOntology.CITY));
			ps.add(FORECASTREQUEST_DATE, (PrimitiveSchema) getSchema(BasicOntology.DATE));
			ps.add(FORECASTREQUEST_DEGREEUNIT, (PrimitiveSchema) getSchema(DEGREEUNIT));
			
			ps = (PredicateSchema) getSchema(FORECASTRESPONSE);
			ps.add(FORECASTRESPONSE_DAYFORCAST, (ConceptSchema) getSchema(DAYFORECAST), ObjectSchema.OPTIONAL);
			ps.add(FORECASTRESPONSE_HEAD, (ConceptSchema) getSchema(HEAD), ObjectSchema.OPTIONAL);
			ps.add(FORECASTRESPONSE_LOCATION, (ConceptSchema) getSchema(LOCATION), ObjectSchema.OPTIONAL);
		} catch (OntologyException oe) {
			oe.printStackTrace();
		}
	}
	
}
