/*
 * Created on May 12, 2008
 */
package br.pucrio.olis.util;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

/**
 * <p>
 * This class has several useful static methods to be used in the application.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class OlisUtils {

	/**
	 * <p>
	 * Return the date that represent the month begin.
	 * </p>
	 * 
	 * @param month
	 *            the month.
	 * @param year
	 *            the year.
	 * @return the date that is the month begin.
	 */
	public static Date getMonthBegin(Integer month, Integer year) {
		Calendar calendar = GregorianCalendar.getInstance();
		calendar.set(year, month, 1, 0, 0, 0);
		return calendar.getTime();
	}

	/**
	 * <p>
	 * Return the date that represent the month end.
	 * </p>
	 * 
	 * @param month
	 *            the month.
	 * @param year
	 *            the year.
	 * @return the date that is the month end.
	 */
	public static Date getMonthEnd(Integer month, Integer year) {
		Calendar calendar = GregorianCalendar.getInstance();
		calendar.set(year, month, 1, 0, 0, 0);
		calendar.add(Calendar.MONTH, 1);
		calendar.add(Calendar.SECOND, -1);
		return calendar.getTime();
	}

	/**
	 * <p>
	 * Returns an instance of calendar with the time set in the next month.
	 * </p>
	 * 
	 * @param month
	 *            the current month.
	 * @param year
	 *            the current year.
	 * @return the calendar instance set in the next month
	 */
	public static Calendar getNextMonth(Integer month, Integer year) {
		Calendar calendar = GregorianCalendar.getInstance();
		calendar.set(Calendar.MONTH, month);
		calendar.set(Calendar.YEAR, year);
		calendar.add(Calendar.MONTH, 1);
		return calendar;
	}

	/**
	 * <p>
	 * Returns an instance of calendar with the time set in the previous month.
	 * </p>
	 * 
	 * @param month
	 *            the current month.
	 * @param year
	 *            the current year.
	 * @return the calendar instance set in the previous month
	 */
	public static Calendar getPreviousMonth(Integer month, Integer year) {
		Calendar calendar = GregorianCalendar.getInstance();
		calendar.set(Calendar.MONTH, month);
		calendar.set(Calendar.YEAR, year);
		calendar.add(Calendar.MONTH, -1);
		return calendar;
	}

	/**
	 * <p>
	 * Return the end time of the provided date.
	 * </p>
	 * 
	 * @return the end time of the provided date.
	 */
	public static Date getDayEnd(Date date) {
		Calendar calendar = GregorianCalendar.getInstance();
		calendar.setTime(date);
		calendar.set(Calendar.HOUR_OF_DAY, 23);
		calendar.set(Calendar.MINUTE, 59);
		calendar.set(Calendar.SECOND, 59);
		calendar.set(Calendar.MILLISECOND, 59);
		return calendar.getTime();
	}

	/**
	 * <p>
	 * Check if the string is invalid.
	 * </p>
	 * 
	 * @param value
	 *            the value to be checked.
	 * @return true if the string is invalid, false otherwise.
	 */
	public static boolean isEmpty(String value) {
		return (value == null || value.length() == 0);
	}

	/**
	 * <p>
	 * Chech if at least one of the given strings is invalid.
	 * </p>
	 * 
	 * @param values
	 *            the string array.
	 * @return true if one of the the string is invalid, false otherwise.
	 */
	public static boolean isMandatoryPropertiesMissing(String[] values) {
		for (int i = 0; i < values.length; i++) {
			if (isEmpty(values[i]))
				return true;
		}
		return false;
	}

}
