/*
 * Created on Apr 8, 2008
 */
package br.pucrio.olis.web;

import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts2.interceptor.ApplicationAware;
import org.apache.struts2.interceptor.SessionAware;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.domain.User;

import com.opensymphony.xwork2.ActionSupport;

/**
 * <p>
 * Base Action class for the OLIS Application.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class OlisActionSupport extends ActionSupport implements SessionAware,
		ApplicationAware {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 2924865239784359384L;

	/**
	 * <p>
	 * Field to store application context or its proxy.
	 * </p>
	 * <p/>
	 * <p>
	 * The application context lasts for the life of the application. A
	 * reference to the database is stored in the application context at
	 * startup.
	 * </p>
	 */
	@SuppressWarnings("unchecked")
	private Map application;

	/**
	 * <p>
	 * <code>Log</code> instance for this application.
	 * </p>
	 */
	protected Log log = LogFactory.getLog(Constants.PACKAGE);

	/**
	 * <p>
	 * Field to store session context, or its proxy.
	 * </p>
	 */
	@SuppressWarnings("unchecked")
	private Map session;

	/**
	 * <p>
	 * Field to store workflow task.
	 * </p>
	 * <p/>
	 * <p>
	 * The Task is used to track the state of the CRUD workflows. It can be set
	 * to Constant.CREATE, Constant.EDIT, or Constant.DELETE as needed.
	 * </p>
	 */
	private String task = null;

	/**
	 * <p>
	 * Field to store double-submit guard.
	 * </p>
	 */
	private String token = null;

	protected void addActionError(BusinessException businessException) {
		if (businessException.getArgs() == null
				|| businessException.getArgs().length == 0) {
			addActionError(getText(businessException.getMessage()));
		} else {

			String[] args = new String[businessException.getArgs().length];
			for (int i = 0; i < businessException.getArgs().length; i++) {
				args[i] = getText(businessException.getArgs()[i]);
			}
			addActionError(getText(businessException.getMessage(), args));
		}
	}

	protected void addFieldError(String fieldName, String message,
			String fieldKey) {
		addFieldError(fieldName, getText(message,
				new String[] { getText(fieldKey) }));
	}

	/**
	 * Return CANCEL so apropriate result can be selected.
	 * 
	 * @return "cancel" so apropriate result can be selected.
	 */
	public String cancel() {
		return Constants.CANCEL;
	}

	/**
	 * <p>
	 * Provide application context.
	 * </p>
	 */
	@SuppressWarnings("unchecked")
	public Map getApplication() {
		return application;
	}

	/**
	 * <p>
	 * Provide session context.
	 * </p>
	 * 
	 * @return session context
	 */
	@SuppressWarnings("unchecked")
	public Map getSession() {
		return session;
	}

	/**
	 * <p>
	 * Provide worklow task.
	 * </p>
	 * 
	 * @return Returns the task.
	 */
	public String getTask() {
		return task;
	}

	/**
	 * <p>
	 * Provide Token.
	 * </p>
	 * 
	 * @return Returns the token.
	 */
	public String getToken() {
		return token;
	}

	/**
	 * <p>
	 * Provide reference to User object for authenticated user.
	 * </p>
	 * 
	 * @return User object for authenticated user.
	 */
	public User getUser() {
		return (User) getSession().get(Constants.USER_KEY);
	}

	/**
	 * <p>
	 * Check if the user is logged.
	 * </p>
	 * 
	 * @return True if the user is logged.
	 */
	public boolean isUserLogged() {
		return getUser() != null;
	}

	/**
	 * <p>
	 * Store a new application context.
	 * </p>
	 * 
	 * @param value
	 *            A Map representing application state
	 */
	@SuppressWarnings("unchecked")
	public void setApplication(Map value) {
		application = value;
	}

	/**
	 * <p>
	 * Store a new session context.
	 * </p>
	 * 
	 * @param value
	 *            A Map representing session state
	 */
	@SuppressWarnings("unchecked")
	public void setSession(Map value) {
		session = value;
	}

	/**
	 * <p>
	 * Store new workflow task.
	 * </p>
	 * 
	 * @param value
	 *            The task to set.
	 */
	public void setTask(String value) {
		task = value;
	}

	/**
	 * <p>
	 * Store new Token.
	 * </p>
	 * 
	 * @param value
	 *            The token to set.
	 */
	public void setToken(String value) {
		token = value;
	}

	/**
	 * <p>
	 * Store new reference to User Object.
	 * </p>
	 * 
	 * @param user
	 *            User object for authenticated user
	 */
	@SuppressWarnings("unchecked")
	public void setUser(User user) {
		getSession().put(Constants.USER_KEY, user);
	}

}
