/*
 * Created on May 24, 2008
 */
package br.pucrio.olis.web;

import java.util.ArrayList;
import java.util.List;

import br.pucrio.olis.business.service.LocationService;
import br.pucrio.olis.business.service.TravelEventService;
import br.pucrio.olis.domain.ActivityType;
import br.pucrio.olis.domain.Event;
import br.pucrio.olis.domain.PlaceType;
import br.pucrio.olis.domain.TravelEvent;

/**
 * <p>
 * This class represents the data manager of travel events. It contains the
 * fields and lists necessary to manage travel events. It is used by different
 * actions.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class TravelEventDataManager extends EventDataManager {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -3659097783870534536L;

	private List<Long> activities;
	private List<ActivityType> activitiesTypes;
	private PlaceType placeType;
	private List<PlaceType> placeTypes;
	private TravelEventService travelEventService;

	/**
	 * <p>
	 * Create an instance of TravelEventAbstractAction.
	 * </p>
	 * 
	 * @param travelEventService
	 *            the travel event service.
	 * @param locationService
	 *            the location service.
	 */
	protected TravelEventDataManager(TravelEventService travelEventService,
			LocationService locationService) {
		super(locationService);
		this.travelEventService = travelEventService;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.web.EventAbstractAction#createEvent()
	 */
	@Override
	public Event createEvent() throws Exception {
		return new TravelEvent();
	}

	public List<Long> getActivities() {
		return activities;
	}

	public List<ActivityType> getActivitiesTypes() {
		return activitiesTypes;
	}

	/* (non-Javadoc)
	 * @see br.pucrio.olis.web.EventDataManager#getEventPage()
	 */
	@Override
	public String getEventPage() {
		return "TravelEventInput";
	}

	public PlaceType getPlaceType() {
		return placeType;
	}

	public List<PlaceType> getPlaceTypes() {
		return placeTypes;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.web.EventAbstractAction#initializeFields(br.pucrio.olis.domain.Event)
	 */
	@Override
	public void initializeFields(Event event) throws Exception {
		super.initializeFields(event);
		if (event != null) {
			TravelEvent travelEvent = (TravelEvent) event;
			this.activities = new ArrayList<Long>(travelEvent.getActivities()
					.size());
			for (ActivityType activityType : travelEvent.getActivities()) {
				this.activities.add(activityType.getId());
			}
			this.placeType = travelEvent.getPlaceType();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.web.EventDataManager#prepare()
	 */
	@Override
	public void prepare() throws Exception {
		super.prepare();
		this.activitiesTypes = travelEventService.loadAllActivityTypes();
		this.placeTypes = travelEventService.loadAllPlaceTypes();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.pucrio.olis.web.EventAbstractAction#readEvent()
	 */
	@Override
	public Event readEvent() throws Exception {
		TravelEvent event = (TravelEvent) super.readEvent();
		if (activities != null) {
			for (Long id : activities) {
				ActivityType activityType = new ActivityType();
				activityType.setId(id);
				event.addActivities(activityType);
			}
		}
		event.setPlaceType(placeType);
		return event;
	}

	public void setActivities(List<Long> activities) {
		this.activities = activities;
	}

	public void setActivitiesTypes(List<ActivityType> activitiesTypes) {
		this.activitiesTypes = activitiesTypes;
	}

	public void setPlaceType(PlaceType placeType) {
		this.placeType = placeType;
	}

	public void setPlaceTypes(List<PlaceType> placeTypes) {
		this.placeTypes = placeTypes;
	}

}
