/*
 * Created on Jun 1, 2008
 */
package br.pucrio.olis.web.calendar;

import org.springframework.dao.DataAccessException;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.service.CalendarService;
import br.pucrio.olis.domain.CalendarUserService;
import br.pucrio.olis.domain.User;
import br.pucrio.olis.web.Constants;
import br.pucrio.olis.web.OlisActionSupport;

/**
 * <p>
 * This class is the action that performs the update of the calendar service
 * options.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class CalendarOptionsAction extends OlisActionSupport {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -218432179215465387L;

	private CalendarUserService calendarUserService;
	private CalendarService calendarService;

	/**
	 * <p>
	 * Create an instance of CalendarOptionsAction.
	 * </p>
	 * 
	 * @param calendarService
	 *            the calendar service.
	 */
	public CalendarOptionsAction(CalendarService calendarService) {
		this.calendarService = calendarService;
	}

	/**
	 * <p>
	 * Update calendar options in the persistent store.
	 * </p>
	 * 
	 * @return The "outcome" result code
	 * @throws Exception
	 *             on any error
	 */
	public String execute() throws Exception {
		try {
			calendarUserService.setUser(getUser());
			User user = calendarService.update(calendarUserService);
			setUser(user);
		} catch (BusinessException exc) {
			log.error(Constants.LOG_BUSINESS_ERROR);
			exc.printStackTrace();
			addActionError(exc);
		} catch (DataAccessException exc) {
			log.error(Constants.LOG_DATABASE_ERROR);
			exc.printStackTrace();
			addActionError(getText("error.unexcepected", exc.getMessage()));
		}

		if (hasErrors()) {
			return INPUT;
		}

		return SUCCESS;
	}

	public CalendarUserService getCalendarUserService() {
		return calendarUserService;
	}

	/**
	 * <p>
	 * Retrieve user advanced options and populate the action fields.
	 * </p>
	 * 
	 * @return The "Success" result for this mapping
	 * @throws Exception
	 *             on any error
	 */
	public String input() throws Exception {
		User user = getUser();
		this.calendarUserService = (CalendarUserService) user
				.getService(CalendarUserService.class);
		return INPUT;
	}

	public void setCalendarUserService(CalendarUserService calendarUserService) {
		this.calendarUserService = calendarUserService;
	}

}
