/*
 * Created on Jun 1, 2008
 */
package br.pucrio.olis.web.eventsannouncement;

import java.text.SimpleDateFormat;

import org.springframework.dao.DataAccessException;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.service.EventsAnnouncementService;
import br.pucrio.olis.domain.EventsAnnouncementUserService;
import br.pucrio.olis.domain.TimeFormat;
import br.pucrio.olis.domain.User;
import br.pucrio.olis.web.Constants;
import br.pucrio.olis.web.OlisActionSupport;

/**
 * <p>
 * This class is the action that performs the update of the events announcement
 * service options.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class EventsAnnouncementOptionsAction extends OlisActionSupport {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -218432179215465387L;

	private EventsAnnouncementService eventsAnnouncementService;
	private EventsAnnouncementUserService eventsAnnouncementUserService;
	private String workTimeBegin;
	private String workTimeEnd;

	/**
	 * <p>
	 * Create an instance of EventsAnnouncementOptionsAction.
	 * </p>
	 * 
	 * @param eventsAnnouncementService
	 *            the events announcement service.
	 */
	public EventsAnnouncementOptionsAction(
			EventsAnnouncementService eventsAnnouncementService) {
		this.eventsAnnouncementService = eventsAnnouncementService;
	}

	/**
	 * <p>
	 * Update events announcement service options in the persistent store.
	 * </p>
	 * 
	 * @return The "outcome" result code
	 * @throws Exception
	 *             on any error
	 */
	public String execute() throws Exception {
		SimpleDateFormat timeFormat = new SimpleDateFormat(TimeFormat.HOURS_24
				.getFormat());
		if (workTimeBegin == null || "".equals(workTimeBegin)) {
			eventsAnnouncementUserService.setWorkTimeBegin(null);
		} else {
			try {
				eventsAnnouncementUserService.setWorkTimeBegin(timeFormat
						.parse(workTimeBegin));
			} catch (Exception e) {
				addFieldError("workTimeBegin", "error.invalid", "workTimeBegin");
			}
		}
		if (workTimeEnd == null || "".equals(workTimeEnd)) {
			eventsAnnouncementUserService.setWorkTimeEnd(null);
		} else {
			try {
				eventsAnnouncementUserService.setWorkTimeEnd(timeFormat
						.parse(workTimeEnd));
			} catch (Exception e) {
				addFieldError("workTimeEnd", "error.invalid", "workTimeBegin");
			}
		}
		eventsAnnouncementUserService.setUser(getUser());

		if (hasErrors()) {
			return INPUT;
		}

		try {
			User user = eventsAnnouncementService
					.update(eventsAnnouncementUserService);
			setUser(user);
		} catch (BusinessException exc) {
			log.error(Constants.LOG_BUSINESS_ERROR);
			exc.printStackTrace();
			addActionError(exc);
		} catch (DataAccessException exc) {
			log.error(Constants.LOG_DATABASE_ERROR);
			exc.printStackTrace();
			addActionError(getText("error.unexcepected", exc.getMessage()));
		}

		if (hasErrors()) {
			return INPUT;
		}

		return SUCCESS;
	}

	public EventsAnnouncementUserService getEventsAnnouncementUserService() {
		return eventsAnnouncementUserService;
	}

	public String getWorkTimeBegin() {
		return workTimeBegin;
	}

	public String getWorkTimeEnd() {
		return workTimeEnd;
	}

	/**
	 * <p>
	 * Retrieve events announcement service options and populate the action
	 * fields.
	 * </p>
	 * 
	 * @return The "Success" result for this mapping
	 * @throws Exception
	 *             on any error
	 */
	public String input() throws Exception {
		User user = getUser();
		this.eventsAnnouncementUserService = (EventsAnnouncementUserService) user
				.getService(EventsAnnouncementUserService.class);
		SimpleDateFormat timeFormat = new SimpleDateFormat(TimeFormat.HOURS_24
				.getFormat());
		if (eventsAnnouncementUserService.getWorkTimeBegin() == null) {
			this.workTimeBegin = null;
		} else {
			this.workTimeBegin = timeFormat
					.format(eventsAnnouncementUserService.getWorkTimeBegin());
		}
		if (eventsAnnouncementUserService.getWorkTimeEnd() == null) {
			this.workTimeEnd = null;
		} else {
			this.workTimeEnd = timeFormat.format(eventsAnnouncementUserService
					.getWorkTimeEnd());
		}
		return INPUT;
	}

	public void setEventsAnnouncementUserService(
			EventsAnnouncementUserService eventsAnnouncementUserService) {
		this.eventsAnnouncementUserService = eventsAnnouncementUserService;
	}

	public void setWorkTimeBegin(String workTimeBegin) {
		this.workTimeBegin = workTimeBegin;
	}

	public void setWorkTimeEnd(String workTimeEnd) {
		this.workTimeEnd = workTimeEnd;
	}

}
