/*
 * Created on May 22, 2008
 */
package br.pucrio.olis.web.eventsannouncement;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;

import org.springframework.dao.DataAccessException;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.service.EventsAnnouncementService;
import br.pucrio.olis.domain.Event;
import br.pucrio.olis.util.OlisUtils;
import br.pucrio.olis.web.Constants;
import br.pucrio.olis.web.OlisActionSupport;

/**
 * <p>
 * This class is the action that displays the events board of the events
 * announcement service.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class EventsBoardAction extends OlisActionSupport {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -3403015049440342096L;

	private List<Event> events;
	private EventsAnnouncementService eventsAnnouncementService;
	private Integer month;
	private String monthLabel;
	private List<Event> periodicEvents;
	private Integer year;

	/**
	 * <p>
	 * Create an instance of EventsBoardAction.
	 * </p>
	 * 
	 * @param eventsAnnouncementService
	 *            the events announcement service.
	 */
	public EventsBoardAction(EventsAnnouncementService eventsAnnouncementService) {
		this.eventsAnnouncementService = eventsAnnouncementService;
	}

	/**
	 * <p>
	 * Open the events and periodic events in a certain month.
	 * </p>
	 * 
	 * @return The "outcome" result code
	 * @throws Exception
	 *             on any error
	 */
	public String execute() throws Exception {
		try {
			this.monthLabel = getText("month." + this.month);
			this.events = eventsAnnouncementService.loadEventsInMonth(month,
					year);
			this.periodicEvents = eventsAnnouncementService
					.loadPeriodicEventsInMonth(month, year);
		} catch (BusinessException exc) {
			log.error(Constants.LOG_BUSINESS_ERROR);
			exc.printStackTrace();
			addActionError(exc);
		} catch (DataAccessException exc) {
			log.error(Constants.LOG_DATABASE_ERROR);
			exc.printStackTrace();
			addActionError(getText("error.unexcepected", exc.getMessage()));
		}

		return SUCCESS;
	}

	public List<Event> getEvents() {
		return events;
	}

	public Integer getMonth() {
		return month;
	}

	public String getMonthLabel() {
		return monthLabel;
	}

	public List<Event> getPeriodicEvents() {
		return periodicEvents;
	}

	public Integer getYear() {
		return year;
	}

	/**
	 * <p>
	 * Open the events board in the next month.
	 * </p>
	 * 
	 * @return
	 * @throws Exception
	 */
	public String nextMonth() throws Exception {
		Calendar calendar = OlisUtils.getNextMonth(month, year);
		this.month = calendar.get(Calendar.MONTH);
		this.year = calendar.get(Calendar.YEAR);
		return execute();
	}

	/**
	 * <p>
	 * Open the events board in the current month.
	 * </p>
	 * 
	 * @return
	 * @throws Exception
	 */
	public String open() throws Exception {
		Calendar calendar = GregorianCalendar.getInstance();
		this.month = calendar.get(Calendar.MONTH);
		this.year = calendar.get(Calendar.YEAR);
		return execute();
	}

	/**
	 * <p>
	 * Open the events board in the previous month.
	 * </p>
	 * 
	 * @return
	 * @throws Exception
	 */
	public String previousMonth() throws Exception {
		Calendar calendar = OlisUtils.getPreviousMonth(month, year);
		this.month = calendar.get(Calendar.MONTH);
		this.year = calendar.get(Calendar.YEAR);
		return execute();
	}

	public void setEvents(List<Event> events) {
		this.events = events;
	}

	public void setMonth(Integer month) {
		this.month = month;
	}

	public void setMonthLabel(String monthLabel) {
		this.monthLabel = monthLabel;
	}

	public void setPeriodicEvents(List<Event> periodicEvents) {
		this.periodicEvents = periodicEvents;
	}

	public void setYear(Integer year) {
		this.year = year;
	}

}
