/*
 * Created on Apr 8, 2008
 */
package br.pucrio.olis.web.support;

import org.springframework.dao.DataAccessException;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.service.UserService;
import br.pucrio.olis.domain.User;
import br.pucrio.olis.util.OlisUtils;
import br.pucrio.olis.web.Constants;
import br.pucrio.olis.web.OlisActionSupport;

/**
 * <p>
 * This class is the action that performs the login of the user in the system.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class LoginAction extends OlisActionSupport {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 4079953184055795341L;

	/**
	 * <p>
	 * Field to store User password property.
	 * </p>
	 * <p/>
	 * <p>
	 * The User DAO object password proerty is immutable, so we store it locally
	 * until we are ready to create the object.
	 * </p>
	 */
	private String password = null;
	/**
	 * <p>
	 * Field to store User username.
	 * </p>
	 * <p/>
	 * <p>
	 * The User DAO object password proerty is immutable, so we store it locally
	 * until we are ready to create the object.
	 * </p>
	 */
	private String username = null;
	private UserService userService;

	/**
	 * <p>
	 * Create an instance of LoginAction.
	 * </p>
	 * 
	 * @param userService
	 *            the user service.
	 */
	public LoginAction(UserService userService) {
		this.userService = userService;
	}

	/**
	 * <p>
	 * Perform the user login. Try to find the user with the provided
	 * credentials. Return to the login page if the data is not correct.
	 * </p>
	 */
	public String execute() throws Exception {
		if (OlisUtils.isEmpty(getUsername())
				|| OlisUtils.isEmpty(getPassword()))
			return INPUT;

		User user = null;
		try {
			user = userService.login(username, password);
		} catch (BusinessException exc) {
			log.error(Constants.LOG_BUSINESS_ERROR);
			exc.printStackTrace();
			addActionError(exc);
		} catch (DataAccessException exc) {
			log.error(Constants.LOG_DATABASE_ERROR);
			exc.printStackTrace();
			addActionError(getText("error.unexcepected", exc.getMessage()));
		}

		if (user != null)
			super.setUser(user);

		if (hasErrors()) {
			return INPUT;
		}

		return SUCCESS;
	}

	/**
	 * <p>
	 * Provide User password.
	 * </p>
	 * 
	 * @return Returns the password.
	 */
	public String getPassword() {
		return password;
	}

	/**
	 * <p>
	 * Provide User username.
	 * </p>
	 * 
	 * @return Returns the User username.
	 */
	public String getUsername() {
		return username;
	}

	/**
	 * <p>
	 * Store new User Password.
	 * </p>
	 * 
	 * @param value
	 *            The password to set.
	 */
	public void setPassword(String value) {
		password = value;
	}

	/**
	 * <p>
	 * Store new User username.
	 * </p>
	 * 
	 * @param value
	 *            The username to set.
	 */
	public void setUsername(String value) {
		username = value;
	}

}
