/*
 * Created on Jun 21, 2008
 */
package br.pucrio.olis.web.support;

import java.util.Map;
import java.util.Set;

import org.springframework.dao.DataAccessException;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.service.UserService;
import br.pucrio.olis.domain.Notification;
import br.pucrio.olis.web.Constants;
import br.pucrio.olis.web.OlisActionSupport;

/**
 * @author Ingrid Nunes
 * 
 */
public class NotificationAction extends OlisActionSupport {

	private static Map<String, String> notificationPages;

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -5277831502644253354L;

	private Notification notification;
	private Set<Notification> notifications;
	private UserService userService;

	/**
	 * <p>
	 * Create an instance of NotificationAction.
	 * </p>
	 * 
	 * @param userService
	 *            the userService service.
	 */
	public NotificationAction(UserService userService,
			Map<String, String> notificationPages) {
		this.userService = userService;
		NotificationAction.notificationPages = notificationPages;
	}

	/**
	 * <p>
	 * Delete operation.
	 * </p>
	 * 
	 * @return The "outcome" result code
	 * @throws Exception
	 *             on any error
	 */
	public String delete() throws Exception {
		setTask(Constants.DELETE);
		return execute();
	}

	/**
	 * <p>
	 * Insert or update a Event object to the persistent store.
	 * </p>
	 * 
	 * @return The "outcome" result code
	 * @throws Exception
	 *             on any error
	 */
	public String execute() throws Exception {
		try {
			if (Constants.DELETE.equals(getTask())) {
				userService.deleteNotification(getUser(), notification);
			}
		} catch (BusinessException exc) {
			log.error(Constants.LOG_BUSINESS_ERROR);
			exc.printStackTrace();
			addActionError(exc);
		} catch (DataAccessException exc) {
			log.error(Constants.LOG_DATABASE_ERROR);
			exc.printStackTrace();
			addActionError(getText("error.unexcepected", exc.getMessage()));
		}

		if (hasErrors()) {
			return INPUT;
		}

		return open();
	}

	public Notification getNotification() {
		return notification;
	}

	public String getNotificationPage(Notification notification) {
		return notificationPages.get(notification.getClass().getName());
	}

	public Set<Notification> getNotifications() {
		return notifications;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.opensymphony.xwork2.Preparable#prepare()
	 */
	public String open() throws Exception {
		this.notifications = userService.loadNotifications(getUser());
		return SUCCESS;
	}

	public void setNotification(Notification notification) {
		this.notification = notification;
	}

}
