/*
 * Created on May 25, 2008
 */
package br.pucrio.olis.web.support;

import java.util.ArrayList;
import java.util.List;

import org.springframework.dao.DataAccessException;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.service.TravelEventService;
import br.pucrio.olis.domain.ActivityType;
import br.pucrio.olis.domain.PlaceType;
import br.pucrio.olis.domain.TravelPreferences;
import br.pucrio.olis.domain.User;
import br.pucrio.olis.web.Constants;
import br.pucrio.olis.web.OlisActionSupport;

import com.opensymphony.xwork2.Preparable;

/**
 * <p>
 * This class is the action that performs the update of the travel user
 * preferences.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class TravelPreferencesAction extends OlisActionSupport implements
		Preparable {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = 5386758589317689686L;

	private List<Long> activities;
	private List<ActivityType> activitiesTypes;
	private Long id;
	private List<PlaceType> placeTypeList;
	private List<Long> placeTypes;
	private TravelEventService travelEventService;

	/**
	 * <p>
	 * Create an instance of TravelPreferencesAction.
	 * </p>
	 * 
	 * @param travelEventService
	 *            the travel event service.
	 */
	public TravelPreferencesAction(TravelEventService travelEventService) {
		this.travelEventService = travelEventService;
	}

	/**
	 * <p>
	 * Update user preferences in the persistent store.
	 * </p>
	 * 
	 * @return The "outcome" result code
	 * @throws Exception
	 *             on any error
	 */
	public String execute() throws Exception {
		try {
			TravelPreferences travelPreferences = new TravelPreferences();
			travelPreferences.setId(id);
			travelPreferences.setUser(getUser());
			for (Long id : activities) {
				ActivityType activityType = new ActivityType();
				activityType.setId(id);
				travelPreferences.addActivities(activityType);
			}
			for (Long id : placeTypes) {
				PlaceType placeType = new PlaceType();
				placeType.setId(id);
				travelPreferences.addPlaceTypes(placeType);
			}
			User user = travelEventService.updatePreferences(travelPreferences);
			setUser(user);
		} catch (BusinessException exc) {
			log.error(Constants.LOG_BUSINESS_ERROR);
			exc.printStackTrace();
			addActionError(exc);
		} catch (DataAccessException exc) {
			log.error(Constants.LOG_DATABASE_ERROR);
			exc.printStackTrace();
			addActionError(getText("error.unexcepected", exc.getMessage()));
		}

		if (hasErrors()) {
			return INPUT;
		}

		return SUCCESS;
	}

	public List<Long> getActivities() {
		return activities;
	}

	public List<ActivityType> getActivitiesTypes() {
		return activitiesTypes;
	}

	public Long getId() {
		return id;
	}

	public List<PlaceType> getPlaceTypeList() {
		return placeTypeList;
	}

	public List<Long> getPlaceTypes() {
		return placeTypes;
	}

	/**
	 * <p>
	 * Retrieve user preferences and populate the action fields.
	 * </p>
	 * 
	 * @return The "Success" result for this mapping
	 * @throws Exception
	 *             on any error
	 */
	public String input() throws Exception {
		User user = getUser();
		TravelPreferences travelPreferences = (TravelPreferences) user
				.getPreferences();
		this.id = travelPreferences.getId();
		this.activities = new ArrayList<Long>(travelPreferences.getActivities()
				.size());
		for (ActivityType activityType : travelPreferences.getActivities()) {
			this.activities.add(activityType.getId());
		}
		this.placeTypes = new ArrayList<Long>(travelPreferences.getPlaceTypes()
				.size());
		for (PlaceType placeType : travelPreferences.getPlaceTypes()) {
			this.placeTypes.add(placeType.getId());
		}
		return INPUT;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.opensymphony.xwork2.Preparable#prepare()
	 */
	@Override
	public void prepare() throws Exception {
		this.activitiesTypes = travelEventService.loadAllActivityTypes();
		this.placeTypeList = travelEventService.loadAllPlaceTypes();
	}

	public void setActivities(List<Long> activities) {
		this.activities = activities;
	}

	public void setActivitiesTypes(List<ActivityType> activitiesTypes) {
		this.activitiesTypes = activitiesTypes;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public void setPlaceTypeList(List<PlaceType> placeTypeList) {
		this.placeTypeList = placeTypeList;
	}

	public void setPlaceTypes(List<Long> placeTypes) {
		this.placeTypes = placeTypes;
	}

}
