/*
 * Created on May 25, 2008
 */
package br.pucrio.olis.web.support;

import java.util.Arrays;
import java.util.List;

import org.springframework.dao.DataAccessException;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.service.UserService;
import br.pucrio.olis.domain.DateFormat;
import br.pucrio.olis.domain.TimeFormat;
import br.pucrio.olis.domain.User;
import br.pucrio.olis.web.Constants;
import br.pucrio.olis.web.OlisActionSupport;

/**
 * <p>
 * This class is the action that performs the update of the user advanced
 * options.
 * </p>
 * 
 * @author Ingrid Nunes
 * @version 1.0
 * 
 */
public class UserAdvancedOptionsAction extends OlisActionSupport {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -218432179215465387L;

	private String dateFormat;
	private String timeFormat;
	private UserService userService;

	/**
	 * <p>
	 * Create an instance of UserAdvancedOptionsAction.
	 * </p>
	 * 
	 * @param userService
	 *            the user service.
	 */
	public UserAdvancedOptionsAction(UserService userService) {
		this.userService = userService;
	}

	/**
	 * <p>
	 * Update user advanced options in the persistent store.
	 * </p>
	 * 
	 * @return The "outcome" result code
	 * @throws Exception
	 *             on any error
	 */
	public String execute() throws Exception {
		try {
			User user = new User();
			user.setId(getUser().getId());
			user.setDateFormat(DateFormat.valueOf(dateFormat));
			user.setTimeFormat(TimeFormat.valueOf(timeFormat));
			user = userService.updateAdvancedOptions(user);
			setUser(user);
		} catch (BusinessException exc) {
			log.error(Constants.LOG_BUSINESS_ERROR);
			exc.printStackTrace();
			addActionError(exc);
		} catch (DataAccessException exc) {
			log.error(Constants.LOG_DATABASE_ERROR);
			exc.printStackTrace();
			addActionError(getText("error.unexcepected", exc.getMessage()));
		}

		if (hasErrors()) {
			return INPUT;
		}

		return SUCCESS;
	}

	public String getDateFormat() {
		return dateFormat;
	}

	public List<DateFormat> getDateFormats() {
		return Arrays.asList(DateFormat.values());
	}

	public String getTimeFormat() {
		return timeFormat;
	}

	public List<TimeFormat> getTimeFormats() {
		return Arrays.asList(TimeFormat.values());
	}

	/**
	 * <p>
	 * Retrieve user advanced options and populate the action fields.
	 * </p>
	 * 
	 * @return The "Success" result for this mapping
	 * @throws Exception
	 *             on any error
	 */
	public String input() throws Exception {
		User user = getUser();
		this.dateFormat = user.getDateFormat().toString();
		this.timeFormat = user.getTimeFormat().toString();
		return INPUT;
	}

	public void setDateFormat(String dateFormat) {
		this.dateFormat = dateFormat;
	}

	public void setTimeFormat(String timeFormat) {
		this.timeFormat = timeFormat;
	}

}
