/*
 * Created on 20/06/2008
 */
package br.pucrio.olis.web.weather;

import java.util.ArrayList;
import java.util.List;

import br.pucrio.olis.business.BusinessException;
import br.pucrio.olis.business.service.LocationService;
import br.pucrio.olis.business.service.WeatherService;
import br.pucrio.olis.domain.City;
import br.pucrio.olis.domain.Country;
import br.pucrio.olis.domain.WeatherUserService;
import br.pucrio.olis.domain.weather.Weather;
import br.pucrio.olis.web.Constants;
import br.pucrio.olis.web.OlisActionSupport;

import com.opensymphony.xwork2.Preparable;

/**
 * @author ingridnunes
 * @version 1.0
 * 
 */
public class WeatherAction extends OlisActionSupport implements Preparable {

	/**
	 * <p>
	 * UID generated.
	 * </p>
	 */
	private static final long serialVersionUID = -2749102463214995321L;

	private City city;
	private List<Country> countries;
	private Country country;
	private LocationService locationService;
	private Weather weather;
	private WeatherService weatherService;

	public WeatherAction(WeatherService weatherService,
			LocationService locationService) {
		this.weatherService = weatherService;
		this.locationService = locationService;
	}

	/**
	 * <p>
	 * Retrieve the city list according to the selected country.
	 * </p>
	 * 
	 * @return The "Success" result for this mapping
	 * @throws Exception
	 *             on any error
	 */
	public String city() throws Exception {
		return SUCCESS;
	}

	public String execute() throws Exception {
		try {
			WeatherUserService weatherUserService = (WeatherUserService) getUser()
					.getService(WeatherUserService.class);
			this.weather = weatherService.getWeather(city, weatherUserService
					.getDegreeUnit());
		} catch (BusinessException e) {
			log.warn(Constants.LOG_BUSINESS_ERROR);
			e.printStackTrace();
			addActionError(e);
		}
		return SUCCESS;
	}

	public List<City> getCities() throws BusinessException {
		if (country == null) {
			return new ArrayList<City>();
		} else {
			return locationService.loadCitiesByCountry(country);
		}
	}

	public City getCity() {
		return city;
	}

	public List<Country> getCountries() {
		return countries;
	}

	public Country getCountry() {
		return country;
	}

	public Weather getWeather() {
		return weather;
	}

	public String input() throws Exception {
		WeatherUserService weatherUserService = (WeatherUserService) getUser()
				.getService(WeatherUserService.class);
		this.city = weatherUserService.getDefaultCity();
		this.country = weatherUserService.getDefaultCity().getCountry();
		return execute();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.opensymphony.xwork2.Preparable#prepare()
	 */
	@Override
	public void prepare() throws Exception {
		this.countries = locationService.loadAllCountries();
	}

	public void setCity(City city) {
		this.city = city;
	}

	public void setCountry(Country country) {
		this.country = country;
	}

	public void setWeather(Weather weather) {
		this.weather = weather;
	}

}
